<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Add to crontab (to have it run every 1 minute):
//
// * * * * * /usr/local/bin/php /home/ppcron/cronjobs/cleanup_unused_accounts_cron.php
//

// pwd of a cronjob seems to be the cron user's (ppcron) home directory.
// This messes up includes, so we're going to explicitly use the directory
// of this file.
$current_directory = substr($argv[0], 0, strrpos($argv[0], "/"));

//
// Remove unused user accounts.
// Any user account of configured classes with last_activity older than account_ttl is deleted.
//
$_SERVER["DOCUMENT_ROOT"] = $current_directory;
include($current_directory."/includes/user_class_helpers.php");
include($current_directory."/config.php");
include($current_directory."/includes/sql_error_cli.php");
include($current_directory."/includes/cr.php");
include($current_directory."/includes/database_helpers.php");

// Don't run if this job is already running.

if ($f = @fopen($config["cleanup_unused_accounts_pid_file"], "r"))
{
    $pid = (int)fgets($f);
    fclose($f);
    exit("{$argv[0]} is already running, pid {$pid}\n");
}

if ($f = @fopen($config["cleanup_unused_accounts_pid_file"], "w"))
{
    $pid = getmypid();
    fputs($f, $pid."\n");
    fclose($f);
}
else
{
    echo "Can't create {$config["cleanup_unused_accounts_pid_file"]}, keep running anyway.\n";
}


// Construct SQL condition of user classes configured for account cleanup.
$param_string = "";
$param_array = array();
string_list_to_params(implode(",", $config["unused_account_cleanup_affected_user_classes"]), ",", "p_", $param_string, $param_array);
$query_inactive_user_ids_from_part = " FROM user WHERE
user.last_activity < (unix_timestamp() - :unused_account_ttl)
AND
user.locked = :locked
AND
user.grp IN (".$param_string.")
";

// Cleanup unused non-disabled accounts
$param_array[':locked'] = 0;
$param_array[':unused_account_ttl'] = $config["unused_account_ttl"];
delete_user_and_all_related_data($config, $query_inactive_user_ids_from_part, $param_array);

// Cleanup unused disabled accounts
$param_array[':locked'] = 1;
$param_array[':unused_account_ttl'] = $config["unused_disabled_account_ttl"];
delete_user_and_all_related_data($config, $query_inactive_user_ids_from_part, $param_array);

@unlink($config["cleanup_unused_accounts_pid_file"]);

exit(0);

function delete_user_and_all_related_data($config, $query_inactive_user_ids_from_part, Array $param_array) {
    // Call function from database_helpers.php
    if (!delete_user_and_related_data($config, $query_inactive_user_ids_from_part, $param_array)) {
        @unlink($config["cleanup_unused_accounts_pid_file"]);
        sql_error_cli("DELETE " . $query_inactive_user_ids_from_part, $config["sql"]);
    }
}
?>
