/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**********************************************************************
BASED ON GNU wget cookies.c 
AND ON KDE core krfcdate.cpp IMPLEMENTATION

See: 
http://antinode.info/ftp/wget/wget-1_10_2c_vms/cookies.c
http://developer.kde.org/documentation/library/3.2-api/kdecore/html/krfcdate_8cpp-source.html

FOR ORIGINAL LICENCE INFORMATION
************************************************************************/

#include "mod_psiphon_headers.h"

unsigned int ymdhms_to_seconds(int year, int mon, int day, int hour, int minute,
int second)
{
    if(sizeof(time_t) == 4)
    {
        if((time_t) - 1 < 0)
        {
            if(year >= 2038)
            {
                year = 2038;
                mon = 0;
                day = 1;
                hour = 0;
                minute = 0;
                second = 0;
            }
        }
        else
        {
            if(year >= 2115)
            {
                year = 2115;
                mon = 0;
                day = 1;
                hour = 0;
                minute = 0;
                second = 0;
            }
        }
    }

                                 /* days */
    unsigned int ret = (day - 32075)
        +1461L * (year + 4800L + (mon - 14) / 12) / 4 + 367 * (mon - 2 - (mon - 14) / 12 * 12) / 12 - 3 * ((year + 4900L + (mon - 14) / 12) / 100) / 4 - 2440588;

    ret = 24 * ret + hour;       /* hours   */
    ret = 60 * ret + minute;     /* minutes */
    ret = 60 * ret + second;     /* seconds */

    return ret;
}


typedef struct
{
    const char* tzName;
    int tzOffset;
} Zone ;

static const Zone known_zones[] =
{
    { "UT", 0 },
    { "GMT", 0 },
    { "EST", -300 },
    { "EDT", -240 },
    { "CST", -360 },
    { "CDT", -300 },
    { "MST", -420 },
    { "MDT", -360 },
    { "PST", -480 },
    { "PDT", -420 },
    { 0, 0 }
};

time_t cookie_get_date(const char *dateString)
{

    //     Wednesday, 09-Nov-99 23:12:40 GMT
    //     Sat, 01-Jan-2000 08:00:00 GMT
    //     Sat, 01 Jan 2000 08:00:00 GMT
    //     01 Jan 99 22:00 +0100

    static const char haystack[37] = "janfebmaraprmayjunjulaugsepoctnovdec";
    time_t result = 1;
    int offset = 0;
    char* newPosStr;
    int day = 0;
    char monthStr[4];
    int month = -1;
    int year = 0;
    int hour = 0;
    int minute = 0;
    int second = 0;
    int i;

    psiphon_headers_log("cookie_get_date(%s)", dateString);

    // Strip leading space
    while(*dateString && isspace(*dateString))
        dateString++;

    // Strip weekday
    while(*dateString && !isdigit(*dateString) && !isspace(*dateString))
        dateString++;

    // Strip trailing space
    while(*dateString && isspace(*dateString))
        dateString++;

    if(!*dateString)
        return result;           // Invalid date

    if(isalpha(*dateString))
    {

        // ' Nov 5 1994 18:15:30 GMT'
        // Strip leading space
        while (*dateString && isspace(*dateString))
            dateString++;

        for(i = 0;i < 3;i++)
        {
            if(!*dateString || (*dateString == '-') || isspace(*dateString))
            {
                return result;   // Invalid date
            }

            monthStr[i] = tolower(*dateString++);
        }
        monthStr[3] = '\0';

        newPosStr = (char*)strstr(haystack, monthStr);

        if(!newPosStr)
            return result;       // Invalid date

                                 // Jan=00, Feb=01....
        month = (newPosStr - haystack) / 3;

        if((month < 0) || (month > 11))
            return result;       // Invalid date

        while(*dateString && isalpha(*dateString))
            dateString++;        // Skip rest of month-name
    }

    // ' 09-Nov-99 23:12:40 GMT'
    // ' 5 1994 18:15:30 GMT'
    day = strtol(dateString, &newPosStr, 10);
    dateString = newPosStr;

    if((day < 1) || (day > 31))
        return result;           // Invalid date;

    if(!*dateString)
        return result;           // Invalid date

    while(*dateString && (isspace(*dateString) || (*dateString == '-')))
        dateString++;

    if(month == -1)
    {
        for(i = 0;i < 3;i++)
        {
            if(!*dateString || (*dateString == '-') || isspace(*dateString))
                return result;   // Invalid date
            monthStr[i] = tolower(*dateString++);
        }
        monthStr[3] = '\0';

        newPosStr = (char*)strstr(haystack, monthStr);

        if(!newPosStr)
            return result;       // Invalid date

                                 // Jan=00, Feb=01, Mar=02, ..
        month = (newPosStr - haystack) / 3;

        if((month < 0) || (month > 11))
            return result;       // Invalid date

        while(*dateString && isalpha(*dateString))
            dateString++;        // Skip rest of month-name
    }

    // '-99 23:12:40 GMT'
    while(*dateString && (isspace(*dateString) || (*dateString == '-')))
        dateString++;

    if(!*dateString || !isdigit(*dateString))
        return result;           // Invalid date

    // '99 23:12:40 GMT'
    year = strtol(dateString, &newPosStr, 10);
    dateString = newPosStr;

    // Y2K: Solve 2 digit years
    if((year >= 0) && (year < 50))
        year += 2000;

    if((year >= 50) && (year < 100))
        year += 1900;            // Y2K

    if((year < 1900) || (year > 2500))
        return result;           // Invalid date

    // Don't fail if the time is missing.
    if(*dateString)
    {
        // ' 23:12:40 GMT'
        if(!isspace(*dateString++))
            return result;       // Invalid date

        hour = strtol(dateString, &newPosStr, 10);
        dateString = newPosStr;

        if((hour < 0) || (hour > 23))
            return result;       // Invalid date

        if(!*dateString)
            return result;       // Invalid date

        // ':12:40 GMT'
        if(*dateString++ != ':')
            return result;       // Invalid date
        minute = strtol(dateString, &newPosStr, 10);
        dateString = newPosStr;

        if((minute < 0) || (minute > 59))
            return result;       // Invalid date

        if(!*dateString)
            return result;       // Invalid date

        // ':40 GMT'
        if(*dateString != ':' && !isspace(*dateString))
            return result;       // Invalid date

        // seconds are optional in rfc822 + rfc2822
        if(*dateString == ':')
        {
            dateString++;

            second = strtol(dateString, &newPosStr, 10);
            dateString = newPosStr;

            if((second < 0) || (second > 59))
                return result;   // Invalid date
        }
        else
        {
            dateString++;
        }

        while(*dateString && isspace(*dateString))
            dateString++;
    }

    // don't fail if the time zone is missing, some
    // broken mail-/news-clients omit the time zone
    if(*dateString)
    {
        if((strncasecmp(dateString, "gmt", 3) == 0) || (strncasecmp(dateString, "utc", 3) == 0))
        {

            dateString += 3;
            while(*dateString && isspace(*dateString))
                dateString++;
        }

        if((*dateString == '+') || (*dateString == '-'))
        {
            offset = strtol(dateString, &newPosStr, 10);
            if(abs(offset) < 30)
            {
                dateString = newPosStr;

                offset = offset * 100;

                if(*dateString && *(dateString + 1))
                {

                    dateString++;
                    int minutes = strtol(dateString, &newPosStr, 10);
                    if(offset > 0)
                        offset += minutes;

                    else
                        offset -= minutes;
                }
            }

            if((offset < -9959) || (offset > 9959))
                return result;   // Invalid date

            int sgn = (offset < 0) ? -1 : 1;
            offset = abs(offset);
            offset = ((offset / 100) * 60 + (offset % 100)) * sgn;
        }
        else
        {
            for(i = 0;known_zones[i].tzName != 0;i++)
            {
                if(0 == strncasecmp(dateString, known_zones[i].tzName, strlen(known_zones[i].tzName)))
                {
                    offset = known_zones[i].tzOffset;
                    break;
                }
            }
        }
    }

    result = ymdhms_to_seconds(year, month + 1, day, hour, minute, second);

    // avoid negative time values
    if((offset > 0) && (offset > result))
        offset = 0;

    result -= offset * 60;

    // If epoch 0 return epoch +1 which is Thu, 01-Jan-70 00:00:01 GMT
    // This is so that parse error and valid epoch 0 return values won't
    // be the same for sensitive applications...
    if(result < 1)
        result = 1;

    return result;

    return(10);
}


int IsNumericAddress(const char* pAddr)
{
    const char* p = pAddr;

    if (!isdigit(*p)) return 0; for (++p; isdigit(*p); p++);
    if (*p++ != '.') return 0;
    if (!isdigit(*p)) return 0; for (++p; isdigit(*p); p++);
    if (*p++ != '.') return 0;
    if (!isdigit(*p)) return 0; for (++p; isdigit(*p); p++);
    if (*p++ != '.') return 0;
    if (!isdigit(*p)) return 0; for (++p; isdigit(*p); p++);

    if (!*p)
        return 1;
    else
        return 0;

}


/*

             int cookie_valid_domain(const char *cookie_domain, const char *request_domain) {
             if (!strcmp(cookie_domain, request_domain)) {
             psiphon_headers_log("cookie_valid_domain(%s, %s): 1", cookie_domain, request_domain);
             return 1;
} else {
             psiphon_headers_log("cookie_valid_domain(%s, %s): 0", cookie_domain, request_domain);
             return 0;
}
}
*/

int cookie_valid_domain(const char* sCookieDomain, const char* Host)
{
    if(sCookieDomain == NULL)
        return 0;

    if(Host == NULL)
        return 0;

    /* Numeric address requires exact match.  It also requires HOST to
               be an IP address.  */
    if(IsNumericAddress(sCookieDomain))
        return (0 == strcmp(sCookieDomain, Host));

    /* For the sake of efficiency, check for exact match first. */
    if(0 == strcasecmp(sCookieDomain, Host))
        return 1;

    /* HOST must match the tail of cookie_domain. */
    if(!MatchTail(Host, sCookieDomain, 1))
        return 0;

    /* We know that COOKIE_DOMAIN is a subset of HOST; however, we must
               make sure that somebody is not trying to set the cookie for a
               subdomain shared by many entities.  For example, "company.co.uk"
               must not be allowed to set a cookie for ".co.uk".  On the other
               hand, "sso.redhat.de" should be able to set a cookie for
               ".redhat.de".
               The only marginally sane way to handle this I can think of is to
               reject on the basis of the length of the second-level domain name
               (but when the top-level domain is unknown), with the assumption
               that those of three or less characters could be reserved.  For
       example:
               .co.org -> works because the TLD is known
               .co.uk -> doesn't work because "co" is only two chars long
               .com.au -> doesn't work because "com" is only 3 chars long
               .cnn.uk -> doesn't work because "cnn" is also only 3 chars long (ugh)
               .cnn.de -> doesn't work for the same reason (ugh!!)
               .abcd.de -> works because "abcd" is 4 chars long
               .img.cnn.de -> works because it's not trying to set the 2nd level domain
               .cnn.co.uk -> works for the same reason
               That should prevent misuse, while allowing reasonable usage.  If
               someone knows of a better way to handle this, please let me
               know.  */
    {
        const char* p = sCookieDomain;
        int dccount = 1;         /* number of domain components */
        int ldcl = 0;            /* last domain component length */
        int nldcl = 0;           /* next to last domain component length */
        int out = 0;
        if(*p == '.')
            /* Ignore leading period in this calculation. */
            ++p;

        for(out = 0;!out;p++)
            switch(*p)
            {
                case '\0':
                    out = 1;
                    break;
                case '.':
                    if(ldcl == 0)
                    /* Empty domain component found -- the domain is invalid. */
                        return 0;
                    if(*(p + 1) == '\0')
                    {
                    /* Tolerate trailing '.' by not treating the domain as one ending with an empty domain component.  */
                        out = 1;
                        break;
                    }
                    nldcl = ldcl;
                    ldcl = 0;
                    ++dccount;
                    break;
                default:
                    ++ldcl;
            }

        if(dccount < 2)
            return 0;

        if(dccount == 2)
        {
            int i = 0;
            int known_toplevel = 0;
            static const char* known_toplevel_domains[] =
            {
                ".com", ".edu", ".net", ".org", ".gov", ".mil", ".int"
            };
            for(i = 0;i < (sizeof(known_toplevel_domains) / sizeof((known_toplevel_domains)[0]));i++)
                if(MatchTail(sCookieDomain, known_toplevel_domains[i], 1))
            {
                known_toplevel = 1;
                break;
            }
            if(!known_toplevel && nldcl <= 3)
                return 0;
        }
    }

    /* Don't allow the host "foobar.com" to set a cookie for domain
               "bar.com".  */
    if(*sCookieDomain != '.')
    {
        int dlen = strlen(sCookieDomain);
        int hlen = strlen(Host);
        /* cookie host:    hostname.foobar.com */
        /* desired domain:             bar.com */
        /* '.' must be here in host-> ^        */
        if(hlen > dlen && Host[hlen - dlen - 1] != '.')
            return 0;
    }

    return 1;
}


int MatchTail(const char* sString, const char* sTail, int nCaseSensitive)
{
    int i = 0;
    int j = 0;
    if(!nCaseSensitive)
    {
        for(i = strlen(sString) , j = strlen(sTail);i >= 0 && j >= 0;i--, j--)
        {
            if(sString[i] != sTail[j])
                break;
        }
    }
    else
    {
        for(i = strlen(sString) , j = strlen(sTail);i >= 0 && j >= 0;i--, j--)
        {
            if(tolower(sString[i]) != tolower(sTail[j]))
                break;
        }
    }

    if(j == -1 || (j == 0 && sTail[0]=='.' && i == -1))
        return 1;
    return 0;
}
