/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/********************************************************************
  Copyright (c) 2005-6, WebThing Ltd
  Author: Nick Kew <nick@webthing.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*********************************************************************/

#include <ctype.h>
#include <syslog.h>
#include <http_protocol.h>
#include <http_config.h>
#include <apr_optional.h>
#include <apr_strings.h>

#include "mod_proxy_flash.h"

#define DEBUG
#undef DEBUG

#ifdef DEBUG
#define proxy_flash_log(args...) syslog(LOG_DEBUG, args)
#else
#define proxy_flash_log(args...)  ""
#endif


typedef struct
{
    int proxy_flash;
    const char* loc;
} proxy_flash_dir_conf ;

typedef struct
{
    const char* module_location;
} proxy_flash_srv_conf;

module AP_MODULE_DECLARE_DATA proxy_flash_module ;

const char* psiphon_get_flash_prefix(request_rec *r)
{
    proxy_flash_srv_conf *sconf;
    sconf = (proxy_flash_srv_conf *)
        ap_get_module_config(r->server->module_config, &proxy_flash_module);
    return sconf->module_location;
}

static void proxy_flash_hooks(apr_pool_t* pool)
{
    APR_REGISTER_OPTIONAL_FN(psiphon_get_flash_prefix);
}

static void* proxy_flash_dir_config(apr_pool_t* pool, char* dir)
{
    proxy_flash_dir_conf* ret = apr_palloc(pool, sizeof(proxy_flash_dir_conf)) ;
    ret->loc = dir;
    return ret ;
}

static void* proxy_flash_srv_config(apr_pool_t * p, server_rec * x)
{
    proxy_flash_srv_conf *cfg = apr_pcalloc(p, sizeof(proxy_flash_srv_conf));
    cfg->module_location = NULL;
    return cfg;
}

static int register_module_location(apr_pool_t* pool, server_rec* s,
const char* location)
{
    proxy_flash_srv_conf *conf =
        ap_get_module_config(s->module_config, &proxy_flash_module);
    if(conf->module_location == NULL)
    {
        conf->module_location = apr_pstrdup(pool, location);
        return 0;
    }
    return 1;
}

static const char* set_proxy_flash(cmd_parms *parms, void *in_dconf, int arg)
{
    proxy_flash_dir_conf *dconf = in_dconf;
    dconf->proxy_flash = arg;
    if(!dconf->loc)
    {
        return "proxy-flash must be set in per-directory context only";
    }
    if(register_module_location(parms->pool, parms->server, dconf->loc) !=0 )
    {
        return "proxy-flash must be set only once per server, no multiple locations allowed";
    }
    return NULL;
}

static const command_rec proxy_flash_cmds[] =
{
    AP_INIT_FLAG("ProxyFlash", set_proxy_flash, NULL, ACCESS_CONF, 
            "on/off flag"),
    {NULL}
} ;

module AP_MODULE_DECLARE_DATA proxy_flash_module =
{
    STANDARD20_MODULE_STUFF,
    proxy_flash_dir_config,
    NULL,
    proxy_flash_srv_config,
    NULL,
    proxy_flash_cmds,
    proxy_flash_hooks
};
