/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/********************************************************************
apxs -ic -I /usr/local/include/libxml2 -I utils/ mod_proxy_html.c
    Copyright (c) 2003-7, WebThing Ltd
    Author: Nick Kew <nick@webthing.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License  Version 2,
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You can obtain a copy of the GNU General Poblic License Version 2
from http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or
http://apache.webthing.com/COPYING.txt

*********************************************************************/

/********************************************************************
   Note to Users

   You are requested to register as a user, at
   http://apache.webthing.com/registration.html

   This entitles you to support from the developer.
   I'm unlikely to reply to help/support requests from
   non-registered users, unless you're paying and/or offering
   constructive feedback such as bug reports or sensible
   suggestions for further development.

   It also makes a small contribution to the effort
   that's gone into developing this work.
*********************************************************************/

/* End of Notices */
#ifndef MIN
#define MIN(x,y) ((x) <= (y) ? (x) : (y))
#endif

#include <ctype.h>


/* apache */
#include <http_protocol.h>
#include <http_config.h>
#include <http_log.h>
#include <apr_strings.h>
#include <apr_hash.h>
#include <apr_uri.h>
#include <apr_strmatch.h>
#include <apr_optional.h>

/*psiphon */
#include "mod_proxy_html.h"
#include "psiphon_url.h"
#include "css_scanner.h"
#include "stringbuf.h"
#include "linetok.h"
#include "jsf.h"
#include "psiHTMLparser.h"

#define DEBUG
#undef DEBUG

#ifdef DEBUG
#define proxy_html_log(FMT, ARG...) syslog(LOG_DEBUG, "[%s] %s #%d: " FMT, __FILE__, __FUNCTION__, __LINE__, ## ARG)
#else
#define proxy_html_log(args...)  ""
#endif

#define JS_VOID "javascript:void(0);"
#define UTF8_ENC "UTF8"
#define DEFAULT_ETAG " />"

module AP_MODULE_DECLARE_DATA proxy_html_module ;

typedef struct linkprefix
{
    const char* htmlprefix;
    const char* frmprefix;
    const char* jsprefix;
    const char* cssprefix;
} linkprefix ;

typedef struct
{
    const char* val;
} tattr;

typedef struct
{
    const char* etag;
    size_t bufsz;
    apr_hash_t* links;
    apr_array_header_t* events;
    apr_array_header_t* skipelements;
    int proxy_html;
    const char* loc;
    const char* framebluebar;
} proxy_html_dir_conf ;

typedef struct
{
    const char* module_location;
} proxy_html_srv_conf;

typedef struct
{
    stringbuf* object;
    stringbuf* embed;
    stringbuf* param;
    int param_ndx;
} flash;

typedef struct
{
    ap_filter_t* f ;
    proxy_html_dir_conf* cfg ;
    linkprefix* prefix;
    stringbuf* html;
    stringbuf* css;
    stringbuf* jscript;
    flash* flash;
    int flash_ndx;
    htmlParserCtxtPtr parser ;
    apr_bucket_brigade* bb ;
    regs_v_t** js_regrules;
    const char* encoding;
    const char* baseurl;
    int frame_fix;
} saxctxt ;

static htmlSAXHandler sax ;

typedef enum { ATTR_IGNORE, ATTR_URI, ATTR_EVENT, ATTR_URI_EVENT, ATTR_URI_MAILTO, ATTR_STYLE }
rewrite_t ;


const char* psiphon_get_html_prefix(request_rec *r)
{
    proxy_html_srv_conf *sconf;
    sconf = (proxy_html_srv_conf *)
        ap_get_module_config(r->server->module_config, &proxy_html_module);
    return sconf->module_location;
}

static void consume_buffer(saxctxt* ctx, const char* inbuf,
int bytes, int flag)
{
    int len = MIN(32, bytes);
    int rc;

    proxy_html_log("inbuf start: %s",
        apr_pstrndup(ctx->f->r->pool, inbuf, len));
    proxy_html_log("inbuf tail: %s",
        apr_pstrndup(ctx->f->r->pool, inbuf+bytes-len, len));

    rc = psihtmlParseChunk(ctx->parser, inbuf, bytes, flag) ;

    proxy_html_log("return=%d", rc);
}

static void saxcharacters(void* ctxt, const xmlChar *uchars, int length)
{
    const char* chars = (const char*) uchars;
    saxctxt* ctx = (saxctxt*) ctxt ;
    if(chars && length)
    {
        ap_fwrite(ctx->f->next, ctx->bb, chars, length);
    }
}
#define FLUSH ap_fwrite(ctx->f->next, ctx->bb, (chars+begin), (i-begin)) ; begin = i+1

static void htmlspecialchars(void* ctxt, const char* chars, int length)
{
    saxctxt* ctx = (saxctxt*) ctxt ;
    int i ;
    int begin ;

    proxy_html_log("chars: %s", chars);

    for ( begin=i=0; i<length; i++ )
    {
        switch (chars[i])
        {
            case '&' : FLUSH ; ap_fputs(ctx->f->next, ctx->bb, "&amp;") ; break ;
            case '<' : FLUSH ; ap_fputs(ctx->f->next, ctx->bb, "&lt;") ; break ;
            case '>' : FLUSH ; ap_fputs(ctx->f->next, ctx->bb, "&gt;") ; break ;
            case '"' : FLUSH ; ap_fputs(ctx->f->next, ctx->bb, "&quot;") ; break ;
            default : break ;
        }
    }
    FLUSH ;
}

char* escape_quotes(const char *str)
{
    char* buf;
    if (str == NULL)
    {
        buf = (char*) malloc(sizeof(char));
        *buf = '\0';
        return buf;
    }
    int n = strlen(str);
    buf = (char*) malloc(n * 2 * sizeof(char) + 1);
    const char* s;
    char* b = buf;
    for (s = str; *s; s++)
    {
        if (*s == '\\')
        {
            *b++ = '\\';
            *b++ = '\\';
        }
        else if (*s == '"')
        {
            *b++ = '\\';
            *b++ = '"';
        }
        else
            *b++ = *s;
    }

    *b = '\0';
    return buf;
}

static void json_serialize_pair(apr_pool_t* pool, stringbuf* strbuf, const char* name, const char* value, int is_first)
{
    char* escaped_str = escape_quotes(value);
    if (!is_first)
        stringbuf_append(pool, strbuf, ", ", 2);
    stringbuf_append(pool, strbuf, name, strlen(name));
    stringbuf_append(pool, strbuf, ": \"", 3);
    stringbuf_append(pool, strbuf, escaped_str, strlen(escaped_str));
    stringbuf_append(pool, strbuf, "\"", 1);
    free(escaped_str);
}

static void json_serialize_attributes_array(apr_pool_t* pool, stringbuf* strbuf, const char* root, const char** attrs)
{
    const char** a;
    char* escaped_str;
    
    stringbuf_append(pool, strbuf, root, strlen(root));
    stringbuf_append(pool, strbuf, ": {", 3);
    
    if (attrs)
    {
        for (a = attrs; *a; a += 2)
        {
            if (a != attrs) stringbuf_append(pool, strbuf, ", ", 2);
            escaped_str = escape_quotes(a[1]);

            stringbuf_append(pool, strbuf, a[0], strlen(a[0]));
            stringbuf_append(pool, strbuf, ": \"", 3);
            stringbuf_append(pool, strbuf, escaped_str, strlen(escaped_str));
            stringbuf_append(pool, strbuf, "\"", 1);

            free(escaped_str);
        }
    }
    stringbuf_append(pool, strbuf, "}", 1);
}

static void add_flash(saxctxt* ctx, stringbuf* strbuf, const char* name)
{

    ap_fprintf(ctx->f->next, ctx->bb,
            "\n<div id=\"psi_flash%d\"></div>\n", ctx->flash_ndx);
    ap_fprintf(ctx->f->next, ctx->bb,
            "<script type=\"text/javascript\">\n//<![CDATA[\nregisterFlash('%s', 'psi_flash%d', {\n\t", name, ctx->flash_ndx);

    stringbuf2str(ctx->f->r->pool, strbuf) ;
    const char* res_str = jsf(ctx->f->r, ctx->js_regrules, strbuf->buf, strlen(strbuf->buf)) ;
    if(res_str)
    {
        ap_fwrite(ctx->f->next, ctx->bb, res_str, strlen(res_str));
    }
    ap_fputstrs(ctx->f->next, ctx->bb,
            "\n});\n//]]>\n</script>", NULL);
}

char* psi_proxify_meta_refresh(apr_pool_t* p, char* s, char* base_url, char* prefix)
{
    int len = 0;
    int pos = 0;
    int quoted=0;

    char* url_b = 0;
    char* url_e = 0;
    char* meta_content = 0;
    char* meta_url = 0;
    char* proxified_url = 0;

    if(!base_url)
        return NULL;

    if(!s || !*s)
        return NULL;
    len = strlen(s);

    while (pos != len && s[pos] != ',' && s[pos] != ';')
        ++pos;

    if(pos == len)               //no url
        return NULL;
    ++pos;

    while (pos != len && (s[pos] == ' ' || s[pos] == '\t'
        || s[pos] == '\n' || s[pos] == '\r'))
        ++pos;

    if(pos == len)               //no url
        return NULL;

    if(strncasecmp(&s[pos], "url", 3))
        return NULL;

    pos+=3;

    while (pos != len && (s[pos] == ' ' || s[pos] == '\t'
        || s[pos] == '\n' || s[pos] == '\r'))
        ++pos;
    if(pos == len)               //no url
        return NULL;

    if(s[pos] != '=')
        return NULL;
    ++pos;

    while (pos != len && (s[pos] == ' ' || s[pos] == '\t'
        || s[pos] == '\n' || s[pos] == '\r'))
        ++pos;

    if(pos == len)               //no url
        return NULL;

    url_b = &s[pos];

    if(s[pos] == '\'')
    {
        quoted = 1;
        pos++;
        url_b = &s[pos];
    }
    else if(s[pos] == '"')
    {
        quoted = 2;
        pos++;
        url_b = &s[pos];
    }

    if(quoted == 1)
    {
        if(quoted == 1)
            url_e = strchr(&s[pos], '\'');
        if(quoted == 2)
            url_e = strchr(&s[pos], '"');
        if(!url_e)
        {
            url_b = &s[pos-1];
            url_e = url_b + strcspn(&s[pos], "\t\n\r ") +1;
            if(!url_e)
                url_e = &s[len];
        }
    }

    meta_url =  apr_pstrndup(p, url_b, url_e-url_b);

    proxified_url = psi_proxify_url(p, meta_url, base_url);
    proxified_url = apr_pstrcat(p, prefix, "/", proxified_url, NULL);

    meta_content =  apr_pstrndup(p, s, url_b-s);
    meta_content = apr_pstrcat(p,meta_content, proxified_url, NULL);
    meta_content = apr_pstrcat(p,meta_content, url_e, NULL);

    return  meta_content;
}


static void saxcomment(void* ctxt, const xmlChar *uchars)
{
    const char* chars = (const char*) uchars;
    saxctxt* ctx = (saxctxt*) ctxt ;
    const char* name = (const char*) ctx->parser->name;
    int le_found = 0;
    int length;
    const char* res_str = NULL;

    if(!chars)
        return;

    proxy_html_log("uchars: %s", apr_pstrndup(ctx->f->r->pool, uchars, MIN(32, length)));

    stringbuf_append(ctx->f->r->pool, ctx->html, "<!--", 4) ;

    length = strlen(chars);

    if(name && !strcasecmp(name, "style"))
    {
        chars = psi_proxify_css(
                    ctx->f->r->pool,
                    chars,
                    strlen(chars),
                    ctx->baseurl,
                    ctx->prefix->htmlprefix,
                    ctx->prefix->cssprefix);
        proxy_html_log("trying to stringbuf_append....");
        stringbuf_append(ctx->f->r->pool, ctx->html, chars, strlen(chars));
        proxy_html_log("tstringbuf_append successful");
    }
    stringbuf_append(ctx->f->r->pool, ctx->html, "-->", 3) ;
}


static void saxcdata(void* ctxt, const xmlChar *uchars, int length)
{
    saxctxt* ctx = (saxctxt*) ctxt ;
    const char* chars = (const char*) uchars;
    const char* name = (const char*)ctx->parser->name ;

    if(length == 0)
        return;

    proxy_html_log("uchars: %s", apr_pstrndup(ctx->f->r->pool, uchars, MIN(32, length)));

    if(!name || !*name)
    {
        return;
    }

    if(ctx->js_regrules &&*(ctx->js_regrules) && !strcasecmp(name, "script"))
    {
        stringbuf_append(ctx->f->r->pool, ctx->jscript, chars, length);
    }
    else if(!strcasecmp(name, "style"))
    {
        stringbuf_append(ctx->f->r->pool, ctx->css, chars, length);
    }
}


static void saxendElement(void* ctxt, const xmlChar* uname)
{
    int i = 0;
    saxctxt* ctx = (saxctxt*) ctxt ;
    const char* name = (const char*) uname;
    if(!name || !*name)
    {
        return;
    }
    const htmlElemDesc* desc = htmlTagLookup(uname);
    char* chars = apr_pstrdup(ctx->f->r->pool, "");
    const char* le = NULL;
    const char* res_str = NULL;

    proxy_html_log("name: %s", uname);

    if(name && (!strcasecmp(name, "body") || !strcasecmp(name, "html")))
    {
        return;
    }

    if(ctx->cfg->skipelements != NULL)
    {
        for (i=0; i < ctx->cfg->skipelements->nelts; ++i)
        {
            tattr* attrs = (tattr*) ctx->cfg->skipelements->elts;
            if ( !strcmp(name, attrs[i].val))
            {
                ctx->html->offset = 0 ;
                ctx->jscript->offset = 0 ;
                ctx->css->offset = 0 ;
                return ;
            }
        }
    }

    int is_embed = !strcasecmp(name, "embed");
    int is_object = !strcasecmp(name, "object");

    if (((is_embed && !ctx->flash->object->offset) ||
        (is_object && ctx->flash->object->offset)
         ) && (ctx->flash_ndx == 0)
            )
    {
        ap_fputstrs(ctx->f->next, ctx->bb, "\n<script type=\"text/javascript\" src=\"/js/swfobject.js\"></script>", NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, "\n<script type=\"text/javascript\" src=\"/js/xmlhttp.js\"></script>", NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, "\n<script type=\"text/javascript\" src=\"/js/registerFlash.js\"></script>", NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, "\n<script type=\"text/javascript\" src=\"/js/json.js\"></script>", NULL);
    }

    if (is_embed)
    {
        if (ctx->flash->object->offset)
            return;

        stringbuf* jscode = apr_pcalloc(ctx->f->r->pool, sizeof(stringbuf));
        stringbuf_append(ctx->f->r->pool, jscode, "renderFlash: false", 18);
        
        stringbuf2str(ctx->f->r->pool, ctx->flash->embed);
        int len = strlen(ctx->flash->embed->buf);
        if (len > 0) {
            stringbuf_append(ctx->f->r->pool, jscode, ", ", 2);
            stringbuf_append(ctx->f->r->pool, jscode, ctx->flash->embed->buf, len);
        }
        
        add_flash(ctx, jscode, name);
        ctx->flash->embed->offset = 0;
        ctx->flash_ndx++;
        return;
    }
    
    if (is_object && ctx->flash->object->offset)
    {
        stringbuf* jscode = apr_pcalloc(ctx->f->r->pool, sizeof(stringbuf));
        jscode->buf = NULL;
        jscode->avail = 0;
        jscode->offset = 0;

        stringbuf_append(ctx->f->r->pool, jscode, "renderFlash: false", 18);
        
        stringbuf2str(ctx->f->r->pool, ctx->flash->object);
        int len = strlen(ctx->flash->object->buf);
        if (len > 0)
        {
            stringbuf_append(ctx->f->r->pool, jscode, ", ", 2);
            stringbuf_append(ctx->f->r->pool, jscode, ctx->flash->object->buf, len);
        }

        if (ctx->flash->param->offset)
        {
            stringbuf_append(ctx->f->r->pool, jscode, ",\n\t", 3);
            stringbuf_append(ctx->f->r->pool, jscode, "params: {", 9);
            stringbuf2str(ctx->f->r->pool, ctx->flash->param);
            stringbuf_append(ctx->f->r->pool, jscode, ctx->flash->param->buf, strlen(ctx->flash->param->buf));
            stringbuf_append(ctx->f->r->pool, jscode, "}", 1);
        }

        if (ctx->flash->embed->offset)
        {
            stringbuf_append(ctx->f->r->pool, jscode, ",\n\t", 3);
            stringbuf2str(ctx->f->r->pool, ctx->flash->embed);
            stringbuf_append(ctx->f->r->pool, jscode, ctx->flash->embed->buf, strlen(ctx->flash->embed->buf));
        }

        add_flash(ctx, jscode, name);
        
        ctx->flash->object->offset = 0;
        ctx->flash->param->offset = 0;
        ctx->flash->embed->offset = 0;
        ctx->flash->param_ndx = 0;
        ctx->flash_ndx++;
        return;
    }
    else if (!strcasecmp(name, "style") && ctx->css->offset )
    {
        chars = psi_proxify_css(
                    ctx->f->r->pool,
                    ctx->css->buf,
                    ctx->css->offset,
                    ctx->baseurl,
                    ctx->prefix->htmlprefix,
                    ctx->prefix->cssprefix);
        ap_fwrite(ctx->f->next, ctx->bb, chars, strlen(chars));
        ctx->css->offset = 0;
    }
    else if (!strcasecmp(name, "script") && ctx->jscript->offset)
    {
                                 /* append null */
        stringbuf2str(ctx->f->r->pool, ctx->jscript) ;
        chars = ctx->jscript->buf;
        while(chars && (le = get_le(chars, strlen(chars))) )
        {
            if(le == chars)
            {
                chars++;
                continue;
            }
            res_str = jsf(ctx->f->r, ctx->js_regrules, chars, le - chars);
            if(res_str)
            {
                ap_fwrite(ctx->f->next, ctx->bb, res_str, strlen(res_str));
            }

            chars = (char*)le;
        }
        if(chars)
        {
            res_str = jsf(ctx->f->r, ctx->js_regrules, chars, strlen(chars)) ;
            if(res_str)
            {
                ap_fwrite(ctx->f->next, ctx->bb, res_str, strlen(res_str));
            }
        }

        ctx->jscript->offset = 0;
    }
    if ( ctx->html->offset > 0 )
    {
        ap_fwrite(ctx->f->next, ctx->bb, ctx->html->buf, ctx->html->offset);
        ctx->html->offset = 0 ;
    }
    if ( !desc || ! desc->empty )
    {
        ap_fprintf(ctx->f->next, ctx->bb, "</%s>", name) ;
    }
}

static void saxstartElement(void* ctxt, const xmlChar* uname,
const xmlChar** uattrs )
{

    int linkfrm = 0;
    int linkcss = 0;
    int linkjs = 0;
    int num_match ;
    rewrite_t is_uri ;
    const char** a;
    const char** aa;
    size_t s_to, s_from ;
    saxctxt* ctx = (saxctxt*) ctxt ;
    apr_array_header_t *linkattrs;
    apr_pool_t* pool = ctx->f->r->pool;
    int i;
    const char* prefix = 0;
    const char* proxified_content = 0;
    int method_found = 0;
    int action_found = 0;
    
    const char* name = (const char*) uname;
    if(!name || !*name)
        return;
    const htmlElemDesc* desc = htmlTagLookup(uname);
    const char** attrs = NULL;

    proxy_html_log("name: %s", uname);

    attrs = (const char**)uattrs;

    if(ctx->cfg->skipelements != NULL)
    {
        for (i=0; i < ctx->cfg->skipelements->nelts; ++i)
        {
            tattr* attrs = (tattr*) ctx->cfg->skipelements->elts;
            if ( !strcmp(name, attrs[i].val))
            {
                return ;
            }
        }
    }
    
    if (ctx->flash->object->offset)
    {
        if (!strcasecmp(name, "embed"))
        {
            json_serialize_attributes_array(pool, ctx->flash->embed, "embed", attrs);
            
            return;
        }
        else if (!strcasecmp(name, "param") && ctx->flash->object->offset)
        {
             int is_first = ctx->flash->param_ndx == 0;
             
             const char *param_name = NULL, *param_value = NULL;

             if (attrs)
             {
                 for (a = attrs; *a; a += 2)
                 {
                     if (!strcasecmp(a[0], "name"))
                         param_name = a[1];
                     else if (!strcasecmp(a[0], "value"))
                         param_value = a[1];
                 }
             }
             if (param_name && param_value)
             {
                 json_serialize_pair(pool, ctx->flash->param, param_name, param_value, is_first);
                 
                 ctx->flash->param_ndx++;
             }
        }
        return;
    }
    else if (!strcasecmp(name, "embed"))
    {
        json_serialize_attributes_array(pool, ctx->flash->embed, "attributes", attrs);
        return;
    }
    else if (!strcasecmp(name, "object"))
    {
        json_serialize_attributes_array(pool, ctx->flash->object, "attributes", attrs);
        return;
    }
    
    ap_fputc(ctx->f->next, ctx->bb, '<') ;

    if(!strcasecmp(name, "style"))
    {
        ctx->css->offset = 0;
    }
    if(!strcasecmp(name, "script"))
    {
        ctx->jscript->offset = 0;
    }
    /*meta refresh fix*/
    if(!strcasecmp(name, "meta") && attrs)
    {
        for(a = attrs ; *a ; a+=2)
        {
            if( a[0] && a[1] && !strcasecmp(a[0], "http-equiv") && !strcasecmp(a[1], "refresh"))
            {
                for(aa = attrs ; *aa ; aa+=2)
                {
                    if( aa[0] && aa[1] &&!strcasecmp(aa[0], "content"))
                    {

                        proxified_content = psi_proxify_meta_refresh(pool,
                            (char*)aa[1], (char*)ctx->baseurl,
                            (char*)ctx->prefix->htmlprefix);
                        if(!proxified_content)
                            continue;

                        xmlFree((xmlChar*) aa[1]);
                        aa[1] = xmlCharStrdup(proxified_content);
                    }
                }
            }
        }

    }
    /*end of meta refresh fix*/

    /*extract baseurl*/
    if(!strcasecmp(name, "base") && attrs)
    {
        for ( a = attrs ; *a ; a += 2 )
        {
            if ( a[0] && a[1])
            {
                if (!strcasecmp(a[0], "href") && *a[1])
                {
                    ctx->baseurl = apr_pstrdup(pool, a[1]);
                    //unset attribute
                    a[0] = a[1] = NULL;
                }
            }
        }
    }

    if(!strcasecmp(name, "form"))
    {
        size_t a_len = 0;
        linkfrm = 1;

        if ( attrs )
        {
            for ( a = attrs ; *a ; a += 2 )
            {
                a_len += 2;
                if ( a[0] && a[1])
                {
                    if (!strcasecmp(a[0], "method"))
                    {
                        if(!strcasecmp(a[1], "post"))
                        {
                            linkfrm = 0;
                            break;
                        }
                        else
                        {
                            xmlFree((xmlChar*)a[1]);
                            a[1] = xmlStrdup("post");
                        }
                        method_found = 1;
                    }
                    if (!strcasecmp(a[0], "action"))
                        action_found = 1;
                }
            }
            a_len++;             //last NULL
        }
    }
    if(!strcasecmp(name, "frameset") && !ctx->frame_fix && ctx->cfg->framebluebar)
    {
        proxified_content = &ctx->f->r->filename[6];
        ap_fputstrs(ctx->f->next, ctx->bb, "frameset rows=\"30, *\" >\n", NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, "<frame src=\"", NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, ctx->cfg->framebluebar, NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, proxified_content, NULL);
        ap_fputstrs(ctx->f->next, ctx->bb, "\" scrolling=\"no\" frameborder=\"0\">\n", NULL);
        ap_fputc(ctx->f->next, ctx->bb, '<') ;
        ctx->frame_fix = 1;
    }

    /*find out if link is of js type*/
    if(!strcasecmp(name, "script"))
    {
        linkjs = 1;
    }
    /*find out if link is of css type*/
    if(!strcasecmp(name, "link"))
    {
        if ( attrs )
        {
            for ( a = attrs ; *a ; a += 2 )
            {
                if ( a[0] && a[1])
                {
                    if (!strcasecmp(a[0], "type") && !strcasecmp(a[1], "text/css"))
                    {
                        linkcss = 1;
                    }
                    /*
                       type is not neccessarily present as we learned
                       http://www.w3.org/TR/REC-html40/present/styles.html#h-14.3.2
                    */
                    if (!strcasecmp(a[0], "rel") && 
                            (!strcasecmp(a[1], "stylesheet") || 
                             !strcasecmp(a[1], "alternate stylesheet")))
                    {
                        linkcss = 1;
                    }
                }
            }
        }

    }

    ap_fputs(ctx->f->next, ctx->bb, name) ;

    if ( attrs )
    {
        linkattrs = apr_hash_get(ctx->cfg->links, name, APR_HASH_KEY_STRING) ;
        for ( a = attrs ; *a ; a += 2 )
        {
            ctx->html->offset = 0 ;
            if (a[0] && a[1])
            {
                is_uri = ATTR_IGNORE ;
                if(!strcasecmp(a[0], "style"))
                {
                    is_uri = ATTR_STYLE ;
                }
                else if ( linkattrs )
                {
                    tattr* attrs = (tattr*) linkattrs->elts;
                    for (i=0; i < linkattrs->nelts; ++i)
                    {
                        if ( !strcasecmp(a[0], attrs[i].val))
                        {
                            is_uri = ATTR_URI ;

                            if(!strcasecmp(name, "script") && !strcasecmp(a[0], "src"))
                            {
                                if(!ctx->js_regrules || !*ctx->js_regrules)
                                {
                                    is_uri = ATTR_EVENT ;
                                }
                            }

                            if(!strncasecmp("javascript:", a[1], 11))
                            {
                                stringbuf_append(pool, ctx->html, "javascript:", 11);
                                is_uri = ATTR_URI_EVENT;
                            }
                            else if(!strncasecmp("mailto:", a[1], 7))
                            {
                                is_uri = ATTR_URI_MAILTO;
                            }
                            break ;
                        }
                    }
                }
                if ( (is_uri == ATTR_IGNORE)
                    && (ctx->cfg->events != NULL) )
                {
                    for (i=0; i < ctx->cfg->events->nelts; ++i)
                    {
                        tattr* attrs = (tattr*) ctx->cfg->events->elts;
                        if ( !strcasecmp(*a, attrs[i].val))
                        {
                            is_uri = ATTR_EVENT ;
                            break ;
                        }
                    }
                }
                switch ( is_uri )
                {
                    case ATTR_URI:
                        /* proxify url*/

                        if (!ctx->baseurl)
                        {
                            proxified_content = apr_pstrdup(pool, "#");
                        }
                        else
                        {
                            if(linkfrm)
                            {
                                prefix = ctx->prefix->frmprefix ;
                            }
                            else if(linkjs)
                            {
                                prefix = ctx->prefix->jsprefix ;
                            }
                            else if(linkcss)
                            {
                                prefix = ctx->prefix->cssprefix ;
                            }
                            else
                            {
                                prefix = ctx->prefix->htmlprefix;
                            }

                            proxified_content = psi_proxify_url(
                                                    pool,
                                                    a[1],
                                                    ctx->baseurl);

                            proxified_content = apr_pstrcat(pool,
                                prefix,
                                "/", proxified_content, NULL);
                        }
                        if(!proxified_content)
                        {
                            proxified_content = apr_pstrdup(pool, "#");
                        }

                        s_to = strlen(proxified_content);

                        stringbuf_append(pool, ctx->html, proxified_content, s_to);
                        break ;

                    case ATTR_EVENT:
                        if(ctx->js_regrules && *ctx->js_regrules)
                            proxified_content = jsf(ctx->f->r, ctx->js_regrules,
                                a[1], strlen(a[1]));
                        else
                            proxified_content = "javascript:void(0);";

                        if(!proxified_content || !*proxified_content)
                            proxified_content = "javascript:void(0);";

                        s_to = strlen(proxified_content);

                        stringbuf_append(pool, ctx->html, proxified_content, s_to);
                        break ;

                    case ATTR_URI_EVENT:
                        s_from = strlen(a[1]) - 11;
                        if(ctx->js_regrules && *ctx->js_regrules)
                            proxified_content = jsf(ctx->f->r, ctx->js_regrules,
                                a[1] + 11, s_from);
                        else
                            proxified_content = "void(0);";

                        if(!proxified_content || !*proxified_content)
                            proxified_content = "void(0);";

                        s_to = strlen(proxified_content);

                        stringbuf_append(pool, ctx->html, proxified_content, s_to);

                        break ;

                    case ATTR_STYLE:
                        s_from = strlen(a[1]);
                        proxified_content = psi_proxify_css(
                                                pool,
                                                a[1],
                                                s_from,
                                                ctx->baseurl,
                                                ctx->prefix->htmlprefix,
                                                ctx->prefix->cssprefix);

                        s_to = strlen(proxified_content);

                        stringbuf_append(pool, ctx->html, proxified_content, s_to);

                        break ;

                    default:
                        stringbuf_append(pool, ctx->html, a[1], strlen(a[1]));
                }
            }
            if (a[0])
            {
                ap_fputstrs(ctx->f->next, ctx->bb, " ", a[0], NULL) ;
            }
            if (a[1])
            {
                ap_fputstrs(ctx->f->next, ctx->bb, "=\"", NULL) ;
                if(ctx->html->offset)
                {
                    stringbuf2str(pool, ctx->html);
                    htmlspecialchars(ctx, ctx->html->buf, strlen(ctx->html->buf));
                }
                ap_fputc(ctx->f->next, ctx->bb, '"') ;
            }
        }
    }
    ctx->html->offset = 0 ;
    if(linkfrm)
    {
        linkfrm = 0;
        if(!method_found)
        {
            method_found = 0;
            ap_fputstrs(ctx->f->next, ctx->bb, " method=\"post\"", NULL) ;
        }
        if(!action_found)
        {
            action_found = 0;
            proxified_content = psi_proxify_url(pool, ctx->baseurl, ctx->baseurl);

            proxified_content = apr_pstrcat(pool,
                ctx->prefix->frmprefix,
                "/", proxified_content, NULL);
            ap_fputstrs(ctx->f->next, ctx->bb, " action=\"", proxified_content,"\"", NULL) ;
        }
    }
    if ( desc && desc->empty )
        ap_fputs(ctx->f->next, ctx->bb, ctx->cfg->etag) ;
    else
        ap_fputc(ctx->f->next, ctx->bb, '>') ;

}


static void saxprocessingInstruction(void* ctxt, const xmlChar *target,
const xmlChar* data)
{
    saxctxt* ctx = (saxctxt*) ctxt ;

    if(target && *target)
    {
        ap_fputstrs(ctx->f->next, ctx->bb, "<?", (const char*)target, NULL) ;

        if(data && *data)
        {
            ap_fputstrs(ctx->f->next, ctx->bb, " ", (const char*)data, NULL) ;
        }

        ap_fputs(ctx->f->next, ctx->bb, ">\n") ;
    }
}


static void saxinternalSubset(void* ctxt,  const xmlChar * name,
const xmlChar * ExternalID,
const xmlChar * SystemID)
{
    saxctxt* ctx = (saxctxt*) ctxt ;
    if(name && *name)
    {
        ap_fputstrs(ctx->f->next, ctx->bb, "<!DOCTYPE ", (const char*)name, " PUBLIC",  NULL) ;

        if(ExternalID && *ExternalID)
        {
            ap_fprintf(ctx->f->next, ctx->bb, " \"%s\"", (const char*)ExternalID) ;

            if(SystemID && *SystemID)
            {
                ap_fprintf(ctx->f->next, ctx->bb, " \"%s\"", (const char*)SystemID) ;
            }
        }

        ap_fputs(ctx->f->next, ctx->bb, ">\n") ;
    }
}


static saxctxt* check_filter_init(ap_filter_t* f)
{
    saxctxt* fctx ;
    linkprefix* prfx;
    regs_v_t** jsrrules;
    if (!f->ctx)
    {
        proxy_html_dir_conf* cfg
            = ap_get_module_config(f->r->per_dir_config, &proxy_html_module);

        const char* html_prefix = apr_table_get(f->r->subprocess_env, "psiphon_html_prefix");;
        const char* frm_prefix = apr_table_get(f->r->subprocess_env, "psiphon_frm_prefix");;
        const char* js_prefix = apr_table_get(f->r->subprocess_env, "psiphon_js_prefix");;
        const char* css_prefix = apr_table_get(f->r->subprocess_env, "psiphon_css_prefix");;

        const char* errmsg = NULL ;
        if (!cfg->proxy_html)
        {
            errmsg = "Filter is OFF";
        }
        if ( ! f->r->proxyreq )
        {
            errmsg = "Non-proxy request; not inserting proxy-html filter" ;
        }
        else if ( ! f->r->content_type )
        {
            errmsg = "No content-type; bailing out of proxy-html filter" ;
        } else if ( strncasecmp(f->r->content_type, "text/html", 9) &&
            strncasecmp(f->r->content_type, "application/xhtml+xml", 21) )
        {
            errmsg = "Non-HTML content; not inserting proxy-html filter" ;
        }
        if (!cfg->links)
        {
            errmsg = "No links configured: nothing for proxy-html filter to do";
        }
        if (!html_prefix || !*html_prefix)
        {
            errmsg = "No HTML prefix found";
        }
        if (!frm_prefix || !*frm_prefix)
        {
            errmsg = "No form action prefix found";
        }
        if (!js_prefix || !*js_prefix)
        {
            errmsg = "No Javascript prefix found";
        }
        if (!css_prefix || !*css_prefix)
        {
            errmsg = "No CSS prefix found";
        }
        if (!cfg->links)
        {
            errmsg = "No links configured: nothing for proxy-html filter to do";
        }
        if ( errmsg )
        {
            ap_remove_output_filter(f) ;
            return NULL ;
        }

        fctx = f->ctx = apr_pcalloc(f->r->pool, sizeof(saxctxt)) ;
        prfx = fctx->prefix = apr_pcalloc(f->r->pool, sizeof(linkprefix)) ;
        fctx->html = apr_pcalloc(f->r->pool, sizeof(stringbuf)) ;
        fctx->html->buf = NULL;
        fctx->html->avail = 0;
        fctx->html->offset = 0;
        fctx->css = apr_pcalloc(f->r->pool, sizeof(stringbuf)) ;
        fctx->css->buf = NULL;
        fctx->css->avail = 0;
        fctx->css->offset = 0;
        fctx->jscript = apr_pcalloc(f->r->pool, sizeof(stringbuf)) ;
        fctx->jscript->buf = NULL;
        fctx->jscript->avail = 0;
        fctx->jscript->offset = 0;
        
        fctx->flash = apr_pcalloc(f->r->pool, sizeof(flash));

        fctx->flash->object = apr_pcalloc(f->r->pool, sizeof(stringbuf)) ;
        fctx->flash->object->buf = NULL;
        fctx->flash->object->avail = 0;
        fctx->flash->object->offset = 0;

        fctx->flash->embed = apr_pcalloc(f->r->pool, sizeof(stringbuf)) ;
        fctx->flash->embed->buf = NULL;
        fctx->flash->embed->avail = 0;
        fctx->flash->embed->offset = 0;

        fctx->flash->param = apr_pcalloc(f->r->pool, sizeof(stringbuf)) ;
        fctx->flash->param->buf = NULL;
        fctx->flash->param->avail = 0;
        fctx->flash->param->offset = 0;

        fctx->flash->param_ndx = 0;
        fctx->flash_ndx = 0;

        fctx->f = f ;
        fctx->bb = apr_brigade_create(f->r->pool, f->r->connection->bucket_alloc) ;
        fctx->cfg = cfg;
        apr_table_unset(f->r->headers_out, "Content-Length") ;
        apr_table_unset(f->r->headers_out, "ETag") ;
        apr_table_set(f->r->headers_out, "Cache-Control", "no-cache, must-revalidate") ;
        apr_table_set(f->r->headers_out, "Expires", "Sat, 26 Jul 1990 05:00:00 GMT") ;

        prfx->htmlprefix = html_prefix;
        prfx->frmprefix = frm_prefix;
        prfx->jsprefix = js_prefix;
        prfx->cssprefix = css_prefix;

        fctx->js_regrules = jsf_init(fctx->f->r);

        if (fctx->f->r->proxyreq && fctx->f->r->filename
            && strncmp(fctx->f->r->filename, "proxy:", 6) == 0)
        {
            fctx->baseurl = apr_pstrdup(fctx->f->r->pool, &fctx->f->r->filename[6]);
        }
        else
            fctx->baseurl = NULL;

        fctx->frame_fix = 0;
    }

    return f->ctx ;
}


static int proxy_html_filter(ap_filter_t* f, apr_bucket_brigade* bb)
{

    apr_bucket* b ;
    xmlCharEncoding enc ;
    const char* buf = 0 ;
    apr_size_t bytes = 0 ;
    #ifndef USE_OLD_LIBXML2
    int xmlopts = XML_PARSE_RECOVER | XML_PARSE_NONET |
        XML_PARSE_NOERROR | XML_PARSE_NOWARNING | XML_PARSE_NOBLANKS;
    #endif

    if (f->r->main != NULL)      /*no subrequests*/
    {
        ap_remove_output_filter(f);
        return ap_pass_brigade(f->next, bb);
    }
    saxctxt* ctxt = check_filter_init(f) ;
    if ( ! ctxt )
        return ap_pass_brigade(f->next, bb) ;

    for ( b = APR_BRIGADE_FIRST(bb) ;
        b != APR_BRIGADE_SENTINEL(bb) ;
        b = APR_BUCKET_NEXT(b) )
    {
        if ( APR_BUCKET_IS_METADATA(b) )
        {
            if ( APR_BUCKET_IS_EOS(b) )
            {
                if ( ctxt->parser != NULL )
                {
                    consume_buffer(ctxt, buf, 0, 1);
                    proxy_html_log("terminating the parser");
                }
                APR_BRIGADE_INSERT_TAIL(ctxt->bb,
                    apr_bucket_eos_create(ctxt->bb->bucket_alloc) ) ;
                ap_pass_brigade(ctxt->f->next, ctxt->bb) ;
            }
            else if ( APR_BUCKET_IS_FLUSH(b) )
            {
                if ( ctxt->parser != NULL )
                {
                    ap_fflush(ctxt->f->next, ctxt->bb) ;
                }
            }
        } else if ( apr_bucket_read(b, &buf, &bytes, APR_BLOCK_READ)
            == APR_SUCCESS )
        {
            if ( ctxt->parser == NULL )
            {
                if ( buf[bytes] != 0 )
                {
                    /* make a string for parse routines to play with */
                    char* buf1 = apr_palloc(f->r->pool, bytes+1) ;
                    memcpy(buf1, buf, bytes) ;
                    buf1[bytes] = 0 ;
                    buf = buf1 ;
                }
                ctxt->parser = htmlCreatePushParserCtxt(&sax, ctxt, buf, 4, 0, XML_CHAR_ENCODING_UTF8) ;
                ctxt->parser->input->encoding = xmlStrdup(UTF8_ENC);
                buf += 4;
                bytes -= 4;
                if (ctxt->parser == NULL)
                {
                    apr_status_t rv = ap_pass_brigade(f->next, bb) ;
                    ap_remove_output_filter(f) ;
                    ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, f->r,
                        "htmlCreatePushParserCtxt failed, filter removed.");

                    return rv;
                }
                apr_pool_cleanup_register(f->r->pool, ctxt->parser,
                    (void*)htmlFreeParserCtxt, apr_pool_cleanup_null) ;
                #ifndef USE_OLD_LIBXML2
                if ( xmlopts = xmlCtxtUseOptions(ctxt->parser, xmlopts ), xmlopts )
                    ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, f->r,
                        "Unsupported parser opts %x", xmlopts) ;
                #endif
                consume_buffer(ctxt, buf, bytes, 0) ;
            }
            else
            {
                consume_buffer(ctxt, buf, bytes, 0) ;
            }
        }
        else
        {
            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, f->r, "Error in bucket read") ;
        }
    }
    /*ap_fflush(ctxt->f->next, ctxt->bb) ;	// uncomment for debug */
    apr_brigade_cleanup(bb) ;
    return APR_SUCCESS ;
}


static void* proxy_html_dir_config(apr_pool_t* pool, char* dir)
{
    proxy_html_dir_conf* ret = apr_pcalloc(pool, sizeof(proxy_html_dir_conf) ) ;
    ret->etag = DEFAULT_ETAG ;
    ret->bufsz = 8192 ;
    ret->proxy_html = 0;
    ret->loc = dir;
    return ret ;
}


static void* proxy_html_srv_config(apr_pool_t * p, server_rec * x)
{
    proxy_html_srv_conf *cfg = apr_pcalloc(p, sizeof(proxy_html_srv_conf));
    cfg->module_location = NULL;
    return cfg;
}


static int register_module_location(apr_pool_t* pool, server_rec* s,
const char* location)
{
    proxy_html_srv_conf *conf =
        ap_get_module_config(s->module_config, &proxy_html_module);
    if(conf->module_location == NULL)
    {
        conf->module_location = apr_pstrdup(pool, location);
        return 0;
    }
    return 1;
}


static const char* set_proxy_html(cmd_parms *parms, void *in_dconf, int arg)
{
    proxy_html_dir_conf *dconf = in_dconf;
    dconf->proxy_html = arg;
    if(!dconf->loc)
    {
        return "proxy-html must be set in per-directory context only";
    }
    if(register_module_location(parms->pool, parms->server, dconf->loc) !=0 )
    {
        return "proxy-html must be set only once per server, no multiple locations allowed";
    }
    return NULL;
}


static const char* set_events(cmd_parms* cmd, void* CFG, const char* arg)
{
    tattr* attr;
    proxy_html_dir_conf* cfg = CFG;
    if (cfg->events == NULL)
        cfg->events = apr_array_make(cmd->pool, 20, sizeof(tattr));
    attr = apr_array_push(cfg->events) ;
    attr->val = arg;
    return NULL ;
}


static const char* set_skip_elements(cmd_parms* cmd, void* CFG, const char* arg)
{
    tattr* attr;
    proxy_html_dir_conf* cfg = CFG;
    if (cfg->skipelements == NULL)
        cfg->skipelements = apr_array_make(cmd->pool, 10, sizeof(tattr));
    attr = apr_array_push(cfg->skipelements) ;
    attr->val = arg;
    return NULL ;
}


static const char* set_links(cmd_parms* cmd, void* CFG,
const char* elt, const char* att)
{
    apr_array_header_t* attrs;
    tattr* attr ;
    proxy_html_dir_conf* cfg = CFG;

    if (cfg->links == NULL)
        cfg->links = apr_hash_make(cmd->pool);

    attrs = apr_hash_get(cfg->links, elt, APR_HASH_KEY_STRING) ;
    if (!attrs)
    {
        attrs = apr_array_make(cmd->pool, 2, sizeof(tattr*)) ;
        apr_hash_set(cfg->links, elt, APR_HASH_KEY_STRING, attrs) ;
    }
    attr = apr_array_push(attrs) ;
    attr->val = att ;
    return NULL ;
}


static const command_rec proxy_html_cmds[] =
{
    AP_INIT_FLAG("ProxyHTML", set_proxy_html, NULL,
    ACCESS_CONF, "on/off flag"),
    AP_INIT_ITERATE("ProxyHTMLEvents", set_events, NULL,
    ACCESS_CONF, "Strings to be treated as scripting events"),
    AP_INIT_ITERATE("ProxyHTMLSkipElements", set_skip_elements, NULL,
    ACCESS_CONF, "HTML elements to remove from output"),
    AP_INIT_ITERATE2("ProxyHTMLLinks", set_links, NULL,
    ACCESS_CONF, "Declare HTML Attributes"),
    AP_INIT_TAKE1("ProxyHTMLBufSize", ap_set_int_slot,
    (void*)APR_OFFSETOF(proxy_html_dir_conf, bufsz),
    ACCESS_CONF, "Buffer size" ) ,
    AP_INIT_TAKE1("ProxyHTMLBluebarFrame", ap_set_string_slot,
    (void*)APR_OFFSETOF(proxy_html_dir_conf, framebluebar),
    ACCESS_CONF, "BlueBar to be displayed in frames" ) ,
    { NULL }
} ;
static int mod_proxy_html(apr_pool_t* p, apr_pool_t* p1, apr_pool_t* p2,
server_rec* s)
{

    memset(&sax, 0, sizeof(htmlSAXHandler));
    sax.startElement = saxstartElement ;
    sax.endElement = saxendElement ;
    sax.characters = saxcharacters;
    sax.comment = saxcomment ;
    sax.cdataBlock = saxcdata ;
    sax.processingInstruction = saxprocessingInstruction;
    sax.internalSubset = saxinternalSubset;    return OK ;
}


static void proxy_html_hooks(apr_pool_t* p)
{
    ap_register_output_filter_protocol("proxy-html", proxy_html_filter,
        NULL, AP_FTYPE_RESOURCE,
        AP_FILTER_PROTO_CHANGE|AP_FILTER_PROTO_CHANGE_LENGTH) ;
    ap_hook_post_config(mod_proxy_html, NULL, NULL, APR_HOOK_MIDDLE) ;
    APR_REGISTER_OPTIONAL_FN(psiphon_get_html_prefix);

}


module AP_MODULE_DECLARE_DATA proxy_html_module =
{
    STANDARD20_MODULE_STUFF,
    proxy_html_dir_config,
    NULL,
    proxy_html_srv_config,
    NULL,
    proxy_html_cmds,
    proxy_html_hooks
} ;
