#include "httpd.h"
#include "http_core.h"
#include "http_protocol.h"
#include "http_config.h"
#include "apr_pools.h"
#include "apr_strings.h"
#include "syslog.h"

#define test_log(args...) syslog(LOG_DEBUG, args)

#define test_err_log(args...) syslog(LOG_ERR, args)

module AP_MODULE_DECLARE_DATA test_module;


typedef struct {
    int enable;
} struct_test_dir_config;

static void *test_create_dir_config(apr_pool_t *p, char *dummy)
{
    struct_test_dir_config *c = (struct_test_dir_config *)
                              apr_pcalloc(p, sizeof(struct_test_dir_config));
    c->enable = 0;
    test_log("test_create_dir_config(%s)", dummy);
    return (void *) c;
}

static const char *test_enable(cmd_parms *parms, void *in_dconf, int arg)
{
    struct_test_dir_config *dconf = in_dconf;
    dconf->enable = arg;
    test_log("test_enable(%d)", arg);
    return NULL;
}


static int test_hook(request_rec *r)
{
    struct_test_dir_config *dconf;

    test_log("test_hook()");

    dconf = (struct_test_dir_config *) ap_get_module_config(r->per_dir_config,
            &test_module);

    if (!dconf->enable) {
        test_log("test_hook(): don't need test");
        return DECLINED;
    };
    
    test_log("test_hook(): running");

    ap_set_content_type(r, "text/plain");


    ap_rputs("Some content is here\nand here\n", r);

    return OK;
}


static const command_rec dir_cmds[] = {
    AP_INIT_FLAG("ModTest", test_enable, NULL, OR_ALL, "on/off flag"),
    {NULL}
};



static void test_hooks(apr_pool_t *pool)
{
//    ap_hook_fixups(test_hook, NULL, NULL, APR_HOOK_LAST);
    ap_hook_handler(test_hook, NULL, NULL, APR_HOOK_MIDDLE);
}

module AP_MODULE_DECLARE_DATA test_module= {
    STANDARD20_MODULE_STUFF,
    test_create_dir_config,
    NULL,
    NULL,
    NULL,
    dir_cmds,
    test_hooks
};
