%{
/**/
#include "css_scanner.h"
#include "psiphon_url.h"
#include "stringbuf.h"

#define T_EOF 0
#define T_IN_STYLE 1
#define T_STYLE_URL 2
#define T_IMPORT_URL 3

int importurl;
%}


%option case-insensitive
%option noyywrap
%option never-interactive


%x S_IMPORT
%x S_URL
%x S_ALPHAIMAGELOADER
%x ST_VALUE_ONE_QUOTED
%x ST_VALUE_DOUBLE_QUOTED
%x ST_VALUE_NOT_QUOTED
%x ST_EOF

I               i|\\0{0,4}(49|69)(\r\n|[ \t\r\n\f])?|\\i
M               m|\\0{0,4}(4d|6d)(\r\n|[ \t\r\n\f])?|\\m
O               o|\\0{0,4}(51|71)(\r\n|[ \t\r\n\f])?|\\o
P               p|\\0{0,4}(50|70)(\r\n|[ \t\r\n\f])?|\\p
R               r|\\0{0,4}(52|72)(\r\n|[ \t\r\n\f])?|\\r
T               t|\\0{0,4}(54|74)(\r\n|[ \t\r\n\f])?|\\t

IMPORT (@{I}{M}{P}{O}{R}{T})
URL "url"
WHITESPACE [\n\r\ \t\b\012]
QUOT '
DOUBLEQUOT \"
ONEQUOTEDSTRING [^']*
DOUBLEQUOTEDSTRING [^\"]*
NOTQUOTEDSTRING [^\n\r\ \t\b\012)]*
ALPHAIMAGELOADER "AlphaImageLoader"{WHITESPACE}*"("{WHITESPACE}*"src"{WHITESPACE}*"="


%%
<INITIAL><ST_EOF> {return T_EOF;}

<<EOF>> {return T_EOF;}

<INITIAL>{WHITESPACE} {BEGIN(INITIAL);  return T_IN_STYLE;}

<INITIAL>{URL} {importurl=0; BEGIN (S_URL);  return T_IN_STYLE;}
<INITIAL>{IMPORT} {importurl=1; BEGIN (S_IMPORT);  return T_IN_STYLE;}
<INITIAL>{ALPHAIMAGELOADER} {importurl=0; BEGIN (S_ALPHAIMAGELOADER);  return T_IN_STYLE;}

<S_IMPORT>{WHITESPACE}*{URL} {BEGIN (S_URL);  return T_IN_STYLE;}


<S_IMPORT>{WHITESPACE}*"("*{QUOT}{WHITESPACE}* {BEGIN (ST_VALUE_ONE_QUOTED);  return T_IN_STYLE;}
<S_IMPORT>{WHITESPACE}*"("*{DOUBLEQUOT}{WHITESPACE}* {BEGIN (ST_VALUE_DOUBLE_QUOTED);  return T_IN_STYLE;}
<S_IMPORT>{WHITESPACE}*"("{WHITESPACE}* {BEGIN (ST_VALUE_NOT_QUOTED);  return T_IN_STYLE;}
<S_IMPORT>.|\n {BEGIN(INITIAL);  return T_IN_STYLE;}


<S_URL>{WHITESPACE}*"("*{WHITESPACE}*{QUOT}{WHITESPACE}* {BEGIN (ST_VALUE_ONE_QUOTED); return T_IN_STYLE;}
<S_URL>{WHITESPACE}*"("*{WHITESPACE}*{DOUBLEQUOT}{WHITESPACE}* {BEGIN (ST_VALUE_DOUBLE_QUOTED);  return T_IN_STYLE;}
<S_URL>{WHITESPACE}*"("{WHITESPACE}* {BEGIN (ST_VALUE_NOT_QUOTED);  return T_IN_STYLE;}
<S_URL>.|\n {BEGIN(INITIAL);  return T_IN_STYLE;}

<S_ALPHAIMAGELOADER>{WHITESPACE}*{QUOT}{WHITESPACE}* {BEGIN (ST_VALUE_ONE_QUOTED); return T_IN_STYLE;}
<S_ALPHAIMAGELOADER>{WHITESPACE}*{DOUBLEQUOT}{WHITESPACE}* {BEGIN (ST_VALUE_DOUBLE_QUOTED);  return T_IN_STYLE;}
<S_ALPHAIMAGELOADER>{WHITESPACE}* {BEGIN (ST_VALUE_NOT_QUOTED);  return T_IN_STYLE;}
<S_ALPHAIMAGELOADER>.|\n {BEGIN(INITIAL);  return T_IN_STYLE;}

<ST_VALUE_ONE_QUOTED>{ONEQUOTEDSTRING} {
    BEGIN(INITIAL);
    if(importurl)
    {
        importurl = 0;
        return(T_IMPORT_URL);
    }
    importurl = 0;
    return(T_STYLE_URL);
}
<ST_VALUE_DOUBLE_QUOTED>{DOUBLEQUOTEDSTRING} {
    BEGIN(INITIAL);
    if(importurl)
    {
        importurl = 0;
        return(T_IMPORT_URL);
    }
    importurl = 0;
    return(T_STYLE_URL);
}
<ST_VALUE_NOT_QUOTED>{NOTQUOTEDSTRING} {
    BEGIN(INITIAL);
    if(importurl)
    {
        importurl = 0;
        return(T_IMPORT_URL);
    }
    importurl = 0;
    return(T_STYLE_URL);
}

.|\n  {BEGIN(INITIAL);  return T_IN_STYLE;}
%%
char* psi_proxify_css(apr_pool_t* pool, const char* css,
                      size_t css_len,
                      const char* base_url,
                      const char* prefixhtml,
                      const char* prefixcss,
                      const char* enc_key)
{
    if(!prefixhtml || !css || !base_url)
        return NULL;

    stringbuf* bufctx = apr_palloc(pool, sizeof(stringbuf) ) ;
    bufctx->buf = NULL;
    bufctx->offset = 0;
    bufctx->avail = 0;

    char* token = NULL;
    const char* prefix = NULL;
    char* chars = NULL;
    int nstate = T_IN_STYLE;

    yy_scan_bytes(css, css_len);
    while(nstate = yylex())
    {
        token = yytext;
        if(nstate == T_STYLE_URL)
        {
            prefix = prefixhtml;
        }
        else if(nstate == T_IMPORT_URL )
        {
            prefix = prefixcss;
        }
        else
            prefix = NULL;

        if(prefix)
        {
            chars = psi_proxify_url(pool, token, base_url, enc_key);
            if(chars)
            {
                if(!stringbuf_append(pool, bufctx, prefix, strlen(prefix)))
                    break;
                if(!stringbuf_append(pool, bufctx, "/", 1))
                    break;
                if(!stringbuf_append(pool, bufctx, chars, strlen(chars)))
                    break;
            }
        }
        else
            if(!stringbuf_append(pool, bufctx, token, strlen(token)))
                break;
    }
    return stringbuf2str(pool, bufctx);
}

