#include "jsf.h"
#include "site_rules.h"

regs_v_t **jsf_init(request_rec *r) {

    char *search;
    int flags;
    const char *domain;
    const char *domain_db;
    const char *path;
    const char *path_db;

    char *c;

    apr_status_t error;
    apr_dbd_results_t *res = NULL;
    apr_dbd_row_t *row = NULL;
    ap_dbd_t *dbd;

    char *query;

    ap_regex_t *regex;
    regs_v_t **regs_v ;
    regs_v_t* newreg;
    apr_array_header_t* regs_array;

    apr_pool_t *p = r->pool;
    regs_array = apr_array_make(p, 0, sizeof(regs_v_t*));


    // getting domain information
    if (!(domain = get_request_domain(r)))
    {
        jsf_err_log("jsf():no r->subprocess_env->psiphon_proxy_site found");
        return NULL;
    }


    // getting the path
    path = get_request_path(r);
    jsf_log("jsf(): path: %s", path);


    if (!(dbd = ap_dbd_acquire(r))) {
        jsf_err_log("jsf(): Can't get DB connection");
        return NULL;
    }

    query = apr_pstrdup(p, "select srch, repl, flags, domain, path from jsf order by ord desc");

    if (error=apr_dbd_select(dbd->driver,p,dbd->handle, &res, query, 0)) {
        jsf_err_log("jsf(): %s: %s", apr_dbd_error(dbd->driver, dbd->handle, error), query);
        return NULL;
    }


    
    for (error = apr_dbd_get_row(dbd->driver, r->pool, res, &row, -1);
         error != -1;
         error = apr_dbd_get_row(dbd->driver, r->pool, res, &row, -1))     
    {
        if (error != 0) {
            jsf_err_log("jsf(): apr_dbd_get_row(): %d - %s", error, apr_dbd_error(dbd->driver, dbd->handle, error));
            break;
        }

        domain_db = apr_pstrdup(p,apr_dbd_get_entry(dbd->driver, row, 3));
        path_db = apr_pstrdup(p,apr_dbd_get_entry(dbd->driver, row, 4));

        if (!request_matches_url_rule(domain, path, domain_db, path_db))
        {
            jsf_log("jsf(): \"%s%s\" does not match \"%s%s\"", domain_db, path_db, domain, path);
            continue;
        }

        jsf_log("jsf(): \"%s%s\" is in \"%s%s\"", domain_db, path_db, domain, path);


        search  = apr_pstrdup(p, apr_dbd_get_entry(dbd->driver, row, 0));
        flags   = atoi(apr_dbd_get_entry(dbd->driver, row, 2));

        if (!(regex = ap_pregcomp(p, search, flags))) {
            jsf_log("jsf(): Can't compile regex: %s", search);
            continue;
        }

        regs_v = (regs_v_t**)apr_array_push(regs_array);

        newreg = apr_palloc(p, sizeof(regs_v_t));
        
        newreg->regex = regex;
        newreg->search = search;
        newreg->flags = flags;
        newreg->replace = apr_pstrdup(p, jsf_interpolate_vars(r, apr_dbd_get_entry(dbd->driver, row, 1)));
        *regs_v = newreg;

    }
    regs_v = apr_array_push(regs_array);
    *regs_v = NULL;
    
    return (regs_v_t**)regs_array->elts;
}

const char * escape_regex_symbols(apr_pool_t *p, const char *s)
{
    int i, j, extra_chars;
    char *res;

    /* first, count the number of extra characters */
    for (i = 0, extra_chars = 0; s[i] != '\0'; i++)
        if (s[i] == '$' || s[i] == '&')
            extra_chars++;

    if (extra_chars == 0)
        return apr_pstrmemdup(p, s, i);

    res = apr_palloc(p, i + extra_chars + 1);
    for (i = 0, j = 0; s[i] != '\0'; i++, j++)
    {
        if (s[i] == '$' || s[i] == '&')
        {
            res[j] = '\\';
            j++;
        }
        res[j] = s[i];
    }

    res[j] = '\0';
    return res;
}

const char *jsf_interpolate_vars(request_rec* r, const char* str) {
    const char* start;
    const char* end;
    const char* delim;
    const char* before;
    const char* after;
    const char* replacement;
    const char* var;
    for (;;) {
        start = str ;
        if (start = ap_strstr_c(start, "${"), start == NULL)
            break;

        if (end = ap_strchr_c(start+2, '}'), end == NULL)
            break;

        delim = ap_strchr_c(start, '|');
        before = apr_pstrndup(r->pool, str, start-str);
        after = end+1;
        if (delim) {
            var = apr_pstrndup(r->pool, start+2, delim-start-2) ;
        } else {
            var = apr_pstrndup(r->pool, start+2, end-start-2) ;
        }
        replacement = apr_table_get(r->subprocess_env, var) ;
        if (!replacement) {
            if (delim)
                replacement = apr_pstrndup(r->pool, delim+1, end-delim-1);
            else
                replacement = "";
        }
        str = apr_pstrcat(r->pool, before, escape_regex_symbols(r->pool,replacement), after, NULL);
        jsf_log("jsf_interpolate_vars: "
                "Interpolating %s  =>  %s", var, replacement) ;
    }
    return str;
}

const char *jsf(request_rec *r, regs_v_t **regs_v, const char *buf, size_t len) {

    char *re = "";
    const char *a;
    char *sub_r;

    int last_expr;
    apr_pool_t *p = r->pool;

    ap_regmatch_t regmatch[AP_MAX_REG_MATCH];
    int i;
    const char *body;

    if(!buf || !len || !*buf)
        return NULL;


    body = apr_pstrndup(p, buf, len);

    if(!body || !*body)
        return NULL;

    a = body;

    last_expr = 0;
    for (i = 0; regs_v[i]; i++) {
        re = "";
        while (!ap_regexec(regs_v[i]->regex, a, AP_MAX_REG_MATCH, regmatch,0)) {
            jsf_log("jsf(): found: \"%s\" in \"%s\"", regs_v[i]->search, a);

            if (regmatch->rm_eo == 0) { // there are some cases with rm_eo=rm_so=0
                break;
            }

            sub_r = ap_pregsub(p, regs_v[i]->replace, a,
                                AP_MAX_REG_MATCH, regmatch);

            re = apr_pstrcat(p,
                          re,
                          apr_pstrndup(p, a, regmatch->rm_so),
                          sub_r,
                          NULL);

            a += regmatch->rm_eo;
            if (regs_v[i]->flags & JSF_LAST_RULE) {
            last_expr = 1;
            }
        }

        if (a != body) {
            re = apr_pstrcat(p, re, a, NULL);
            a = re;
        }

        jsf_log("jsf(): result: \"%s\"", a);

        if (last_expr) {
            jsf_log("jsf(): last rule - going to the next line");
            break;
        }
    }
    return a;
}
