#include "psiphon_url.h"


unsigned char psi_uri_acceptable_chars[96] =
{
    0x0,  0xF,  0x0,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0x0,  0xF,  0x0,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0x0,  0xF,  0x0,  0xF,
    0x0,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0xF,   0xF,  0xF,  0xF,  0xF,
    0xF,  0xF,  0xF,  0x0,   0x0,  0x0,  0xF,  0x0
};

char x2c(char c) {
    if (c >= '0' && c <= '9') {
        return (c - '0');
    } else if (c >= 'a' && c <= 'z') {
        return (c - 'a' + 10);
    } else if (c >= 'A' && c <= 'Z') {
        return (c - 'A' + 10);
    } else {
        return (0); //unprintable char
    }
}
char* psi_url_encode (apr_pool_t* pool, char *str)
{
    const char *p;
    char *q;
    char *result;
    
    int unacceptable = 0;
    char *hex = "0123456789ABCDEF";
    int mask = 0x8;
    int len = 0;

    if (!str || !*str)
    {
        return NULL;
    }
    
    //trim right
    len = strlen(str);
    len--;
    while (len > 0 && (str[len] == ' ' || str[len] == '\t'
           || str[len] == '\n' || str[len] == '\r'))
    {
        memmove(str+len, str+len+1, 1);
        --len;
    }
    
    if (!str || !*str)
    {
        return NULL;
    }

    for (p = str; *p; p++)
        if (!CHAR_ACCEPTABLE((unsigned char) (*p)))
            unacceptable++;

    result = (char *)apr_palloc(pool, p - str + unacceptable + unacceptable + 1);

    for (q = result, p = str; *p; p++)
    {
        unsigned char a = *p;
        if (!CHAR_ACCEPTABLE(a))
        {
            *q++ = CHAR_HEX_ESCAPE;
            *q++ = hex[a >> 4];
            *q++ = hex[a & 15];
        }
        else *q++ = *p;
    }
    *q++ = 0;                   /* Terminate */
    return result;
}

char* psi_rawurl_encode (apr_pool_t* pool, char *str)
{
    const char *p;
    char *q;
    char *result;
    
    int unacceptable = 0;
    char *hex = "0123456789ABCDEF";
    int mask = 0x8;
    int len = 0;
    unsigned char a = 0;

    if (!str || !*str)
    {
        return NULL;
    }
    
    //trim right
    len = strlen(str);
    len--;
    while (len > 0 && (str[len] == ' ' || str[len] == '\t'
           || str[len] == '\n' || str[len] == '\r'))
    {
        memmove(str+len, str+len+1, 1);
        --len;
    }
    
    if (!str || !*str)
    {
        return NULL;
    }

    for (p = str; *p; p++)
    {
        a = *p;
        if (!apr_isalnum(a) && !strchr("-_.!~*'()", a))
            unacceptable++;
    }

    result = (char *)apr_palloc(pool, p - str + unacceptable + unacceptable + 1);

    for (q = result, p = str; *p; p++)
    {
        a = *p;
        if (!apr_isalnum(a) && !strchr("-_.!~*'()", a))
        {
            *q++ = CHAR_HEX_ESCAPE;
            *q++ = hex[a >> 4];
            *q++ = hex[a & 15];
        }
        else *q++ = *p;
    }
    *q++ = 0;                   /* Terminate */
    return result;
}

char* psi_rawurl_decode (char * str)
{
    char *hex = "0123456789ABCDEF";
    int mask = 0x8;

    char * p = str;
    char * q = str;

    if (!str)
        return NULL;

    while(*p)
    {
        if (*p == CHAR_HEX_ESCAPE )
        {
            p++;
            if (*p) *q = x2c(*p++) * 16;
            if (*p) *q = FROMASCII(*q + x2c(*p)), ++p;
            q++;
        } else {
            *q++ = *p++;
        }
    }

    *q++ = 0;
    return str;

}

int psi_uri_has_scheme(const char* uri)
{
    const char *p = uri;
 
    /* The first char must be a scheme char. */
    if (!*p || !(apr_isalnum (*p) || (*p) == '-' || (*p) == '+'))
        return 0;
    ++p;
    /* Followed by 0 or more scheme chars. */
    while (*p && (apr_isalnum (*p) || (*p) == '-' || (*p) == '+'))
        ++p;
    /* Terminated by ':' */
    return strncmp(p, "://", 3) == 0;
}

char* psi_merge_url(apr_pool_t* pool, const char* url, const char* base)
{
    int linklength;
    const char* end;
    char* merge;
    apr_pool_t* p = pool;

    // We may not examine BASE past END. 
    end = base + psi_url_path_length(base);
    linklength = strlen(url);

    if(!*url)
    {
        // Empty LINK points back to BASE, query string and all. 
        return apr_pstrdup(p, base);
    }
    else if(*url == '?')
    {
        // LINK points to the same location, but changes the query
        //                      string.  Examples: 
        // uri_merge("path",         "?new") -> "path?new"     
        // uri_merge("path?foo",     "?new") -> "path?new"     
        // uri_merge("path?foo#bar", "?new") -> "path?new"     
        // uri_merge("path#foo",     "?new") -> "path?new"     
        int baselength = end - base;

        merge = (char*)apr_palloc(p, baselength + linklength + 1);

        memcpy(merge, base, baselength);
        memcpy(merge + baselength, url, linklength);
        merge[baselength + linklength] = '\0';
    }
    else if(*url == '#')
    {
        /* uri_merge("path",         "#new") -> "path#new"     */
        /* uri_merge("path#foo",     "#new") -> "path#new"     */
        /* uri_merge("path?foo",     "#new") -> "path?foo#new" */
        /* uri_merge("path?foo#bar", "#new") -> "path?foo#new" */
    
        int baselength;
        const char* end1 = strchr(base, '#');

        if(!end1)
            end1 = base + strlen(base);

        baselength = end1 - base;
        merge = (char*)apr_palloc(p, baselength + linklength + 1);
        memcpy(merge, base, baselength);
        memcpy(merge + baselength, url, linklength);
        merge[baselength + linklength] = '\0';
    }
    else if(*url == '/' && *(url + 1) == '/')
    {
        /* LINK begins with "//" and so is a net path: we need to
        replace everything after (and including) the double slash
        with LINK. */

        /* uri_merge("foo", "//new/bar")            -> "//new/bar"      */
        /* uri_merge("//old/foo", "//new/bar")      -> "//new/bar"      */
        /* uri_merge("http://old/foo", "//new/bar") -> "http://new/bar" */

        int span;
        const char* slash;
        const char* start_insert;

        /* Look for first slash. */
        slash = (const char*)memchr(base, '/', end - base);
        /* If found slash and it is a double slash, then replace
        from this point, else default to replacing from the
        beginning.  */
        if(slash && *(slash + 1) == '/')
            start_insert = slash;
        else
            start_insert = base;

        span = start_insert - base;
        merge = (char*)apr_palloc(p, span + linklength + 1);
        if(span)
            memcpy(merge, base, span);
        memcpy(merge + span, url, linklength);
        merge[span + linklength] = '\0';
    }
    else if(*url == '/')
    {
       /* LINK is an absolute path: we need to replace everything
        after (and including) the FIRST slash with LINK.
        So, if BASE is "http://host/whatever/foo/bar", and LINK is
        "/qux/xyzzy", our result should be
        "http://host/qux/xyzzy".  */
        int span;

        const char* slash;
        const char* start_insert = NULL; /* for gcc to shut up. */
        const char* pos = base;

        int seen_slash_slash = 0;
        /* We're looking for the first slash, but want to ignore
        double slash. */

        again : slash = (const char*)memchr(pos, '/', end - pos);

        if(slash && !seen_slash_slash)
            if(*(slash + 1) == '/')
        {
            pos = slash + 2;
            seen_slash_slash = 1;

            goto again;
        }

        /* At this point, SLASH is the location of the first / after
        "//", or the first slash altogether.  START_INSERT is the
        pointer to the location where LINK will be inserted.  When
        examining the last two examples, keep in mind that LINK
        begins with '/'. */

        if(!slash && !seen_slash_slash)
            /* example: "foo" */
            /*           ^    */
            start_insert = base;
        else if(!slash && seen_slash_slash)
            /* example: "http://foo" */
            /*                     ^ */
            start_insert = end;
        else if(slash && !seen_slash_slash)
            /* example: "foo/bar" */
            /*           ^        */
            start_insert = base;
        else if(slash && seen_slash_slash)
            /* example: "http://something/" */
            /*                           ^  */
            start_insert = slash;

        span = start_insert - base;
        merge = (char*)apr_palloc(p, span + linklength + 1);

        if(span)
            memcpy(merge, base, span);

        memcpy(merge + span, url, linklength);
        merge[span + linklength] = '\0';
    }
    else
    {
        /* LINK is a relative URL: we need to replace everything
        after last slash (possibly empty) with LINK.
        So, if BASE is "whatever/foo/bar", and LINK is "qux/xyzzy",
        our result should be "whatever/foo/qux/xyzzy".  */
        int need_explicit_slash = 0;
        int span;
        const char* start_insert = 0;
        const char* last_slash = 0;

        for(;end > base && *(end + 1) != '/';end--)
            last_slash = end;

        if(last_slash == 0 || !last_slash)
        {
            /* No slash found at all.  Replace what we have with LINK. */
            start_insert = base;
        }
        else if(last_slash && last_slash >= base + 2 && last_slash[-2] == ':' && last_slash[-1] == '/')
        {
            /* example: http://host"  */
            /*                      ^ */
            start_insert = end + 1;
            need_explicit_slash = 1;
        }
        else
        {
            /* example: "whatever/foo/bar" */
            /*                        ^    */
            start_insert = last_slash + 1;
        }

        span = start_insert - base;
        merge = (char*)apr_palloc(p, span + linklength + 1);

        if(span)
            memcpy(merge, base, span);

        if(need_explicit_slash)
            merge[span - 1] = '/';

        memcpy(merge + span, url, linklength);
        merge[span + linklength] = '\0';
    }

    return merge;
}


int psi_url_path_length(const char* url)
{
    const char* q = strpbrk(url, "?#");

    if(!q)
        q = strchr(url, '\0');

    return q - url;
}


char* psi_simplify_uri(apr_pool_t* pool, const char* str)
{

    apr_pool_t *p = pool;
    apr_uri_t * uri = apr_palloc(p, sizeof(apr_uri_t));


    if (APR_SUCCESS != apr_uri_parse(p, str, uri))
        return NULL; /*couldn't parse url*/


    if(!uri->scheme || !*uri->scheme)
        return NULL; /*must have scheme*/

    if(!uri->hostname || !*uri->hostname)
        return NULL; /*must have hostname*/

    if(!uri->path || !*uri->path)
        uri->path = "/"; /*add explicit '/' if path is empty*/

    psi_path_simplify(uri->path);

    return(apr_uri_unparse(p, uri, 0));
}

char* psi_proxify_url(apr_pool_t* pool, const char* url,  const char* base)
{
    char *transformed_url;
    int len = 0;
    int pos = 0;

    apr_pool_t* p = pool;


    len = strlen(url);

    //trim left
    while (pos != len && (url[pos] == ' ' || url[pos] == '\t'
           || url[pos] == '\n' || url[pos] == '\r'))
        ++pos;
    

    url = &url[pos];

    
    if(psi_uri_has_scheme(url))
        transformed_url = apr_pstrdup(p, url);
    else
        transformed_url = (char*)psi_merge_url(p, url, base);

    if(!transformed_url)
        return NULL;

    transformed_url = psi_simplify_uri(p, transformed_url);

    if(!transformed_url)
        return NULL;

    transformed_url = psi_url_encode(p, transformed_url);

    if(!transformed_url)
        return NULL;

    return transformed_url;
}

void psi_path_simplify (char *path)
{
    register int i, start, ddot;
    char stub_char;

    if (!*path)
        return;

    /*stub_char = (*path == '/') ? '/' : '.';*/
    stub_char = '/';

  /* Addition: Remove all `./'-s preceding the string.  If `../'-s
    precede, put `/' in front and remove them too.  */
    i = 0;
    ddot = 0;
    while (1)
    {
        if (path[i] == '.' && path[i + 1] == '/')
            i += 2;
        else if (path[i] == '.' && path[i + 1] == '.' && path[i + 2] == '/')
        {
            i += 3;
            ddot = 1;
        }
        else
            break;
    }
    if (i)
        strcpy (path, path + i - ddot);

    /* Replace single `.' or `..' with `/'.  */
    if ((path[0] == '.' && path[1] == '\0')
         || (path[0] == '.' && path[1] == '.' && path[2] == '\0'))
    {
        path[0] = stub_char;
        path[1] = '\0';
        return;
    }
    /* Walk along PATH looking for things to compact.  */
    i = 0;
    while (1)
    {
        if (!path[i])
            break;

        while (path[i] && path[i] != '/')
            i++;

        start = i++;

        /* If we didn't find any slashes, then there is nothing left to do.  */
        if (!path[start])
            break;

        /* Handle multiple `/'s in a row.  */
        while (path[i] == '/')
            i++;

        if ((start + 1) != i)
        {
            strcpy (path + start + 1, path + i);
            i = start + 1;
        }

        /* Check for trailing `/'.  */
        if (start && !path[i])
        {
            path[i] = '\0';
            break;
            zero_last:  path[--i] = '\0';
            break;
        }

        /* Check for `../', `./' or trailing `.' by itself.  */
        if (path[i] == '.')
        {
            /* Handle trailing `.' by itself.  */
            if (!path[i + 1])
                goto zero_last;

            /* Handle `./'.  */
            if (path[i + 1] == '/')
            {
                strcpy (path + i, path + i + 1);
                i = (start < 0) ? 0 : start;
                continue;
            }

            /* Handle `../' or trailing `..' by itself.  */
            if (path[i + 1] == '.' &&
                (path[i + 2] == '/' || !path[i + 2]))
            {
                while (--start > -1 && path[start] != '/');
                strcpy (path + start + 1, path + i + 2);
                i = (start < 0) ? 0 : start;
                continue;
            }
        }   /* path == '.' */
    } /* while */

    if (!*path)
    {
        *path = stub_char;
        path[1] = '\0';
    }
}

char* psi_strndup(const char *str, size_t len)
{
    register size_t n;
    register char *dst;

    n = strlen(str);
    if (len < n)
        n = len;
    dst = (char *) malloc(n+1);
    if (dst) {
        memcpy(dst, str, n);
        dst[n] = '\0';
    }
    return dst;
}

