/*
    Psiphon Circumvention Platform
    Copyright (C) 2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "site_rules.h"
#include <apr_strings.h>

int str_end_with(const char* str, const char* tail)
{
    int l_diff = strlen(str) - strlen(tail);
    if (l_diff < 0)
    {
        return 0;
    }
    else
    {
        return !strcasecmp(str + l_diff, tail);
    }
}

const char* get_request_domain(request_rec* r)
{
    const char* domain = 0;
    char* c = 0;
    apr_pool_t* p = r->pool;

    if (!(domain = apr_table_get(r->subprocess_env, "psiphon_proxy_site")))
    {
        return 0;
    }

    if (c = strrchr(domain, '/'))
    {
        domain = apr_pstrdup(p, c + 1);
    }
    else
    {
        domain = apr_pstrdup(p, domain);
    }
    
    return domain;
}

const char* get_request_path(request_rec* r)
{
    const char* path = 0;
    apr_pool_t* p = r->pool;

    path = apr_pstrdup(p, r->filename);
    
    if (path = strstr(path, "://"))
    {
        path += 3; /* strlen("://"); */
        path = strchr(path, '/');
    }
    
    if (!path || !*path)
    {
        path = apr_pstrdup(p, "/");
    }
    
    return path;
}

int request_matches_url_rule(const char* request_domain,
                             const char* request_path,
                             const char* rule_domain,
                             const char* rule_path)
{
    if (strcasecmp(request_domain, rule_domain))
    {
        if (*rule_domain != '.')
        {
            /* rule domain is not a wildcharacter */
            return 0;
        }
        if (!str_end_with(request_domain, rule_domain) && !str_end_with(request_domain, rule_domain + 1))
        {
            /* www.hz.com ~= .hz.com
                   hz.com ~= .hz.com */
            return 0;
        }
    }

    if (rule_path && *rule_path)
    {
        if (strncmp(request_path, rule_path, strlen(rule_path)))
        {
            return 0;
        }
    }
    
    return 1;
}
