#include "stringbuf.h"


int stringbuf_resize(apr_pool_t* pool, stringbuf* ctx, const size_t len)
{
    char* newbuf ;
    if ( len <= ( ctx->avail - ctx->offset ) )
        return len;
    else while ( len > ( ctx->avail - ctx->offset ) )
    {
        ctx->avail += BUF_CHUNK ;
        if( ctx->avail > BUF_MAX_SIZE)
        {
            ctx->avail -= BUF_CHUNK ;
            return 0;
        }
    }

    newbuf = realloc(ctx->buf, ctx->avail) ;
    if ( newbuf != ctx->buf )
    {
        if ( ctx->buf )
            apr_pool_cleanup_kill(pool, ctx->buf, (void*)free) ;
        apr_pool_cleanup_register(pool, newbuf,
                                  (void*)free, apr_pool_cleanup_null);
        ctx->buf = newbuf ;
    }
    return len;
}

int stringbuf_append(apr_pool_t* pool, stringbuf* ctx, const char* buf,
                             const size_t len)
{

    if(!stringbuf_resize(pool, ctx, len))
        return 0;
    memcpy(ctx->buf+ctx->offset, buf, len) ;
    ctx->offset += len ;
    return len;
}

char* stringbuf2str(apr_pool_t* pool, stringbuf* ctx)
{
    char c = '\0' ;
    if(!stringbuf_append(pool, ctx, &c, 1))
    {
        ctx->buf[ctx->avail-1] = '\0';
    }
    return ctx->buf;
}
