/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

SET character_set_client = utf8;

USE psiphon;

--
-- Modifications to table user
--

ALTER TABLE user
    ADD COLUMN pass_salt char(32);

ALTER TABLE user
    MODIFY COLUMN pass char(40) DEFAULT '';

--
-- replacing SELECT COUNT(*) INTO row_count FROM user WHERE (user.email = NEW.uname OR user.uname = NEW.email AND NEW.id != user.id;
-- which executes much slower
--

delimiter ;;
DROP TRIGGER IF EXISTS `update_user_trigger`;;
CREATE TRIGGER update_user_trigger
BEFORE UPDATE ON user FOR EACH ROW
BEGIN
 DECLARE row_count INT;
 DECLARE dummy INT;
 SELECT COUNT(*) INTO row_count FROM user WHERE user.uname = NEW.email AND NEW.id != user.id;
 IF row_count > 0 THEN
     SELECT uname_or_email_is_not_unique INTO dummy;
 END IF;
 SELECT COUNT(*) INTO row_count FROM user WHERE user.email = NEW.uname AND NEW.id != user.id;
 IF row_count > 0 THEN
     SELECT uname_or_email_is_not_unique INTO dummy;
 END IF;
END;
;;
delimiter ;

--
-- Modifications/additions to stats tables for per-group reporting
-- (upgrade sets existing stats to "User" [3] group, arbitrarily)
--

ALTER TABLE stat_region
    ADD COLUMN grp int(1) NOT NULL DEFAULT '3';
ALTER TABLE stat_region
    ADD COLUMN proxy_notes text NOT NULL DEFAULT '';
ALTER TABLE stat_region
    DROP PRIMARY KEY,
    ADD PRIMARY KEY (date, region, proxy, grp);

ALTER TABLE stat_site
    ADD COLUMN grp int(1) NOT NULL DEFAULT '3';
ALTER TABLE stat_site
    ADD COLUMN proxy_notes text NOT NULL DEFAULT '';
ALTER TABLE stat_site
    DROP PRIMARY KEY,
    ADD PRIMARY KEY (date, domain, region, proxy, grp);

CREATE TABLE stat_account_created(
    date INT(11) NOT NULL,
    region CHAR(2) NOT NULL,
    proxy INT(11) NOT NULL,
    proxy_notes TEXT  NOT NULL DEFAULT '',
    grp INT(11) NOT NULL,
    cnt INT(11) DEFAULT 0,
    PRIMARY KEY (date, region, proxy, grp)
) TYPE=MyISAM;

--
-- Modifications to the proxy table for management
--

ALTER TABLE proxy
    ADD COLUMN name char(64) NOT NULL DEFAULT 'Proxy';
ALTER TABLE proxy
    ADD COLUMN notes text NOT NULL DEFAULT '';
ALTER TABLE proxy
    ADD COLUMN hostname char(64) NOT NULL DEFAULT '';

--
-- Fix signed inet addr column so SQL INET_NTOA works properly
-- see: http://dev.mysql.com/doc/refman/5.0/en/miscellaneous-functions.html#function_inet-aton
-- WARNING: no transaction, may result in bad state!
--

ALTER TABLE proxy ADD COLUMN addr2 int(11) unsigned;
UPDATE proxy SET addr2 = addr WHERE addr >= 0;
UPDATE proxy SET addr2 = pow(2,32)+addr WHERE addr < 0;
ALTER TABLE proxy DROP INDEX idx_addr;
ALTER TABLE proxy DROP COLUMN addr;
ALTER TABLE proxy ADD COLUMN addr int(11) unsigned;
ALTER TABLE proxy ADD UNIQUE KEY idx_addr (addr);
UPDATE proxy SET addr = addr2;
ALTER TABLE proxy DROP COLUMN addr2;

--
-- Obsolete features
--

DROP TABLE feedback;
DROP TABLE folder;
DROP TABLE msg;
DROP TABLE mtype;
ALTER TABLE proxy DROP COLUMN welcome;
ALTER TABLE proxy DROP COLUMN body;
ALTER TABLE proxy DROP COLUMN vlists;
DROP TABLE vgroup;
DROP TABLE vlist;
ALTER TABLE user DROP COLUMN msgs;
ALTER TABLE user DROP COLUMN sent;
ALTER TABLE user DROP COLUMN msg_ppage;


--
-- New stats tables
--

CREATE TABLE stat_blue_bar_submit(
    date INT(11) NOT NULL,
    domain CHAR(48) NOT NULL,
    region CHAR(2) NOT NULL,
    proxy INT(11) NOT NULL,
    proxy_notes TEXT  NOT NULL DEFAULT '',
    grp INT(11) NOT NULL,
    cnt INT(11) DEFAULT 0,
    PRIMARY KEY (date, domain, region, proxy, grp)
) TYPE=MyISAM;

