--
-- Modifications to table anonymous_browsing(show created date and time)
--
ALTER TABLE anonymous_browsing ADD created int(11) NOT NULL DEFAULT 0;
UPDATE anonymous_browsing SET created=UNIX_TIMESTAMP();

-- Node Admin and Node Host user types have been removed from the system.
-- Delete any user records of those types.
DELETE FROM user WHERE grp=1 OR grp=5;

-- Create the User-Proxy association table.
CREATE TABLE userproxy (
    userid int(11),
    proxyid int(11),
    PRIMARY KEY(userid, proxyid)
);

-- Put the existing user-proxy associations into the new table.
-- Don't copy over values for users with "other_nodes" permission (Admin and Reporter).
INSERT INTO userproxy (userid, proxyid)
    SELECT user.id, user.proxy
    FROM user
    WHERE user.grp <> 0 AND user.grp <> 7;

-- Delete the now-unused proxy column from the user table.
ALTER TABLE user DROP COLUMN proxy;

-- Add required indexes on tables
ALTER TABLE user ADD INDEX idx_user__grp (grp);
ALTER TABLE user ADD INDEX idx_user__last_reg (last_reg);
ALTER TABLE user ADD INDEX idx_user__created (created);
ALTER TABLE user ADD INDEX idx_user__last (last);
ALTER TABLE userproxy ADD INDEX idx_userproxy__proxyid (proxyid);
ALTER TABLE invitation  ADD INDEX idx_proxy(proxy);

-- Add new column that indicates the most recently used proxy.
ALTER TABLE user ADD COLUMN current_proxy INT(11);
ALTER TABLE user ADD INDEX idx_user__current_proxy (current_proxy);
-- Set one of the existing userproxy entries as the current_proxy. (Note that not
-- all users have userproxy entries and so will still have NULL current_proxy.)
UPDATE user JOIN userproxy ON user.id = userproxy.userid SET user.current_proxy = userproxy.proxyid;

-- Create the custom user agent table
DROP TABLE IF EXISTS useragent;
CREATE TABLE useragent (
  id int(11) NOT NULL AUTO_INCREMENT,
  domain char(64) NOT NULL DEFAULT '',
  path char(255) NOT NULL DEFAULT '',
  user_agent_string text,
  PRIMARY KEY (id),
  UNIQUE KEY (domain, path)
);

-- Remove database email queue
DROP TABLE IF EXISTS email_queue;
ALTER TABLE invitation DROP COLUMN email_queue_id;

-- Add autoincrement index to bookmarks for edit/delete operations
-- and create a single unique index on (user, url) 
ALTER TABLE bookmark DROP PRIMARY KEY;
ALTER TABLE bookmark ADD id int(11) PRIMARY KEY AUTO_INCREMENT NOT NULL FIRST;
ALTER TABLE bookmark ADD UNIQUE INDEX idx_user_url(user, url);
