/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

SET character_set_client = utf8;

USE psiphon;

--
-- Modifications to table user
--

ALTER TABLE user
    ADD COLUMN home_page char(255);
ALTER TABLE user
    ADD COLUMN home_page_locked int(1) NOT NULL DEFAULT 0;
ALTER TABLE user
    ADD COLUMN last_activity int(11) NOT NULL DEFAULT 0;
ALTER TABLE user
    ADD COLUMN consecutive_failed_logins int(11) NOT NULL DEFAULT 0;
ALTER TABLE user
    ADD COLUMN login_delay int(11) NOT NULL DEFAULT 0;
ALTER TABLE user
    DROP COLUMN crpt_url;
ALTER TABLE user
    DROP COLUMN feedback;
ALTER TABLE user
    DROP COLUMN creator;
ALTER TABLE user
    ADD COLUMN email_candidate char(64);
ALTER TABLE user
    ADD COLUMN email_candidate_token char(32);
ALTER TABLE user
    ADD COLUMN email_candidate_date int(11);
-- note the default 0 means existing users will be flagged as *not*
-- having received the welcome email and will get it again if they
-- update email address... this is intentional
ALTER TABLE user
    ADD COLUMN welcome_email_sent int(1) NOT NULL DEFAULT 0;
ALTER TABLE user
    ADD COLUMN agreed_to_usage_terms int(1) NOT NULL DEFAULT 0;
ALTER TABLE user
    ADD COLUMN csrf_token char(32);

--
-- Create table anonymous_browsing
--

CREATE TABLE anonymous_browsing (
  id INT(11) NOT NULL AUTO_INCREMENT,
  token CHAR(32) NOT NULL,
  home_page CHAR(255) NOT NULL,
  home_page_locked INT(1) NOT NULL,
  lang CHAR(2) NOT NULL,
  proxy INT(11) NOT NULL,
  PRIMARY KEY (id)
) TYPE=MyISAM AUTO_INCREMENT=1;

--
-- Modifications to table invitation
--

ALTER TABLE invitation
    DROP COLUMN subject;
ALTER TABLE invitation
    ADD COLUMN email_queue_id INT(11);
ALTER TABLE invitation
    ADD COLUMN grp INT(1) DEFAULT 3;

--
-- Create table email_queue
--

CREATE TABLE email_queue (
  id int(11) NOT NULL AUTO_INCREMENT,
  recipient_email_address char(64) NOT NULL,
  subject char(64) NOT NULL,
  body text NOT NULL,
  lang char(2) NOT NULL,
  PRIMARY KEY (id)
) TYPE=MyISAM AUTO_INCREMENT=1;



-- check and fix existing user records
-- before creating triggers

UPDATE user AS u
INNER JOIN user AS t ON u.email = t.uname
SET t.uname = CONCAT(t.uname, '_1')
WHERE u.id != t.id;

--
-- db triggers
--

delimiter ;;
DROP TRIGGER IF EXISTS `insert_user_trigger`;;

CREATE TRIGGER insert_user_trigger
BEFORE INSERT ON user FOR EACH ROW
BEGIN
 DECLARE row_count INT;
 DECLARE dummy INT;
 SELECT COUNT(*) INTO row_count FROM user WHERE
 user.email = NEW.uname OR user.uname = NEW.email;
 IF row_count > 0 THEN
     SELECT uname_or_email_is_not_unique INTO dummy;
 END IF;
END;
;;

DROP TRIGGER IF EXISTS `update_user_trigger`;;
CREATE TRIGGER update_user_trigger
BEFORE UPDATE ON user FOR EACH ROW
BEGIN
 DECLARE row_count INT;
 DECLARE dummy INT;
 SELECT COUNT(*) INTO row_count FROM user WHERE
 (user.email = NEW.uname OR user.uname = NEW.email)
 AND NEW.id != user.id;
 IF row_count > 0 THEN
     SELECT uname_or_email_is_not_unique INTO dummy;
 END IF;
END;
;;
delimiter ;

--
-- Invite token changes
--

ALTER TABLE user
    ADD COLUMN last_token_count INT(11) NOT NULL DEFAULT 0;

ALTER TABLE user
    ADD COLUMN last_token_accrual_time INT(11) DEFAULT NULL;

