#!/bin/sh

#mysql root password prompt
echo "Performing DB upgrades ... "
if [ ! -n "${mysql_root_pass+x}" ] ; then
   echo "Enter current password for mysql root (Press 'Enter' for none)"

   tty_mode=`stty -g`
   stty -echo
   read mysql_root_pass
   stty ${tty_mode}
   echo
fi

if [ -n "${mysql_root_pass}" ]; then 
    pass_string=" -p${mysql_root_pass}"
else    
    pass_string=""
fi

#create temp directory for db upgrades
mkdir -p tmpdb

#get db upgrades history and store in file
echo "DB upgrades history:"
mysql -u root ${pass_string} psiphon -se 'SELECT script_name, FROM_UNIXTIME(date_applied) from db_schema_version ORDER BY date_applied;'  -N
if [ $? != 0 ]; then
    exit 1
fi
mysql -u root ${pass_string} psiphon -se 'SELECT script_name from db_schema_version;'  > tmpdb/tmp
if [ $? != 0 ]; then
    exit 1
fi
sort tmpdb/tmp | uniq > tmpdb/applied_upgrades

#get full list of upgrade scripts and store in file

###find sql/upgrades/  -type f -printf "%f\n" > tmpdb/tmp
###sort tmpdb/tmp | uniq > tmpdb/available_upgrades
#Simplified instead due to FreeBSD incompatibility of find -printf
ls -1 sql/upgrades/*.sql | tr '\n' '\0' | xargs -0 -n 1 basename > tmpdb/available_upgrades
echo

#get pending updates list
PENDING=`comm -13 tmpdb/applied_upgrades tmpdb/available_upgrades` 

if [ "" = "${PENDING}" ]; then 
    echo "There are no pending DB upgrades."
    rm -rf tmpdb
    exit 0
fi
echo "The following DB upgrades are available:"
echo "${PENDING}"
if [ "$1" != "upgrade" ]; then #skip prompt
   echo
   echo -n "Would you like to apply these upgrades now (yes/no)? "
   read db_upgrade_response
else
   db_upgrade_response="yes";
fi
if [ "${db_upgrade_response}" != "yes" ]; then
   echo
   echo "Skipping DB upgrades."
   exit 0
else
   echo
   echo "Applying DB upgrades ..."
   cd sql/upgrades/
   echo "${PENDING}" | while read LINE; do
      echo
      echo "Applying $LINE ..."
      cat $LINE | mysql -f -u root ${pass_string} psiphon
      echo "INSERT INTO db_schema_version(date_applied, script_name) values(UNIX_TIMESTAMP(), '${LINE}');" | mysql -u root ${pass_string} psiphon
      if [ $? != 0 ]; then
         exit 1
      fi
      echo "$LINE has been applied."
   done 
   echo
   echo "DB upgrades done."
   echo
   cd ../../
fi
rm -rf tmpdb
