<?php
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * lang2csv.php
 * 
 * Extracts strings from lang.php and output in CSV format. Outputs to stdout.
 * NOTE: lang.php includes lang_bb.php, so this effectively extracts both.
 *
 * All entries will be enlcosed in double-quotes and all double-quotes in entries
 * will be turned into pair-of-double-quotes. (See http://en.wikipedia.org/wiki/Comma-separated_values )
 *
 * The output will look like this:
 *  First row: "key","lang1","lang2"...
 *  Subsequent rows: "keyval","lang1val","lang2val",...
 */

// NOTE: Usage count stuff is for debugging and should not be run in a normal lang-to-csv
$DEBUG_USAGE_COUNT = false;

// lang.php includes lang_bb.php and needs to know where to find it.
$_SERVER[DOCUMENT_ROOT] = "../../www";
include($_SERVER[DOCUMENT_ROOT]."/includes/lang.php");
@include($_SERVER[DOCUMENT_ROOT]."/admin/lang_admin.php");

// Properly formats an entry. Every entry is enclosed in double-quotes.
// See http://en.wikipedia.org/wiki/Comma-separated_values
function format_item($item)
{
    return "\"".str_replace("\"", "\"\"", $item)."\"";
}

function add_item_to_line($line, $item)
{
    if (strlen($line) > 0)
    {
        $line .= ",";
    }

    $line .= format_item($item);

    return $line;
}

function add_array_to_line($line, $array)
{
    foreach ($array as $item)
    {
        $line = add_item_to_line($line, $item);
    }

    return $line;
}

function output_line($line)
{
    echo $line."\n";
}

// Output the BOM
echo "\xEF\xBB\xBF";

// We'll use the number of expected langs as a sanity check on the string entries.
$num_langs = count($config['lang']);

// Output the language codes in the first row. The first entry is the string key.
$line = "";
$line = add_item_to_line($line, "StringID");
foreach ($config['lang'] as $lang => $lang_info)
{
    $line = add_item_to_line($line, $lang);
}
if ($DEBUG_USAGE_COUNT) $line = add_item_to_line($line, "Usage Count");

output_line($line);

// Output the string entries.
foreach ($config['msg'] as $string_key => $strings)
{
    $line = "";
    $line = add_item_to_line($line, $string_key);
    $line = add_array_to_line($line, $strings);

    if ($DEBUG_USAGE_COUNT)
    {
        // Check usage count
        // e.g.
        // ~/psiphon2/trunk$ find . | xargs grep "msg(\"save\")" | grep ".php:"
        // ./www/users.php:    <input type="submit" value="<?=msg("save")...
        // ./www/jsf.php:      echo "<td>{$_p}<input type=\"submit\" value=\"".msg("save")...
        //
        // Note that there are a few cases where msg(<key>) is called indirectly and
        // so this count isn't totally reliable.  E.g., report_01_0.

        $grep_output = `find $_SERVER[DOCUMENT_ROOT]/.. | xargs grep "msg(\"$string_key\"" | grep ".php:"`;
        $usage_count = substr_count($grep_output, "\n");
        $line = add_item_to_line($line, $usage_count);
    }

    output_line($line);

    // Make sure the number of entries is what we expect. (This line is after
    // the output so that we can easily see where it happened.)
    if (count($strings) != $num_langs)
    {
        exit("Bad lang count");
    }
}

exit(0);
?>
