#!/usr/sbin/php

<?php
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * lang2csv creates a single csv with all languages. We want one file for each
 * language with StringID, English, and the language to translate.
 */

if (count($argv) != 2)
{
    print("Usage: ./csv2lang.php {csvfilename}\n  Outputs results to stdout and updates lang files.\n");
    exit(0);
}

// Input parameters
$filename = $argv[1];

// Reads past the UTF-8 bom if it is there.
// fgetcsv does not seem to like the presence of the BOM.
// Taken from: http://www.php.net/manual/en/function.fopen.php#78308
function fopen_utf8 ($filename, $mode)
{
    $file = @fopen($filename, $mode);
    $bom = fread($file, 3);
    if ($bom != b"\xEF\xBB\xBF")
        rewind($file);
    return $file;
}

// Properly formats an entry. Every entry is enclosed in double-quotes.
// See http://en.wikipedia.org/wiki/Comma-separated_values
function format_item($item)
{
    return "\"".str_replace("\"", "\"\"", $item)."\"";
}

function add_item_to_line($line, $item)
{
    if (strlen($line) > 0)
    {
        $line .= ",";
    }

    $line .= format_item($item);

    return $line;
}

function add_array_to_line($line, $array)
{
    foreach ($array as $item)
    {
        $line = add_item_to_line($line, $item);
    }

    return $line;
}

function output_line($line, $lang_file_info)
{
    fwrite($lang_file_info['file_handle'], $line."\n");
}

// Open the CSV file
$fcsv = fopen_utf8($filename, "r");

// Read the languages present in the CSV
$entry = fgetcsv($fcsv);

if (!$entry)
{
    exit("File not found or otherwise bad (fgetcsv returned false): ".$filename);
}

if (count($entry) < 3 || $entry[0] != "StringID" || $entry[1] != "en")
{
    exit("First line of file must be StringID,en[,langcode2,...]");
}

$langs_in_csv = array_slice($entry, 2);
$langs_in_csv = array_filter($langs_in_csv); // remove blank entries created by empty columns

$lang_files = array();
foreach ($langs_in_csv as $lang)
{
    $lang_filename = $lang."_".$filename;
    $file_handle = fopen($lang_filename, 'w');

    // Write the UTF-8 BOM
    fwrite($file_handle, "\xEF\xBB\xBF");

    $lang_files[$lang] = array('filename' => $lang_filename, 'file_handle' => $file_handle);
}

// Now that we've checked the first row, close and re-open the file so that the
// first row get copied to the files like every other row.
fclose($fcsv);
$fcsv = fopen_utf8($filename, "r");

while ($entry = fgetcsv($fcsv))
{
    foreach (array_slice($entry, 2) as $i => $lang_entry)
    {
        $lang = $langs_in_csv[$i];
        $lang_file = $lang_files[$lang];

        $line = "";
        $line = add_item_to_line($line, $entry[0]);
        $line = add_item_to_line($line, $entry[1]);
        $line = add_item_to_line($line, $lang_entry);
        
        output_line($line, $lang_file);
    }
}

// Done. Close the files and report.
foreach ($langs_in_csv as $lang)
{
    fclose($lang_files[$lang]['file_handle']);
    print("Wrote: ".$lang_files[$lang]['filename']."\n");
}
print("\n");

exit(0);
?>
