#!/bin/sh

# verify parameters ############################################################

if [ $# != 1 ]; then
    echo "Usage: `basename $0` [destination folder]"
    exit 2
fi

# install dependencies #########################################################

apt-get install libpng12-dev libjpeg-dev openssl libssl-dev libxml2-dev \
    libxslt1-dev mysql-server mysql-client libmysqlclient15-dev libgeoip1 \
    realpath gettext

if [ $? != 0 ]; then
    exit 1
fi

# create destination folder ####################################################

if [ ! -d "${1}" ]; then
    mkdir "${1}"
    if [ $? != 0 ]; then
        exit 1
    fi
fi

# set absolute path to destination folder ######################################

dest=`realpath $1`

# install apache ##############################################################

if [ ! -f httpd-2.2.13.tar.gz ] ; then
    wget http://archive.apache.org/dist/httpd/httpd-2.2.13.tar.gz
    if [ $? != 0 ]; then
        exit 1
    fi
fi

tar xzf httpd-2.2.13.tar.gz

if [ $? != 0 ]; then
    exit 1
fi

cd httpd-2.2.13/srclib/apr/
./configure --prefix=${dest}/apache2
make clean
if [ $? != 0 ]; then
    exit 1
fi
make
if [ $? != 0 ]; then
    exit 1
fi
make install
if [ $? != 0 ]; then
    exit 1
fi

# pwd: httpd-2.2.13/srclib/apr/

cd ../apr-util
./configure --prefix=${dest}/apache2 --with-apr=${dest}/apache2 --with-mysql=/usr
make clean
if [ $? != 0 ]; then
    exit 1
fi
make
if [ $? != 0 ]; then
    exit 1
fi
make install
if [ $? != 0 ]; then
    exit 1
fi

# pwd: httpd-2.2.13/srclib/apr-util/

cd ../../

# pwd: httpd-2.2.13/

./configure --prefix=${dest}/apache2 \
--enable-mods-shared=all \
--enable-ssl \
--enable-unique-id  \
--enable-authz-host \
--enable-auth-basic \
--enable-include \
--enable-log-config \
--enable-env \
--enable-mime-magic \
--enable-expires \
--enable-headers \
--enable-setenvif \
--enable-mime \
--enable-status \
--enable-autoindex \
--enable-info \
--enable-vhost-alias \
--enable-dir \
--enable-alias \
--enable-dbd \
--enable-filter \
--libexecdir=${dest}/apache2/libexec/apache22 \
--with-apr=${dest}/apache2 \
--with-apr-util=${dest}/apache2

make clean
if [ $? != 0 ]; then
    exit 1
fi
make
if [ $? != 0 ]; then
    exit 1
fi
make install
if [ $? != 0 ]; then
    exit 1
fi

cd ..

# install php ##################################################################

if [ ! -f php-5.2.9.tar.gz ] ; then
    wget http://museum.php.net/php5/php-5.2.9.tar.gz 
    if [ $? != 0 ]; then
        exit 1
    fi
fi

tar xzf php-5.2.9.tar.gz
if [ $? != 0 ]; then
    exit 1
fi

cd php-5.2.9/

./configure \
--with-apxs2=${dest}/apache2/bin/apxs \
--with-mysql=/usr \
--with-mysqli \
--with-gettext \
--enable-pdo \
--with-pdo-mysql=/usr \
--prefix=${dest}/php \
--with-config-file-path=${dest}/php \
--with-jpeg-dir=/usr/include \
--with-gd \
--enable-mbstring \
--enable-debug \
--enable-maintainer-zts

make clean
if [ $? != 0 ]; then
    exit 1
fi
make
if [ $? != 0 ]; then
    exit 1
fi
make install
if [ $? != 0 ]; then
    exit 1
fi

cd ..


# preparing psiphon compile env ################################################

# set psiphon specific PATH to apxs

PATH=${dest}/apache2/bin:${PATH}


# setting include path for psiphon modules

if [ ! -d /usr/local/include/libxml2 ] ; then
    mkdir /usr/local 2> /dev/null
    mkdir /usr/local/include 2> /dev/null
    ln -s  /usr/include/libxml2 /usr/local/include/libxml2
fi

if [ ! -d /usr/local/include/libxml2 ] ; then
    mkdir /usr/local 2> /dev/null
    mkdir /usr/local/include 2> /dev/null
    ln -s /usr/include/libxslt /usr/local/include/libxslt
fi


# compile psiphon ##############################################################

cd mod_psiphon

make
if [ $? != 0 ]; then
    exit 1
fi

cd ..

# create ssl cert and key ######################################################

mkdir ${dest}/apache2/ssl

echo "####################################################"
echo "### Creating SSL certificate and key for Psiphon ###"
echo "####################################################"

openssl req -nodes -x509 -new -out ${dest}/apache2/ssl/psiphon2.crt -keyout ${dest}/apache2/ssl/psiphon2.key -days 365 -config tools/openssl.cnf -newkey rsa:2048
if [ $? != 0 ]; then 
    exit 1
fi 

# create apache user ###########################################################

groupadd ppwww

useradd -c "Apache Server" -d /nonexistent -g ppwww -s /bin/false ppwww

# create symlinks to xml libs ##################################################

ln -s /usr/lib/libxslt.so ${dest}/apache2/lib
ln -s /usr/lib/libxml2.so ${dest}/apache2/lib
ln -s /usr/lib/libexslt.so ${dest}/apache2/lib

# get mysql:psiphon password ###################################################

echo "Create mysql 'psiphon' user password,"
echo -n "Don't use either '/' or '\"' characters in the password:"

tty_mode=`stty -g`
stty -echo
read mysql_pass
stty ${tty_mode}
echo
export mysql_pass

# httpd.conf ###################################################################

cat mod_psiphon/httpd_conf/httpd.conf | \
sed "s%#ServerRoot#%ServerRoot \"${dest}/apache2\"%" | \
sed "s/#psiphon_mysql_password#/${mysql_pass}/" \
> ${dest}/apache2/conf/httpd.conf


#Get REVNO from bazaar if possible
bzr version-info --custom \
  --template="r{revno}" \
   > www/REVNO

# setup www ####################################################################

rm -r ${dest}/apache2/www
mkdir ${dest}/apache2/www
cp -R www/* ${dest}/apache2/www

# update config file with mysql:psiphon password ##############################

cat www/config.php | sed "s/#psiphon_mysql_password#/${mysql_pass}/" > ${dest}/apache2/www/config.php

# install database #############################################################

./tools/db_install.sh

unset mysql_pass

# create psiphon cron user #####################################################

mkdir ${dest}/ppcron /var/run/ppcron

chmod 700 ${dest}/ppcron /var/run/ppcron

useradd -c "Psiphon cron user" -d ${dest}/ppcron -g ppwww -s /bin/false ppcron

ln -s ${dest}/apache2/www/config.php ${dest}/ppcron
ln -s ${dest}/apache2/www/includes ${dest}/ppcron

cp cronjobs/*php ${dest}/ppcron

chown -Rh ppcron:ppwww ${dest}/ppcron /var/run/ppcron

# cronjobs #####################################################################

cat cronjobs/crontab | sed "s%#dest#%${dest}%g"| crontab -u ppcron -

# (re)start apache #############################################################

echo
echo "##################################################"
echo In order to start/stop psiphon 
echo refer to steps 2 through 5 of INSTALL
echo "##################################################"

