<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// anon.php - the dispatcher for anonymous browsing sessions
//
// expects an argument of the form token=9dc3a577a7246a0ef823b6dfb072f65e
//
// 1. lookup anonymous_browsing config table for matching token
// 2. create a new, anonymous user and session
// 3. redirect to home page
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/cr.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/geoip_helpers.php");

$query = "SELECT * FROM anonymous_browsing WHERE token = :token";
$result = db_query_execute($config, $query, convert_null_array(array(":token" => $_GET[token])));

$anonymous_record = db_fetch_result($config, $result);

if (!$anonymous_record)
{
    // sleep to slow down brute force attacks
    // no specific error for bad token, just HTTP 404
    sleep(1);
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}

// Set anonymous_record[lang] if a language was selected

if (strlen($_POST['anonymous_lang']) && array_key_exists($_POST['anonymous_lang'], $config['lang']))
{
    $anonymous_record['lang'] = $_POST['anonymous_lang'];
}

// token was valid, but first check cookie for a legitimate session
// before creating a new one

$need_new_session = 1;

if ($_COOKIE[p])
{
    $check_record = lookup_user($config, "session = :session_id", array(":session_id" => $_COOKIE[p]));

    if ($check_record)
    {
        $need_new_session = 0;
    }
}

// check accessing proxy matches proxy specified in anonymous browsing record


$query = "SELECT hostname FROM proxy WHERE id = :proxy_id";
$result = db_query_execute($config, $query, convert_null_array(array(":proxy_id" => $anonymous_record['proxy'])));

$proxy_record = db_fetch_result($config, $result);

if (!$proxy_record || $proxy_record[hostname] != getenv("HTTP_HOST"))
{
    // sleep to slow down brute force attacks
    // no specific error for bad token, just HTTP 404
    sleep(1);
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}

// a bit of a hack here: to avoid problems with spiders and monitors hitting
// known guest browsing links and creating sessions, we move the agree-to-terms
// here, before the session is created.  what this does is check if an "agreed"
// value is in the POST params; if not, present the terms dialog first and
// then re-run this form when the user clicks Agree.

if ($need_new_session && !$_POST['terms_submit_agree'])
{
    // the form action URL is simply the same guest browsing link again

    $url = make_anonymous_browsing_url($config, $anonymous_record['proxy'], $anonymous_record['token']);

    // make the terms render in the guest browsing link configured language

    $record_user['last_lang'] = $anonymous_record['lang'];

    // TODO: this presentation code is simply copy-pasted from check_user.php

    header("Content-type: text/html; charset=".charset());
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>    
    <head>
        <title><?=___("Welcome to Psiphon!")?></title>
        <?=echo_css()?>
    </head>

    <body style="direction: <?=direction()?>">
        <table style>
            <tr>
                <table><tr>
                    <td style="width:50%">
<?php
    include($_SERVER[DOCUMENT_ROOT]."/images/psiphonlogo.html");
?>
                    </td>
                    <td><font class="sixteenbluebold"><?=___("Welcome to Psiphon!")?></font></td>
                </tr></table>
            </tr>
            <tr>
                <td><div style="width:40em;"><font class="fourteenblue"><br><?=___("<p>Psiphon is a web-based proxy system that enables you to view
Internet content that might be inaccessible in your region.  To use
Psiphon: </p>
<ol>
 <li>Read and agree to <b>Psiphon's Terms of Service</b></li>
<br>
<li>Enter the address of the site you would like to visit into the
<b>Psiphon blue bar</b>, which will appear at the top of your browser
window.  Click <b>\"Go\"</b>.   (Repeat this for each new site you wish
to visit.)</li>
<br>
<li>To ensure you have consistent access to the Psiphon service, you
may wish to create a Psiphon account.  To do so, simply click the
<b>\"Create Account\"</b> link on the blue bar and follow the
instructions.</li>
</ol>
<p>Before using Psiphon, please read and agree to the Terms of Service.</p>
<p><i><b>Please note</b>, Psiphon is designed to provide you with open
access to online content.  Psiphon does not increase your online
privacy, and should not be considered or used as an online security
tool.</i></p>")?><br><br></font></div></td>
            </tr>
            <tr>
                <td><textarea class="blueback" name="terms" cols="80" rows="8" readonly><?=___("Psiphon Inc. - Terms of Use and Privacy Statement

Privacy

Psiphon collects information that you submit as part of the account creation process: a username (this should not be your real name) a password, and optionally, an e-mail address (which should be from a provider such as GMail that offers SSL and is hosted outside of your censoring country/region).

Psiphon does not store usage information in a manner that allows Psiphon to correlate an individual user’s Psiphon account or IP address with a visit to any individual web site. Psiphon records the date/time of account creation, the date/time and country/region of each user's last login. Psiphon also records the number of invitations a user sends as well as the number of invitations that are accepted.

Psiphon logs aggregate usage statistics on a per in-proxy as well as country/region basis. Psiphon collects domain names visited through the service, the number of visit per domain name, and the number of logins per year/month/day. Psiphon maintains server logs collected on the inproxy (which contain users' IP addresses and obfuscated requested URLs) and server (which contain the inproxy IP addreses and requested URLs) as well as sendmail logs that record the e-mail addresses of invitation recipients.

When necessary in order to troubleshoot its services, Psiphon may temporarily alter is standard practices in order to record usage information in a way that correlates a user’s account with the websites that he or she visits. This will only be done for a limited period of time and information retained in this way will be deleted after it has been analyzed.  Please read our Privacy Policy and Design Overview paper for more information.

Use of Psiphon

By accessing and invoking the Psiphon service, the user warrants that he / she will abide by the following terms of use:

• The user will not utilize Psiphon in ways that are prohibited or restricted by Canadian law including, but not limited to, accessing child pornography, engaging in fraud.

• The user will not utilize Psiphon in a manner that contravenes the right of other users to access the Psiphon service, by publicly disclosing in-proxy addresses and/or exploiting or otherwise over-loading Psiphon servers.

Acceptance of Risk

The user agrees that a degree of risk is associated with the use of any service provided by Psiphon Inc., and accepts that risk as a result of the usage of Psiphon. We strongly recommend that you read our Design Overview document to help you decide if Psiphon is right for you.

Psiphon is intended for non-technical users seeking access to denied content. Psiphon traffic is encrypted to avoid blocking but is unencrypted at the in-proxy level before it is re-encrypted and either passed on to the Psiphon server (for outgoing requests), or the end user (for returned requests).

Psiphon should not be construed as a way of enhancing the privacy or security of your Internet connection. Psiphon does not provide anonymity and does not provide protection from traffic analysis attacks. Psiphon does not disguise the fact that you are using Psiphon.

If you are engaged in politically sensitive behaviour and are at risk of high levels of surveillance, be aware that Psiphon does not have ability to defend you from such attacks.

Limitations of Liability

The user agrees that by utilizing Psiphon, he / she is using the service “as is” and accepts all consequences of doing so. Under no circumstances will Psiphon be held responsible for any way - for direct, indirect and consequential damages - by the user or any third party, even if Psiphon has been advised of the possibility that such damages may occur. Such damages include, but ARE NOT limited to: all personal damages such as reputation; lost data; lost business revenue; failure to realize expected profits or savings or other commercial or economic loss of any kind arising out of the use, or inability to use, the software.

Last updated: 2009-06-03")?></textarea></td>
            </tr>
            <br><br>
            <tr><td>
                <table>
                    <tr>
                        <td>
                            <form name="terms" method="post" action="<?=$url?>">
                                <input type="submit" value="<?=___("Agree")?>">
                                <input type="hidden" name="anonymous_lang" value="<?=$_POST["anonymous_lang"]?>"/>
                                <input name="terms_submit_agree" value="1" type="hidden">
                            </form>
                        </td>
                        <td>
                            <form name="terms" method="post" action="<?=$url?>">
                                <input type="submit" value="<?=___("Don't Agree")?>">
                            </form>
                        </td>
                    </tr>
                </table>
            </td></tr>

        </table>

        <br>
        <br>

        <?echo_language_buttons($config, $record_user, "anonymous_lang", 6);?>

    </body>
</html>
<?

    exit(0);
}

// create new anonymous user and session
//
// Anonyous sessions are implemented by creating temporary users.  This approach
// was chosen since it is most compatible with the existing architecture.  Resources
// for anonymous sessions are reclaimed by the cleanup_sessions_cron.php script.
// A new user class, Anonymous User (group 4) is assigned and the permissions for
// this class are even more limited than the normal User (group 3).

if ($need_new_session)
{
    // unguessable password

    $password = bin2hex(secure_rand());

    // unguessable session ID

    $session_id = bin2hex(secure_rand());

    // GeoIP lookup and stats update

    $login_region = lookup_region($config);
    $user_group = get_anonymous_user_class_id();
    update_login_stats($config, $login_region, $anonymous_record['proxy'], $user_group);

    // create temporary, guest user account

    $query = "INSERT INTO user (".
                "email,".
                "pass,".
                "uname,".
                "session,".
                "last_lang,".
                "last_reg,".
                "grp,".
                "current_proxy,".
                "home_page,".
                "last,".
                "last_activity,".
                "home_page_locked,".
                "agreed_to_usage_terms) ".
              "VALUES (".
                "NULL,".
                ":password,".
                "NULL,".
                ":session_id,".
                ":lang,".                            // lang from anonymous_browsing record
                ":login_region,".                    // region from GeoIP
                ":grp,".                             // "Anonymous User"
                ":current_proxy,".
                ":home_page,".                       // home_page from anonymous_browsing record
                "UNIX_TIMESTAMP(),".                 // last login = now
                "UNIX_TIMESTAMP(),".                 // last activity = now
                ":home_page_locked,".                // home_page_locked from anonymous_browsing record
                "1)";                                // NOTE: agree_to_terms set to 1 now that terms dialog is before session create

    db_query_execute(
        $config,
        $query,
        convert_null_array(array(
            ":password" => $password,
            ":session_id" => $session_id,
            ":lang" => $anonymous_record[lang],
            ":login_region" => $login_region,
            ":grp" => $user_group,
            ":current_proxy" => $anonymous_record['proxy'],
            ":home_page" => $anonymous_record[home_page],
            ":home_page_locked" => $anonymous_record[home_page_locked])));

    // Insert the user-proxy relation
    $query = "INSERT INTO userproxy (userid, proxyid) ".
                "VALUES ((SELECT LAST_INSERT_ID()), :proxyid)";
    $query_params = array(":proxyid" => $anonymous_record[proxy]);
    db_query_execute($config, $query, convert_null_array($query_params));

    // Set "p" cookie to new session ID

    header("Set-Cookie: p={$session_id}; path=/", false);
}

// redirect to home page
// e.g., https://127.0.0.1/001/

$url = make_proxied_url("");
header("HTTP/1.0 302 Found");
header("Location: ". $url);
exit(0);
?>
