<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// summary_report.php
//
// This screen is used to generate proxy summary reports.
// NOTE: This file is not included in the open source package. 
//

include_once($_SERVER['DOCUMENT_ROOT']."/includes/common_includes.php");
//include($_SERVER['DOCUMENT_ROOT']."/admin/lang_admin.php");
include($_SERVER['DOCUMENT_ROOT']."/check_user.php");
include($_SERVER['DOCUMENT_ROOT']."/includes/lang_set.php");

$header_link_prefix = "../";

$blank_region_name = "??";
$by_date_types = array("Day" => "%Y-%m-%d", "Week" => "%Y:%u", "Month" => "%Y-%m", "Year" => "%Y");

//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "reports"))
{
    // We use 404 instead of 403 so as to not leak the existence of this file.
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
    // 404.php exits
}

function create_stat_query(
            $graph_by_interval,
            $table_name,
            $where_clause,
            $date_from_clause,
            $date_to_clause)
{
    global $by_date_types;

    // Don't use any real column names for the result field names.
    $query =
        "SELECT DATE_FORMAT(SeqDate, '{$by_date_types[$graph_by_interval]}') AS _date, SUM(COALESCE(count, 0)) AS _count, COALESCE(region, '') AS _region, COALESCE(grp, 0) AS _grp FROM ".
        date_friendly_table(
            $table_name,
            "__date",
            "SELECT FROM_UNIXTIME(date, '%Y-%m-%d') AS __date, SUM(cnt) AS count, region, grp FROM {$table_name} ".
                (strlen($where_clause) ? " WHERE ".$where_clause : "")." ".
                "GROUP BY (grp = ".get_anonymous_user_class_id()."), region, __date",
            "(SELECT FROM_UNIXTIME(MIN(date), '%Y-%m-%d') FROM {$table_name} ".(strlen($date_from_clause) ? " WHERE ".$date_from_clause : "").")",
            "(SELECT FROM_UNIXTIME(MAX(date), '%Y-%m-%d') FROM {$table_name} ".(strlen($date_to_clause) ? " WHERE ".$date_to_clause : "").")"
            )." ".
        "GROUP BY (_grp = ".get_anonymous_user_class_id()."), _region, _date ORDER BY _date ASC, _region ASC ";

    return $query;
}

$report_results = array();
$report_results['regions'] = array();

function collate_results($query_result, $result_type)
{
    global $report_results;
    global $blank_region_name;
    global $config;

    $report_results['total'][$result_type]['guests']['count'] = 0;
    $report_results['total'][$result_type]['users']['count'] = 0;
    while ($record = escape_html(db_fetch_result($config, $query_result)))
    {
        $user_type = ($record['_grp'] == get_anonymous_user_class_id()) ? 'guests' : 'users';
        $region = strlen($record['_region']) ? $record['_region'] : $blank_region_name;
        $date = strlen($record['_date']) ? $record['_date'] : "";

        $report_results['total'][$result_type][$user_type]['count'] += $record['_count'];

        $report_results['total'][$result_type]['guests']['by_date'][$date] += ($user_type == 'guests' ? $record['_count'] : 0);
        $report_results['total'][$result_type]['users']['by_date'][$date] += ($user_type == 'users' ? $record['_count'] : 0);

        $report_results['regions'][strtoupper($region)][$result_type][$user_type]['count'] += $record['_count'];
    }
}

// process filter form

while ($_GET['filter_submit'] == "1")
{
    $filter_proxy = cast_str_to_int($_GET['filter_proxy']);
    $filter_keywords = (is_null($filter_proxy) && (strlen($_GET['filter_keywords'])) ? trim($_GET['filter_keywords']) : Null);
    $filter_date_from = (strlen($_GET['filter_date_from']) ? $_GET['filter_date_from'] : Null);
    $filter_date_to = (strlen($_GET['filter_date_to']) ? $_GET['filter_date_to'] : Null);
    $graph_by_interval = (strlen($_GET['graph_by_interval']) ? $_GET['graph_by_interval'] : Null);

    if (is_null($graph_by_interval) || !isset($by_date_types[$graph_by_interval]))
    {
        $graph_by_interval = "Day";
    }

    //
    // Create the WHERE clauses that will be used by all queries below.
    //

    $common_where_clause = "";
    $common_where_clause_params = array();
    $common_date_to_clause = "";
    $common_date_from_clause = "";

    // The proxy selection takes precedence over the keywords.
    if (!is_null($filter_proxy))
    {
        $common_where_clause = "(proxy = :filter_proxy)";
        $common_where_clause_params[':filter_proxy'] = $filter_proxy;
    }
    else if (!is_null($filter_keywords))
    {
        $keywords = explode(" ", $filter_keywords); // note trim() above
        foreach ($keywords as $index => $keyword)
        {
            if (strlen($common_where_clause) > 0)
            {
                $common_where_clause .= " AND ";
            }
            
            $common_where_clause .= "(UPPER(proxy_notes) LIKE UPPER(:filter_keywords{$index}))";
            $common_where_clause_params[":filter_keywords{$index}"] = "%".$keyword."%";
        }
    }

    if (!is_null($filter_date_from))
    {
        if (strlen($common_where_clause) > 0)
        {
            $common_where_clause .= " AND ";
        }

        $common_date_from_clause = "(date >= UNIX_TIMESTAMP(DATE(:filter_date_from)))";
        $common_where_clause .= $common_date_from_clause;
        $common_where_clause_params[':filter_date_from'] = $filter_date_from;
    }

    if (!is_null($filter_date_to))
    {
        if (strlen($common_where_clause) > 0)
        {
            $common_where_clause .= " AND ";
        }

        $common_date_to_clause = "(date < UNIX_TIMESTAMP(DATE(:filter_date_to)))";
        $common_where_clause .= $common_date_to_clause;
        $common_where_clause_params[':filter_date_to'] = $filter_date_to;
    }

    //
    // Accounts created
    //

    // Don't use any real column names for the result field names.
    $query = create_stat_query(
                $graph_by_interval,
                "stat_account_created",
                $common_where_clause,
                $common_date_from_clause,
                $common_date_to_clause);

    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));
    
    collate_results($result, "accounts_created");

    //
    // Login counts
    //

    $query = create_stat_query(
                $graph_by_interval,
                "stat_region",
                $common_where_clause,
                $common_date_from_clause,
                $common_date_to_clause);
            
    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    collate_results($result, "logins");

    //
    // Request counts
    //

    $query = create_stat_query(
                $graph_by_interval,
                "stat_site",
                $common_where_clause,
                $common_date_from_clause,
                $common_date_to_clause);

    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    collate_results($result, "requests");

    //
    // Top Sites
    //

    /*
     * We will loop through the regions and user types, doing a separate query
     * for each combination. It's possible that this could be done in a single
     * query, but my efforts have only led to slow, unreadable SQL.
     * See: http://www.xaprb.com/blog/2006/12/07/how-to-select-the-firstleastmax-row-per-group-in-sql/
     */

    $regions = array_keys($report_results['regions']);

    foreach (array('guests', 'users') as $user_type)
    {
        foreach ($regions as $region)
        {
            $query_region = (($region == $blank_region_name) ? "" : $region);

            $query =
                "SELECT SUM(cnt) AS count, region, grp, domain ".
                "FROM stat_site ".
                "WHERE (region = :region) ".
                    "AND (grp".(($user_type == 'guests') ? " = " : " <> ").get_anonymous_user_class_id().") ".
                    (strlen($common_where_clause) > 0 ? "AND ".$common_where_clause." " : "").
                    "GROUP BY (grp = ".get_anonymous_user_class_id()."), region, domain ".
                "ORDER BY region ASC, count DESC ".
                "LIMIT 3";

            $result = db_query_execute($config, $query, convert_null_array(array(':region' => $query_region) + $common_where_clause_params));

            while ($record = escape_html(db_fetch_result($config, $result)))
            {
                $user_type = ($record['grp'] == get_anonymous_user_class_id()) ? 'guests' : 'users';

                $report_results['regions'][strtoupper($region)]['top_sites'][$user_type][] = array('domain' => $record['domain'], 'count' => $record['count']);
            }
        }

        // Now get the per-user-type totals.

        $query =
            "SELECT SUM(cnt) AS count, grp, domain ".
            "FROM stat_site ".
            "WHERE (grp".(($user_type == 'guests') ? " = " : " <> ").get_anonymous_user_class_id().") ".
                (strlen($common_where_clause) > 0 ? "AND ".$common_where_clause." " : "").
            "GROUP BY (grp = ".get_anonymous_user_class_id()."), domain ".
            "ORDER BY count DESC ".
            "LIMIT 3";

        $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

        while ($record = escape_html(db_fetch_result($config, $result)))
        {
            $user_type = ($record['grp'] == get_anonymous_user_class_id()) ? 'guests' : 'users';

            $report_results['total']['top_sites'][$user_type][] = array('domain' => $record['domain'], 'count' => $record['count']);
        }
    }

    //
    // Convert the region codes to full names and sort by those names.
    //

    $expanded_region_code_results = array();
    $unknown_region_name = "";
    
    function expand_region_code_callback($region_data, $region_code)
    {
        global $expanded_region_code_results;
        global $blank_region_name;
        global $unknown_region_name;
        
        $region_name = get_country_name_from_country_code($region_code)." (".$region_code.")";

        // Make sure BLANK sorts to the bottom.
        if ($region_code == $blank_region_name)
        {
            $unknown_region_name = $region_name;
        }

        $expanded_region_code_results[$region_name] = $region_data;
    }

    function region_name_sort_callback($region_name1, $region_name2)
    {
        global $unknown_region_name;
        if ($region_name1 == $unknown_region_name)
        {
            return 1;
        }
        else if ($region_name2 == $unknown_region_name)
        {
            return -1;
        }

        return strcasecmp($region_name1, $region_name2);
    }

    array_walk($report_results['regions'], "expand_region_code_callback");
    uksort($expanded_region_code_results, "region_name_sort_callback");
    $report_results['regions'] = $expanded_region_code_results;

    //
    // Get the list of proxies that were used in the stats queries.
    //

    $proxy_where_clause = (strlen($common_where_clause) ? "WHERE ".$common_where_clause : "");

    $query =
        "SELECT id, name, hostname, notes, ".proxy_label_sql("proxy")." AS label FROM proxy WHERE id IN (".
            "SELECT DISTINCT(proxy) FROM stat_region ".$proxy_where_clause.
            " UNION ".
            "SELECT DISTINCT(proxy) FROM stat_site ".$proxy_where_clause.
            " UNION ".
            "SELECT DISTINCT(proxy) FROM stat_account_created ".$proxy_where_clause.")".
            " ORDER BY id ASC";

    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    $proxies = array();
    while ($record = escape_html(db_fetch_result($config, $result)))
    {
        $proxies[] = $record;
    }

    // DEBUG
    //echo "<pre>";print_r($query);exit();

    // Make the results safe for outputting.
    $report_results = escape_html($report_results);

    break;
}

// Output will be indicated table plus rows for missing days.
// The column with all the days is called SeqDate and has the format %Y-%m-%d
// SEE EXISTING USES FOR EXAMPLES.
function date_friendly_table(
            $table_name,
            $date_column_name,
            $grouped_query,
            $min_date,  // may be SQL expression or literal string
            $max_date)  // may be SQL expression or literal string
{
    return "
    (
        SELECT *
        FROM ({$grouped_query}) AS grouped_results
        RIGHT OUTER JOIN
        (
        SELECT
            SEQ.SeqDate
        FROM
        (
        SELECT
            DATE_ADD({$min_date}, INTERVAL (TEN_THOUSANDS.SeqValue + THOUSANDS.SeqValue + HUNDREDS.SeqValue + TENS.SeqValue + ONES.SeqValue) DAY) AS SeqDate
        FROM
            (SELECT 0  SeqValue UNION ALL SELECT 1 SeqValue UNION ALL SELECT 2 SeqValue UNION ALL SELECT 3 SeqValue UNION ALL SELECT 4 SeqValue UNION ALL SELECT 5 SeqValue UNION ALL SELECT 6 SeqValue UNION ALL SELECT 7 SeqValue UNION ALL SELECT 8 SeqValue UNION ALL SELECT 9 SeqValue)
                AS ONES
        CROSS JOIN
            (SELECT 0 SeqValue UNION ALL SELECT 10 SeqValue UNION ALL SELECT 20 SeqValue UNION ALL SELECT 30 SeqValue UNION ALL SELECT 40 SeqValue UNION ALL SELECT 50 SeqValue UNION ALL SELECT 60 SeqValue UNION ALL SELECT 70 SeqValue UNION ALL SELECT 80 SeqValue UNION ALL SELECT 90 SeqValue)
                AS TENS
        CROSS JOIN
            (SELECT 0 SeqValue UNION ALL SELECT 100 SeqValue UNION ALL SELECT 200 SeqValue UNION ALL SELECT 300 SeqValue UNION ALL SELECT 400 SeqValue UNION ALL SELECT 500 SeqValue UNION ALL SELECT 600 SeqValue UNION ALL SELECT 700 SeqValue UNION ALL SELECT 800 SeqValue UNION ALL SELECT 900 SeqValue)
                AS HUNDREDS
        CROSS JOIN
            (SELECT 0 SeqValue UNION ALL SELECT 1000 SeqValue UNION ALL SELECT 2000 SeqValue UNION ALL SELECT 3000 SeqValue UNION ALL SELECT 4000 SeqValue UNION ALL SELECT 5000 SeqValue UNION ALL SELECT 6000 SeqValue UNION ALL SELECT 7000 SeqValue UNION ALL SELECT 8000 SeqValue UNION ALL SELECT 9000 SeqValue)
                AS THOUSANDS
        CROSS JOIN
            (SELECT 0 SeqValue UNION ALL SELECT 10000 SeqValue UNION ALL SELECT 20000 SeqValue UNION ALL SELECT 30000 SeqValue UNION ALL SELECT 40000 SeqValue UNION ALL SELECT 50000 SeqValue UNION ALL SELECT 60000 SeqValue UNION ALL SELECT 70000 SeqValue UNION ALL SELECT 80000 SeqValue UNION ALL SELECT 90000 SeqValue)
                AS TEN_THOUSANDS
        WHERE DATE_ADD({$min_date}, INTERVAL (TEN_THOUSANDS.SeqValue + THOUSANDS.SeqValue + HUNDREDS.SeqValue + TENS.SeqValue + ONES.SeqValue) DAY) <= {$max_date}
        ) AS SEQ
        ) AS SeqValues
        ON grouped_results.{$date_column_name} = SeqValues.SeqDate
    ) AS {$table_name}
    ";
}

// Called from array_reduce below.
function make_top_sites_display_string($display_string, $site_info)
{
    if (!strlen($display_string))
    {
        $display_string .= "<ul>";
    }

    $display_string .= "<li>" . $site_info['domain']."(".$site_info['count'].")</li>";

    return $display_string;
}

function format_results_section($section_title, $results)
{
return "
    <tr>
        <td rowspan=2 class=\"page_break_avoid\">$section_title</td>
        <td>Guests</td>
        <td>" . $results['accounts_created']['guests']['count'] . "</td>
        <td>" . $results['logins']['guests']['count'] . "</td>
        <td>" . $results['requests']['guests']['count']. "</td>
        <td>" . (isset($results['top_sites']['guests']) ? array_reduce($results['top_sites']['guests'], 'make_top_sites_display_string') : '') . "</td>
    </tr>
    <tr>
        <td>Users</td>
        <td>" . $results['accounts_created']['users']['count'] . "</td>
        <td>" . $results['logins']['users']['count'] . "</td>
        <td>" . $results['requests']['users']['count']. "</td>
        <td>" . (isset($results['top_sites']['users']) ? array_reduce($results['top_sites']['users'], 'make_top_sites_display_string') : '') . "</td>
    </tr>
";
}

// Screen title. Must come before the header.php include.
$title = 'Proxy Summary Report';

// include menu side-bar
include($_SERVER['DOCUMENT_ROOT'].'/header.php');

?>

<?/*
Output forms.
Hidden '*_submit' fields are used to determine when/which form is submitted.
*/?>

<?/* The filtering form */?>
<form name='filter' method='get' action='<?=getenv('SCRIPT_NAME')?>'>
    <?/* Because the method for this form is GET (and the effects are read-only),
         we don't use the anti-CRSF field.
        <input name='csrf_token' value='<?=$record_user['csrf_token']?>' type='hidden'>*/?>
    <input name='filter_submit' value='1' type='hidden'>

    <table class='outline vertspace'>
        <tr class='nowrap'>
        <ul>
        <li>
        This reports summarizes statistics for selected proxies and date range.
        </li>
        <li>
        The filters are optional.  By default, all proxies are selected.
        </li>
        <li>
        If a specific proxy is selected, the keywords are ignored.
        </li>
        <li>
        Use the keywords to select proxies by words that appear in their "notes" field.  All keywords must match.
        </li>
        </ul>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Proxy: </th>
            <td align='left'><?echo_proxy_combo($config, 'filter_proxy', $filter_proxy, 1)?></td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Keywords: </th>
            <td align='left'><input type=text size='50' name='filter_keywords' value='<?=escape_html($filter_keywords)?>'/></td>
        </tr>

        <tr class='nowrap'>
            <th align='right'>Date From:</th>
            <td align='left'><input type=text name='filter_date_from' value='<?=escape_html($filter_date_from)?>'/> YYYY-MM-DD or blank</td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Date To:</th>
            <td align='left'><input type=text name='filter_date_to' value='<?=escape_html($filter_date_to)?>'/> YYYY-MM-DD or blank</td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Graph by:</th>
            <td align='left'>
                <select name='graph_by_interval'>
                <?foreach ($by_date_types as $key => $value)
                {
                    echo
                        "<option value='{$key}' ".(($graph_by_interval == $key) ? "selected" : "").">{$key}</option>";

                }
                ?>
                </select>
            </td>
        </tr>

        <tr>
            <td><input type='submit' value='Submit'></td>
            <!--uncomment to get the box the full width of the screen <td></td><td width='90%'></td>-->
        </tr>

    </table>

</form>
<br/>

<?if ($_GET['filter_submit'] == '1') { ?>

    <?/* Output results menu */?>

    <table class='outline vertspace' id ="print_hide">
        <tr class='nowrap'>
        <td><a href="#results_selected_filter">Selected Filter</a> </td>
        <td><a href="#results_charts">Charts</a> </td>
        <td><a href="#results_statistics">Statistics</a> </td>
        <td><a href="#results_selected_proxies">Selected Proxies</a> </td>
        </tr>
    </table>

<script type="text/javascript" src="../js/ondomload.js"></script>
<script type="text/javascript">
// From http://www.randomsnippets.com/2008/02/12/how-to-hide-and-show-your-div/
function toggle2(showHideDiv, switchTextDiv, showHidePrintDiv)
{
    var ele = document.getElementById(showHideDiv);
    var text = document.getElementById(switchTextDiv);
    var print_ele = document.getElementById(showHidePrintDiv);

    if(ele.style.display == "block")
    {
        ele.style.display = "none";
        text.innerHTML = "+";
        print_ele.className = "print_hide";
    }
    else
    {
        ele.style.display = "block";
        text.innerHTML = "-";
        print_ele.className = "";
    }
}

function toggle_all_results()
{
    toggle2('selected_filter_block','toggle_selected_filter', 'results_selected_header');
    toggle2('charts_block','toggle_charts', 'charts_header');
    toggle2('statistics_block','toggle_statistics', 'statistics_header');
    toggle2('selected_proxies_block','toggle_selected_proxies', 'selected_proxies_header');
}

// This call has to come after the graphs being drawn (they need to be visible).
//run_on_DOM_load(toggle_all_results);
</script>

    <?/* Output filter summary */?>

    <br/>
    <span id="results_selected_header">
        <h2><a name="results_selected_filter"></a><a class=print_hide style="font-family:monospace;" id="toggle_selected_filter" href="javascript:toggle2('selected_filter_block','toggle_selected_filter', 'results_selected_header');">-</a> Selected Filter</h2>
    </span>
    <div id="selected_filter_block" style="display:block;">
    <table class='brd'>
        <tr class='nowrap'>
            <th align='right' width="5%">Proxies: </th>
            <td>
            <?
            if (is_null($filter_proxy) && is_null($filter_keywords))
            {
                echo "All";
            }
            else if (!is_null($filter_proxy))
            {
                echo escape_html($proxies[0]['label']);
            }
            else
            {
                echo "Matching keywords: ".escape_html(str_replace(" ", ", ", $filter_keywords));
            }
            ?>
            </td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Date From: </th><td><?=is_null($filter_date_from) ? "First event" : escape_html($filter_date_from)?></td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Date To: </th><td><?=is_null($filter_date_to) ? "Current time" : escape_html($filter_date_to)?></td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Graph by: </th><td><?=escape_html($graph_by_interval)?></td>
        </tr>
    </table>
    </div>

    <?/* Output chart(s) */?>

    <br/>
    <span id="charts_header">
        <h2><a name="results_charts"></a><a class=print_hide style="font-family:monospace;" id="toggle_charts" href="javascript:toggle2('charts_block','toggle_charts', 'charts_header');">-</a> Charts</h2>
    </span>

    <div id="charts_block" style="display:block;">

    <?
        //
        // Format the data for the javascript charting code.
        //

        if (count($report_results['total']['logins']['users']['by_date']) != count($report_results['total']['logins']['guests']['by_date'])
            || count($report_results['total']['requests']['users']['by_date']) != count($report_results['total']['requests']['guests']['by_date']))
        {
            // Those arrays have to be the same size.
            echo "<br>ARRAYS DON'T MATCH<br>";
            echo "<pre>";
            echo count($report_results['total']['logins']['users']['by_date']).
                "//".count($report_results['total']['logins']['guests']['by_date']).
                "//".count($report_results['total']['requests']['users']['by_date']).
                "//".count($report_results['total']['requests']['guests']['by_date']);
            echo "</pre><br>";

            include($_SERVER['DOCUMENT_ROOT']."/http-errors/503.php");
        }

        $js_logins_dates = "";
        $js_requests_dates = "";
        $js_logins_by_date_for_all_regions_users = "";
        $js_logins_by_date_for_all_regions_guests = "";
        $js_requests_by_date_for_all_regions_users = "";
        $js_requests_by_date_for_all_regions_guests = "";
        $i = 0;
        $logins_by_date_for_all_regions_empty = true;
        foreach ($report_results['total']['logins']['users']['by_date'] as $date => $count)
        {
            $js_logins_dates .= "\"$date\", ";
            $js_logins_by_date_for_all_regions_users .= "[$i, {$report_results['total']['logins']['users']['by_date'][$date]}], ";
            $js_logins_by_date_for_all_regions_guests .= "[$i, {$report_results['total']['logins']['guests']['by_date'][$date]}], ";
            if ($count > 0) $logins_by_date_for_all_regions_empty = false;
            $i++;
        }

        $i = 0;
        $requests_by_date_for_all_regions_empty = true;
        foreach ($report_results['total']['requests']['users']['by_date'] as $date => $count)
        {
            $js_requests_dates .= "\"$date\", ";
            $js_requests_by_date_for_all_regions_users .= "[$i, {$report_results['total']['requests']['users']['by_date'][$date]}], ";
            $js_requests_by_date_for_all_regions_guests .= "[$i, {$report_results['total']['requests']['guests']['by_date'][$date]}], ";
            if ($count > 0) $requests_by_date_for_all_regions_empty = false;
            $i++;
        }

        $js_logins_dates = rtrim($js_logins_dates, ", ");
        $js_requests_dates = rtrim($js_requests_dates, ", ");
        $js_logins_by_date_for_all_regions_users = rtrim($js_logins_by_date_for_all_regions_users, ", ");
        $js_logins_by_date_for_all_regions_guests = rtrim($js_logins_by_date_for_all_regions_guests, ", ");
        $js_requests_by_date_for_all_regions_users = rtrim($js_requests_by_date_for_all_regions_users, ", ");
        $js_requests_by_date_for_all_regions_guests = rtrim($js_requests_by_date_for_all_regions_guests, ", ");

        function js_pie_chart_helper($report_results, $stat, $user_class)
        {
            // build pie chart series from results array

            $serie = array();

            // make simple region => stat value array, then sort it by stat value
            // also count the total of all values use to calculate the "other" category below

            $total = 0;
            $data_array = array();
            foreach ($report_results['regions'] as $region => $region_results)
            {
                $value = $region_results[$stat][$user_class]['count'];
                if ($value > 0)
                {
                    $total += $value;
                    $data_array[$region] = $value;
                }
            }
            asort($data_array);

            if (count($data_array) == 0)
            {
                return "";
            }

            // take top 5 items only and add "other" category if more remain

            $data_array = array_splice(array_reverse($data_array), 0, 5);
            $top5_total = 0;
            foreach ($data_array as $region => $value)
            {
                $top5_total += $value;
                $serie[$region]= $value;
            }

            $serie["All other regions"] = $total - $top5_total;

            $pie_js = "";
            foreach ($serie as $label => $count)
            {
                $pie_js .= "{data:[[0, $count]], label: '$label'}, ";
            }

            return $pie_js;
        }

    ?>

    <!--[if IE]><script language="javascript" type="text/javascript" src="js/flotr/lib/excanvas.js"></script><![endif]-->
    <script language="javascript" type="text/javascript" src="js/flotr/lib/prototype-1.6.0.2.js"></script>
    <script language="javascript" type="text/javascript" src="js/flotr/lib/base64.js"></script>
    <script language="javascript" type="text/javascript" src="js/flotr/lib/canvas2image.js"></script>
    <script language="javascript" type="text/javascript" src="js/flotr/lib/canvastext.js"></script>
    <script language="javascript" type="text/javascript" src="js/flotr/flotr.js"></script>

    <script language="javascript">

    var logins_date_array = new Array(<?=$js_logins_dates?>);
    var requests_date_array = new Array(<?=$js_requests_dates?>);

    function draw_graphs()
    {
        <?if (!$logins_by_date_for_all_regions_empty){?>
        Flotr.draw(
                $('logins_by_date_for_all_regions'),
                [{ 
                    label: "Guests",
                    data: [ <?=$js_logins_by_date_for_all_regions_guests?> ],
                    mouse:{trackFormatter: function (obj) {return "Guests: "+logins_date_array[obj.x]+": "+obj.y;}}
                 },
                 {
                    label: "Users",
                    data: [ <?=$js_logins_by_date_for_all_regions_users?> ],
                    mouse:{trackFormatter: function (obj) {return "Users: "+logins_date_array[obj.x]+": "+obj.y;}}
                 }],
                {
                    title: "Logins by date for all regions",
                    lines: {show: true, fill: true},
                    points: {show: true},
                    xaxis:{
                        noTicks: 10,
                        tickDecimals: 0,
                        tickFormatter: function (val) {return logins_date_array[val];}
                    },
                    yaxis:{
                        autoscaleMargin: 1,
                        noTicks: 10,
                        tickDecimals: 0,
                        min: 0
                    },
                    legend: {position: "ne", backgroundColor: "#EEEEEE"},
                    grid: {labelMargin: 5, outlineWidth: 1},
                    mouse:{
                        track: true,
                        trackDecimals: 0
                    }
                }
        );
        <?}else{?>
        document.getElementById("logins_by_date_for_all_regions").innerHTML = "Logins by date for all regions: no results";
        document.getElementById("logins_by_date_for_all_regions").setStyle("height:20px;");
        <?}?>

        <?if (!$requests_by_date_for_all_regions_empty){?>
        Flotr.draw(
                $('requests_by_date_for_all_regions'),
                [{
                    label: "Guests",
                    data: [ <?=$js_requests_by_date_for_all_regions_guests?> ],
                    mouse:{trackFormatter: function (obj) {return "Guests: "+requests_date_array[obj.x]+": "+obj.y;}}
                 },
                 {
                    label: "Users",
                    data: [ <?=$js_requests_by_date_for_all_regions_users?> ],
                    mouse:{trackFormatter: function (obj) {return "Users: "+requests_date_array[obj.x]+": "+obj.y;}}
                 }],
                {
                    title: "Requests by date for all regions",
                    lines: {show: true, fill: true},
                    points: {show: true},
                    xaxis:{
                        noTicks: 10,
                        tickDecimals: 0,
                        tickFormatter: function (val) {return requests_date_array[val];}
                    },
                    yaxis:{
                        autoscaleMargin: 1,
                        noTicks: 10,
                        tickDecimals: 0,
                        min: 0
                    },
                    legend: {position: "ne", backgroundColor: "#EEEEEE"},
                    grid: {labelMargin: 5, outlineWidth: 1},
                    mouse:{
                        track: true,
                        trackDecimals: 0
                    }
                });
        <?}else{?>
        document.getElementById("requests_by_date_for_all_regions").innerHTML = "Requests by date for all regions: no results";
        document.getElementById("requests_by_date_for_all_regions").setStyle("height:20px;");
        <?}?>

        <?
        $stats = js_pie_chart_helper($report_results, 'logins', 'guests');        
        if (strlen($stats)){?>
        Flotr.draw(
                $('guest_logins_by_region'),
                [ <?=$stats?> ],
                {
                    title: "Guest logins by region",
                    HtmlText: false,
                    grid: {
                        verticalLines: false,
                        horizontalLines: false,
                        outlineWidth: 1
                    },
                    xaxis: {showLabels: false},
                    yaxis: {showLabels: false},
                    pie:{
                        show: true,
                        explode: 15,
                        startAngle: 1,
                        labelFormatter: function (slice) {return slice.name+/*"<br>"*/" "+(slice.fraction * 100).toFixed(0) + "%";}},
                    legend:{
                        position: 'se',
                        backgroundColor: '#EEEEEE' }
               });
        <?}else{?>
        document.getElementById("guest_logins_by_region").innerHTML = "Guest logins by region: no results";
        document.getElementById("guest_logins_by_region").setStyle("height:20px;");
        <?}?>

        <?
        $stats = js_pie_chart_helper($report_results, 'logins', 'users');
        if (strlen($stats)){?>
        Flotr.draw(
                $('user_logins_by_region'),
                [ <?=$stats?> ],
                {
                    title: "User logins by region",
                    HtmlText: false,
                    grid: {
                        verticalLines: false,
                        horizontalLines: false,
                        outlineWidth: 1
                    },
                    xaxis: {showLabels: false},
                    yaxis: {showLabels: false},
                    pie:{
                        show: true,
                        explode: 15,
                        startAngle: 1,
                        labelFormatter: function (slice) {return slice.name+/*"<br>"*/" "+(slice.fraction * 100).toFixed(0) + "%";}},
                    legend:{
                        position: 'se',
                        backgroundColor: '#EEEEEE' }
                });
        <?}else{?>
        document.getElementById("user_logins_by_region").innerHTML = "User logins by region: no results";
        document.getElementById("user_logins_by_region").setStyle("height:20px;");
        <?}?>

        <?
        $stats = js_pie_chart_helper($report_results, 'accounts_created', 'users');
        if (strlen($stats)){?>
        Flotr.draw(
                $('user_accounts_created_by_region'),
                [ <?=$stats?> ],
                {
                    title: "User accounts created by region",
                    HtmlText: false,
                    grid: {
                        verticalLines: false,
                        horizontalLines: false,
                        outlineWidth: 1
                    },
                    xaxis: {showLabels: false},
                    yaxis: {showLabels: false},
                    pie:{
                        show: true,
                        explode: 15,
                        startAngle: 1,
                        labelFormatter: function (slice) {return slice.name+/*"<br>"*/" "+(slice.fraction * 100).toFixed(0) + "%";}},
                    legend:{
                        position: 'se',
                        backgroundColor: '#EEEEEE' }
                });
        <?}else{?>
        document.getElementById("user_accounts_created_by_region").innerHTML = "User accounts created by region: no results";
        document.getElementById("user_accounts_created_by_region").setStyle("height:20px;");
        <?}?>
    }

    run_on_DOM_load(draw_graphs);
    run_on_DOM_load(toggle_all_results);

    </script>

    <table style="width: 820px;">
        <tr><td><div id="logins_by_date_for_all_regions" style="height:300px;"></div></td></tr>
    </table>
    <br/>
    <table style="width: 820px;">
        <tr><td><div id="requests_by_date_for_all_regions" style="height:300px;"></div></td></tr>
    </table>
    <br/>
    <table style="width: 820px;">
        <tr><td><div id="guest_logins_by_region" style="height:400px;"></div></td></tr>
    </table>
    <br/>
    <table style="width: 820px;">
        <tr><td><div id="user_logins_by_region" style="height:400px;"></div></td></tr>
    </table>
    <br/>
    <table style="width: 820px;">
        <tr><td><div id="user_accounts_created_by_region" style="height:400px;"></div></td></tr>
    </table>
    </div>

    <?/* Output results table */?>

    <br/>
    <span id="statistics_header">
        <h2><a name="results_statistics"></a><a class=print_hide style="font-family:monospace;" id="toggle_statistics" href="javascript:toggle2('statistics_block','toggle_statistics', 'statistics_header');">-</a> Statistics</h2>
    </span>
    <div id="statistics_block" style="display:block;">
    <table class="brd">
        <thead>
        <tr style="background-color:DarkGray">
            <th>Region</th>
            <th>Type</th>
            <th>Accounts created</th>
            <th>Logins</th>
            <th>Requests</th>
            <th>Top Sites</th>
        </tr>
        </thead>
        <tbody>
            <?
            // Output the totals section.
            echo format_results_section("All Regions", $report_results['total']);

            // Output the regional sections.
            foreach ($report_results['regions'] as $region => $region_results)
            {
                echo format_results_section($region, $region_results);
            }
            ?>
        </tbody>
    </table>
    </div>

    <?/* Output proxies table */?>

    <br/>
    <span id="selected_proxies_header">
        <h2><a name="results_selected_proxies"></a><a class=print_hide style="font-family:monospace;" id="toggle_selected_proxies" href="javascript:toggle2('selected_proxies_block','toggle_selected_proxies', 'selected_proxies_header');">-</a> Selected Proxies</h2>
    </span>
    <div id="selected_proxies_block" style="display:block;">
    <table class='brd'>
    <thead>
        <tr style='background-color:DarkGray'>
            <th>Name</th>
            <th>Hostname</th>
            <th>Notes</th>
        </tr>
    </thead>
    <tbody>

        <?
        // Output the regional sections.
        foreach ($proxies as $proxy)
        {
            echo
                "<tr>".
                    "<td>{$proxy['name']}</td>".
                    "<td>{$proxy['hostname']}</td>".
                    "<td><pre>{$proxy['notes']}</pre></td>".
                "</tr>";
        }
        ?>
    </tbody>
    </table>
    </div>

<?}?>

<?
// output footer (e.g., copyright)
include($_SERVER['DOCUMENT_ROOT'].'/footer.php');
?>
