<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009-2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// transition.php
//
// This screen is used to move users between proxies. It provides filtering,
// mass moving, and option emailing of a link to the new proxy.
// NOTE: This file is not included in the open source package.
// Transitioning users involves the following:
//  - Move normal users and power users from one proxy to another.
//  - If any power users were moved, add the new proxy to any Propagators that
//    are associated with the source proxy.
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
//include($_SERVER[DOCUMENT_ROOT]."/admin/lang_admin.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

$header_link_prefix = "../";

//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "accounts"))
{
    // We use 404 instead of 403 so as to not leak the existence of this file.
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
    // 404.php exits
}

$_POST['filter_proxy'] = cast_str_to_int($_POST['filter_proxy']);
$_POST['filter_created_date_interval'] = cast_str_to_int($_POST['filter_created_date_interval']);
$_POST['filter_created_date_within'] = cast_str_to_int($_POST['filter_created_date_within']);
$_POST['filter_created_date_units'] = cast_str_to_int($_POST['filter_created_date_units']);
$_POST['filter_last_login_date_interval'] = cast_str_to_int($_POST['filter_last_login_date_interval']);
$_POST['filter_last_login_date_within'] = cast_str_to_int($_POST['filter_last_login_date_within']);
$_POST['filter_last_login_date_units'] = cast_str_to_int($_POST['filter_last_login_date_units']);
$_POST['transition_send_email'] = $_POST['transition_submitted'] ? $_POST['transition_send_email'] : "on";
$_POST['transition_normal_users_with_email'] = cast_str_to_int($_POST['transition_normal_users_with_email']);
$_POST['transition_normal_users_without_email'] = cast_str_to_int($_POST['transition_normal_users_without_email']);
$_POST['transition_power_users_with_email'] = cast_str_to_int($_POST['transition_power_users_with_email']);
$_POST['transition_power_users_without_email'] = cast_str_to_int($_POST['transition_power_users_without_email']);
$_POST['transition_new_proxy'] = cast_str_to_int($_POST['transition_new_proxy']);

//
// Transitioning and filtering use the same WHERE clause, so create it first.
//

$filter_where = "";
$filter_params = array();
$filter_error = false;

// Get the user info based on the filter

$filters = array();

// User class
// Only normal Users and Power Users
$filters[] = "(user.grp = :normal_user_group OR user.grp = :power_user_grp)";
$filter_params[':normal_user_group'] = get_normal_user_class_id();
$filter_params[':power_user_grp'] = get_power_user_class_id();

// Associated proxy
if ($_POST['filter_proxy'])
{
    $filters[] = "user.current_proxy = :proxyid";
    $filter_params[':proxyid'] = $_POST['filter_proxy'];
}

// Last region
if ($_POST['filter_last_region'])
{
    $filters[] = "user.last_reg = :user_last_reg";
    $filter_params[':user_last_reg'] = $_POST['filter_last_region'];
}

//
// Last login and Created
// 
// "within" will use >= and "not within" will use <. This seems like the most
// intuitive way to ensure that exact matches are not completely excluded.
// We're using coalesce so that users with null create-dates/last-logins will
// show up in the not-within results (otherwise they would show up in neither).
// We recognize that this means strange results will occur if the number of days
// is very large (e.g., null last-login will appear in results for "last login
// within 4 billion days") There's no security risk in this.
if ($_POST['filter_created_date_interval'] !== Null)
{
    $multiplier = $_POST['filter_created_date_units'] ? (24*60*60) : (60*60);

    $filters[] =
        " COALESCE(user.created, 0) ".
        ($_POST['filter_created_date_within'] ? ">=" : "<").
        " (UNIX_TIMESTAMP() - :created_time) ";
    $filter_params[':created_time'] = ($_POST['filter_created_date_interval'] * $multiplier);
}

if ($_POST['filter_last_login_date_interval'] !== Null)
{
    $multiplier = $_POST['filter_last_login_date_units'] ? (24*60*60) : (60*60);

    $filters[] =
        " COALESCE(user.last, 0) ".
        ($_POST['filter_last_login_date_within'] ? ">=" : "<").
        " (UNIX_TIMESTAMP() - :login_time) ";
    $filter_params[':login_time'] = ($_POST['filter_last_login_date_interval'] * $multiplier);
}

// Email suffixes to exclude

if (strlen($_POST['filter_email_suffix']))
{
    $suffixes = explode(' ', $_POST['filter_email_suffix']);
    $i = 0;
    $suffix_filters = array();
    $suffix_filter_params = array();
    
    foreach ($suffixes as $suffix)
    {
        if (!strlen(trim($suffix)))
        {
            $processing_errors[] = "Invalid email suffixes";
            $curs = "transition.filter_email_suffix";
            unset ($_POST['transition_submitted']);
            $filter_error = true;
            break;
        }

        $i++;
        $var_name = ':suffix'.$i;
        $suffix_filters[] = " (user.email NOT LIKE $var_name OR email IS NULL) ";
        $suffix_filter_params[$var_name] = '%'.$suffix;
    }
    
    if (!$filter_error && ($i == 0))
    {
        // Suffix field was non-empty, but no suffixes were extracted
        $processing_errors[] = "Invalid email suffixes";
        $curs = "transition.filter_email_suffix";
        unset ($_POST['transition_submitted']);
        $filter_error = true;
    }
    else
    {
        $filters = array_merge($filters, $suffix_filters);
        $filter_params = array_merge($filter_params, $suffix_filter_params);
    }
}

if (count($filters) > 0)
{
    $filter_where .= join(" AND ",$filters);
}
$filter_where .= " ";

//
// Process transition form
//

// Does the transitioning steps.
// $select_query must select (at least) the user.id, user.email, and user.last_lang columns.
// Returns True on success, False on error
function do_user_transition(
            $select_query,
            $filter_params,
            $max_transition_count,
            $destination_proxy,
            $send_email,
            $new_proxy_url,
            &$transition_count)
{
    global $config;
    global $processing_errors;

    $transition_count = 0;

    if (!$max_transition_count)
    {
        return True;
    }

    $select_result = db_query_execute($config, $select_query, convert_null_array($filter_params));

    while ($select_record = db_fetch_result($config, $select_result))
    {
        $transition_count++;

        // The userproxy table effectively keeps a history of all proxies that 
        // user has had access to -- and is allowed to access. The user.current_proxy
        // column keeps track of the proxy that the user has most recently accessed.

        // Note: There is no transaction surrounding these DB ops. Unfortunately.

        $query_params = array(':userid'=>$select_record['id'], ':proxyid'=>$destination_proxy);

        // Update the users's current_proxy value.
        $update_query = "UPDATE user SET current_proxy = :proxyid WHERE id = :userid ";
        db_query_execute($config, $update_query, convert_null_array($query_params));

        // Insert the new user-proxy relation. Note that we're using INSERT IGNORE
        // because the entry may already exist.
        $insert_query = "INSERT IGNORE INTO userproxy (userid, proxyid) VALUES (:userid, :proxyid) ";
        db_query_execute($config, $insert_query, convert_null_array($query_params));

        // Send the transition email to the user. (Note: Unlike the Propagator
        // transtion function below, we send the email even if the user already
        // had this userproxy entry.)

        if ($send_email)
        {
            $user_email = $select_record['email'];
            $user_lang = $select_record['last_lang'];
            $username = $select_record['uname'];

            // Check that the user has a valid email address.

            if ( strlen($user_email) > 0 && !is_email($user_email) )
            {
                // Not a valid address. We won't return an error, but we will
                // note it to the user.
                $processing_errors[] = "Selected User does not have a valid e-mail address: ".$username;
            }
            else // email is valid
            {
                // If the target user doesn't have a last_lang, use the current user's.

                if (!$user_lang)
                {
                    $user_lang = $record_user['last_lang'];
                }

                // generate the email subject and body

                $subject = ___("Your new Psiphon proxy", NULL, $user_lang);
                $body = sprintf(___("Here is your new Psiphon proxy URL:
%s

Please begin using this URL to log into Psiphon.", NULL, $user_lang), $new_proxy_url);

                // enqueue email

                enqueue_email($config, $user_email, $subject, $body, $user_lang);
            }
        }

        if ($transition_count >= $max_transition_count)
        {
            // We're done.
            break;
        }
    }

    // We may not have retrieved all of the results, so we should explicitly free the result set.
    db_free_result($select_result);

    return True;
}

function do_propagator_transition(
            $source_proxy,
            $destination_proxy,
            $send_email,
            $new_proxy_url,
            &$transition_count)
{
    global $config;
    global $processing_errors;

    $transition_count = 0;

    $select_query =
        "SELECT DISTINCT id, uname, email, last_lang ".
        "FROM user LEFT OUTER JOIN userproxy ON user.id = userproxy.userid ".
        "WHERE user.grp = ".get_propagator_user_class_id()." ".
            "AND userproxy.proxyid = :proxyid";

    $select_result = db_query_execute(
                        $config,
                        $select_query,
                        convert_null_array(array(':proxyid'=>$source_proxy)));

    while ($select_record = db_fetch_result($config, $select_result))
    {
        $transition_count++;

        $query_params = array(':userid'=>$select_record['id'], ':proxyid'=>$destination_proxy);

        // Update the users's current_proxy value. Note that this is not strictly
        // necessary, but we'll do it for consistency.
        $update_query = "UPDATE user SET current_proxy = :proxyid WHERE id = :userid ";
        db_query_execute($config, $update_query, convert_null_array($query_params));

        // Insert the new user-proxy relation. Note that we're using INSERT IGNORE because
        // the entry may already exist.
        $insert_query = "INSERT IGNORE INTO userproxy (userid, proxyid) VALUES (:userid, :proxyid) ";
        $result = db_query_execute($config, $insert_query, convert_null_array($query_params));

        // Send the transition email to the user. (Note that we won't bother
        // sending an email if this isn't a new proxy for the Propagator.)

        if ($send_email && db_num_rows_affected($result) > 0)
        {
            $user_email = $select_record['email'];
            $user_lang = $select_record['last_lang'];
            $username = $select_record['uname'];

            // Check that the user has a valid email address.

            if ( strlen($user_email) > 0 && !is_email($user_email) )
            {
                // Not a valid address. We won't return an error, but we will
                // note it to the user.
                $processing_errors[] = "Selected Propagator does not have a valid e-mail address: ".$username;
            }
            else // email is valid
            {
                // If the target user doesn't have a last_lang, use the current user's.

                if ( !$user_lang )
                {
                    $user_lang = $record_user['last_lang'];
                }

                // generate the email subject and body
                // TODO: create custom email subject+body for propagators
                $subject = ___("Your new Psiphon proxy", NULL, $user_lang);
                $body = sprintf(___("Here is your new Psiphon proxy URL:
%s

Please begin using this URL to log into Psiphon.", NULL, $user_lang), $new_proxy_url);

                // enqueue email

                enqueue_email($config, $user_email, $subject, $body, $user_lang);
            }
        }
    }

    return True;
}

$transition_counts = array(
                        'normal_users_with_email' => 0,
                        'normal_users_without_email' => 0,
                        'power_users_with_email' => 0,
                        'power_users_without_email' => 0,
                        'propagators' => 0);
$source_proxy_label = "";
$destination_proxy_label = "";

while ($_POST['transition_submitted'] && !$filter_error)
{
    // Input validation

    if (!$_POST['filter_proxy'])
    {
        $processing_errors[] = "A target proxy must be selected (you can't transition all users at once)";
        $curs = "transition.filter_proxy";
        break;
    }

    if (!$_POST['transition_new_proxy'])
    {
        $processing_errors[] = "A destination proxy must be selected (the users have to go somewhere)";
        $curs = "transition.transition_new_proxy";
        break;
    }

    if (!$_POST['transition_normal_users_with_email']
        && !$_POST['transition_normal_users_without_email']
        && !$_POST['transition_power_users_with_email']
        && !$_POST['transition_power_users_without_email'])
    {
        $processing_errors[] = "No target users were indicated to move";
        $curs = "transition.transition_normal_users_with_email";
        break;
    }

    // Check source proxy selected and get label

    $query = "SELECT ".proxy_label_sql('proxy')." AS label FROM proxy WHERE id = :id";
    $result = db_query_execute($config, $query, convert_null_array(array(":id" => $_POST['filter_proxy'])));

    if (!($record = escape_html(db_fetch_result($config, $result))) || !$record['label'])
    {
        $processing_errors[] = "Error: no proxy selected";
        $curs = "transition.transition_new_proxy";
        break;
    }

    $source_proxy_label = $record['label'];

    // Check destination proxy selected and get address and label

    $query = "SELECT hostname, ".proxy_label_sql('proxy')." AS label FROM proxy WHERE id = :id";
    $result = db_query_execute($config, $query, convert_null_array(array(":id" => $_POST['transition_new_proxy'])));

    if (!($record = escape_html(db_fetch_result($config, $result))) || !$record['hostname'] || !$record['label'])
    {
        $processing_errors[] = "Error: no proxy selected";
        $curs = "transition.transition_new_proxy";
        break;
    }

    $destination_proxy_label = $record['label'];

    // If not sending an email, display the resulting proxy URL.
    $proxy_url = make_home_url($config, $_POST['transition_new_proxy']);

    //
    // Transition the users one by one.
    // Partly this is to compensate for the fact that we don't have transactions.
    //

    $base_query =
        "SELECT DISTINCT id, uname, email, last_lang ".
        "FROM user ".
        "WHERE $filter_where ";

    $transition_counts = array(
                            'normal_users_with_email' => 0,
                            'normal_users_without_email' => 0,
                            'power_users_with_email' => 0,
                            'power_users_without_email' => 0,
                            'propagators' => 0);

    // 
    // Add the new Proxy to any Propagators associated with the filter proxy
    //
    // Only transition Propagators if a Power User was also transitioned (or, at
    // least, if the user intended to transition a Power User).
    if ($_POST['transition_power_users_with_email'] > 0
        || $_POST['transition_power_users_without_email'] > 0)
    {
        if (!do_propagator_transition(
                $_POST['filter_proxy'],
                $_POST['transition_new_proxy'],
                $_POST['transition_send_email'] ? True : False,
                $proxy_url,
                $transition_counts['propagators']))
        {
            break;
        }
    }

    // Transition Normal Users with email address

    $select_query = 
        $base_query." ".
        "AND user.grp = ".get_normal_user_class_id()." ".
        "AND user.email IS NOT NULL ";

    if (!do_user_transition(
            $select_query,
            $filter_params,
            $_POST['transition_normal_users_with_email'],
            $_POST['transition_new_proxy'],
            $_POST['transition_send_email'] ? True : False,
            $proxy_url,
            $transition_counts['normal_users_with_email']))
    {
        break;
    }
    
    // Transition Normal Users without email address

    $select_query =
        $base_query." ".
        "AND user.grp = ".get_normal_user_class_id()." ".
        "AND user.email IS NULL ";

    if (!do_user_transition(
            $select_query,
            $filter_params,
            $_POST['transition_normal_users_without_email'],
            $_POST['transition_new_proxy'],
            False, // users don't have email address
            $proxy_url,
            $transition_counts['normal_users_without_email']))
    {
        break;
    }

    // Transition Power Users with email address

    $select_query =
        $base_query." ".
        "AND user.grp = ".get_power_user_class_id()." ".
        "AND user.email IS NOT NULL ";

    if (!do_user_transition(
            $select_query,
            $filter_params,
            $_POST['transition_power_users_with_email'],
            $_POST['transition_new_proxy'],
            $_POST['transition_send_email'] ? True : False,
            $proxy_url,
            $transition_counts['power_users_with_email']))
    {
        break;
    }

    // Transition Power Users without email address

    $select_query =
        $base_query." ".
        "AND user.grp = ".get_power_user_class_id()." ".
        "AND user.email IS NULL ";

    if (!do_user_transition(
            $select_query,
            $filter_params,
            $_POST['transition_power_users_without_email'],
            $_POST['transition_new_proxy'],
            False, // users don't have email address
            $proxy_url,
            $transition_counts['power_users_without_email']))
    {
        break;
    }

    $transition_success = True;

    // Change the header title to reflect the changes.
    $processing_results[] = "Users transitioned to proxy ".$proxy_url;

    // on success, set the filter to show the new proxy selected, not the old proxy
    // this shows the transitioned users in their new proxy

    $filter_proxy = $_POST['transition_new_proxy'];

    break;
}

//
// Do the filtering (this always happens)
//

if (!$filter_error)
{
    // The filtering should only exclude Propagators based on the proxy (not created date, etc.)
    $propagator_where =
        " user.grp = :propagator_grp ".
        ($_POST['filter_proxy'] ? "AND userproxy.proxyid = :proxyid" : "");
    $filter_params[':propagator_grp'] = get_propagator_user_class_id();


    $query = "SELECT grp, COUNT(DISTINCT uname) AS user_count, COUNT(DISTINCT email) AS email_count ".
             "FROM user LEFT OUTER JOIN userproxy ON user.id = userproxy.userid ".
             "WHERE ($filter_where) OR ($propagator_where) ".
             "GROUP BY grp ";

    $result = db_query_execute($config, $query, convert_null_array($filter_params));

    $user_counts = array('total'=>0, 'with_email'=>0, 'without_email'=>0);
    $user_counts[get_normal_user_class_id()] = array('total'=>0, 'with_email'=>0, 'without_email'=>0);
    $user_counts[get_power_user_class_id()] = array('total'=>0, 'with_email'=>0, 'without_email'=>0);
    $user_counts[get_propagator_user_class_id()] = array('total'=>0, 'with_email'=>0, 'without_email'=>0);
    
    while ($record = db_fetch_result($config, $result))
    {
        $user_counts[$record['grp']] = array(
                'total' => $record['user_count'],
                'with_email' => $record['email_count'],
                'without_email' => ($record['user_count'] - $record['email_count']));

        $user_counts['total'] += $user_counts[$record['grp']]['total'];
        $user_counts['with_email'] += $user_counts[$record['grp']]['with_email'];
        $user_counts['without_email'] += $user_counts[$record['grp']]['without_email'];
    }
}

// Screen title. Must come before the header.php include.
$title = "User-Proxy Transitioning";

// include menu side-bar
include($_SERVER[DOCUMENT_ROOT]."/header.php");

?>

<?/* Summary of the transition */?>
<? if ($transition_success) { ?>
    <div id="summary">
        <p style="border:1px solid black;">
            <b style="font-size: large;">Transition Summary</b><br/>
            <b>Transition</b> from <b><?=$source_proxy_label?></b> to <b><?=$destination_proxy_label?></b><br/>
            <b>Normal Users transitioned: </b><?=($transition_counts['normal_users_with_email']+$transition_counts['normal_users_without_email'])?><br/>
            <b>Power Users transitioned: </b><?=($transition_counts['power_users_with_email']+$transition_counts['power_users_without_email'])?><br/>
            <b>Propagators transitioned: </b><?=$transition_counts['propagators']?><br/>
            <b>Total transitioned: </b><?=($transition_counts['normal_users_with_email']+$transition_counts['normal_users_without_email']+$transition_counts['power_users_with_email']+$transition_counts['power_users_without_email']+$transition_counts['propagators'])?><br/>
            <b>Total not transitioned: </b><?=($user_counts[get_normal_user_class_id()]['with_email']+$user_counts[get_normal_user_class_id()]['without_email']+$user_counts[get_power_user_class_id()]['with_email']+$user_counts[get_power_user_class_id()]['without_email'])?>
        </p>
    </div>
<?}?>

<?/*
Output forms.
Hidden "*_submit" fields are used to determine which form is submitted.
*/?>

<?/* The filtering/transitioning form */?>
<form name="transition" method="post" action="<?=getenv("SCRIPT_NAME")?>">
    <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden">
    <input name="submitted" value="1" type="hidden">

    <table>
        <tr class="nowrap"><td>
                
            <?/* Filter form controls */?>
            <table class="outline vertspace">
                <tr>
                    <th style="font-size: large;" align="left">Select target users</th>
                </tr>
                <tr>
                    <th align="right">*Source Proxy</th>
                    <td>
                        <?=echo_proxy_combo(
                                $config,
                                "filter_proxy",
                                $_POST['filter_proxy'],
                                1)?>
                    </td>
                </tr>
                <tr>
                    <th align="right">Last Region</th>
                    <td>
                        <?=echo_country_combo(
                                $config,
                                "filter_last_region",
                                $_POST['filter_last_region'])?>
                    </td>
                </tr>
                <tr>
                    <th align="right">Created</th>
                    <td>
                        <?echo_combo_from_array(
                                "filter_created_date_within",
                                array("1" => "within", "0" => "not within"),
                                Null,
                                ($_POST['filter_created_date_within'] === Null) ?
                                    "1" :
                                    $_POST['filter_created_date_within'],
                                0)?>
                        <input type="text" name="filter_created_date_interval"
                               maxlength="3" size="3"
                               value="<?=escape_html($_POST['filter_created_date_interval'])?>"/>
                        <?echo_combo_from_array(
                                "filter_created_date_units",
                                array("1" => "days", "0" => "hours"),
                                Null,
                                ($_POST['filter_created_date_units'] === Null) ?
                                    "1" :
                                    $_POST['filter_created_date_units'],
                                0)?>
                    </td>
                </tr>
                <tr>
                    <th align="right">Last Login</th>
                    <td>
                        <?echo_combo_from_array(
                                "filter_last_login_date_within",
                                array("1" => "within", "0" => "not within"),
                                Null,
                                ($_POST['filter_last_login_date_within'] === Null) ?
                                    "1" :
                                    $_POST['filter_last_login_date_within'],
                                0)?>
                        <input type="text" name="filter_last_login_date_interval"
                               maxlength="3" size="3"
                               value="<?=escape_html($_POST['filter_last_login_date_interval'])?>"/>
                        <?echo_combo_from_array(
                                "filter_last_login_date_units",
                                array("1" => "days", "0" => "hours"),
                                Null,
                                ($_POST['filter_last_login_date_units'] === Null) ?
                                    "1" :
                                    $_POST['filter_last_login_date_units'],
                                0)?>
                    </td>
                </tr>
                <tr>
                    <th align="right">Email address suffixes to exclude</th>
                    <td>
                        <input type="text" name="filter_email_suffix" value="<?=escape_html($_POST['filter_email_suffix'])?>"/>
                        (space-separated: e.g. ".cn .ir yahoo.co.uk")
                    </td>
                </tr>
                <tr>
                    <td align="left"><input type="submit" name="filter_submitted" value="Filter"></td>
                </tr>
            </table>

        </td></tr>

        <tr><td>&nbsp;</td></tr>

        <?/* The transitioning controls */?>

        <tr class="nowrap"><td>

            <script type="text/javascript">
                function fill_with_totals(cells)
                {
                    for (var i = 0; i < cells.length; i++)
                    {
                        var total = document.getElementById(cells[i]+'_total').innerHTML;

                        document.getElementById(cells[i]+'_target').getElementsByTagName('input')[0].value = total;
                    }
                }
            </script>

            <table class="outline vertspace">

                <tr><td>

                    <table class="outline vertspace brd" style="width:auto">
                        <tr>
                            <th style="font-size: large;" align="left">*Target user counts</th>
                            <th colspan="2">With E-mail <a href="javascript:fill_with_totals(['normal_with', 'power_with'])">+</a></th>
                            <th colspan="2">Without E-mail <a href="javascript:fill_with_totals(['normal_without', 'power_without'])">+</a></th>
                        </tr>
                        <tr>
                            <th><a href="javascript:fill_with_totals(['normal_with','normal_without', 'power_with', 'power_without'])">all+</a></th>
                            <th>Total</th><th>Target</th>
                            <th>Total</th><th>Target</th>
                        </tr>
                        <tr>
                            <th align="right"><?=get_user_class_name(get_normal_user_class_id())?> <a href="javascript:fill_with_totals(['normal_with','normal_without'])">+</a></th>
                            <td id="normal_with_total" align="right"><?=$user_counts[get_normal_user_class_id()]['with_email']?></td>
                            <td id="normal_with_target" align="left">
                                <input type="text" name="transition_normal_users_with_email"
                                       maxlength="6" size="6"
                                       value="<?=$_POST['transition_normal_users_with_email'] && $user_counts[get_normal_user_class_id()]['with_email']
                                                  ? escape_html($_POST['transition_normal_users_with_email'])
                                                  : 0?>"
                                       <?=!$user_counts[get_normal_user_class_id()]['with_email'] || !$_POST['filter_proxy'] ? "readonly" : ""?>/>
                            </td>
                            <td id="normal_without_total" align="right"><?=$user_counts[get_normal_user_class_id()]['without_email']?></td>
                            <td id="normal_without_target" align="left">
                                <input type="text" name="transition_normal_users_without_email"
                                       maxlength="6" size="6"
                                       value="<?=$_POST['transition_normal_users_without_email'] && $user_counts[get_normal_user_class_id()]['without_email']
                                                  ? escape_html($_POST['transition_normal_users_without_email'])
                                                  : 0?>"
                                       <?=!$user_counts[get_normal_user_class_id()]['without_email'] || !$_POST['filter_proxy'] ? "readonly" : ""?>/>
                            </td>
                        </tr>
                        <tr>
                            <th align="right"><?=get_user_class_name(get_power_user_class_id())?> <a href="javascript:fill_with_totals(['power_with', 'power_without'])">+</a></th>
                            <td id="power_with_total" align="right"><?=$user_counts[get_power_user_class_id()]['with_email']?></td>
                            <td id="power_with_target" align="left">
                                <input type="text" name="transition_power_users_with_email"
                                       maxlength="6" size="6"
                                       value="<?=$_POST['transition_power_users_with_email'] && $user_counts[get_power_user_class_id()]['with_email']
                                                  ? escape_html($_POST['transition_power_users_with_email'])
                                                  : 0?>"
                                       <?=!$user_counts[get_power_user_class_id()]['with_email'] || !$_POST['filter_proxy'] ? "readonly" : ""?>/>

                            </td>
                            <td id="power_without_total" align="right"><?=$user_counts[get_power_user_class_id()]['without_email']?></td>
                            <td id="power_without_target" align="left">
                                <input type="text" name="transition_power_users_without_email"
                                       maxlength="6" size="6"
                                       value="<?=$_POST['transition_power_users_without_email'] && $user_counts[get_power_user_class_id()]['without_email']
                                                  ? escape_html($_POST['transition_power_users_without_email'])
                                                  : 0?>"
                                       <?=!$user_counts[get_power_user_class_id()]['without_email'] || !$_POST['filter_proxy'] ? "readonly" : ""?>/>
                            </td>
                        </tr>
                        <tr>
                            <th align="right"><?=get_user_class_name(get_propagator_user_class_id())?></th>
                            <td align="right"><?=$user_counts[get_propagator_user_class_id()]['with_email']?></td>
                            <td align="left">auto</td>
                            <td align="right"><?=$user_counts[get_propagator_user_class_id()]['without_email']?></td>
                            <td align="left">auto</td>
                        </tr>
                    </table>

                </td></tr>

                <tr><td>
                    <table class="outline vertspace">
                        <tr>
                            <th style="font-size: large;"  align="right">*Destination Proxy</th>
                            <td>
                                <?/* Populate new proxy combo box. Don't restore proxy selection after successful operation. */?>
                                <?=echo_proxy_combo(
                                        $config,
                                        "transition_new_proxy",
                                        $_POST['transition_new_proxy'],
                                        1)?>
                            </td>
                            <td valign="middle">
                                <input type="checkbox" name="transition_send_email"
                                        <?=($_POST['transition_send_email'] ? "checked=1" : "")?> >
                                Send E-mail
                            </td>
                        </tr>
                    </table>
                </td></tr>

                <tr>
                    <td>
                        <input type="submit" name="transition_submitted" value="Transition Target Users"
                               <?=!$_POST['filter_proxy'] ? 'disabled="disabled"' : ''?>>
                    </td>
                </tr>

            </table>

        </td></tr>

        <tr><td>&nbsp;</td></tr>

    </table>
</form>

<?
// output footer (e.g., copyright)
include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>

