<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// usage_report.php
//
// This screen is used to generate the numbers required for Usage reports.
// NOTE: This file is not included in the open source package. 
//

include_once($_SERVER['DOCUMENT_ROOT']."/includes/common_includes.php");
//include($_SERVER['DOCUMENT_ROOT']."/admin/lang_admin.php");
include($_SERVER['DOCUMENT_ROOT']."/check_user.php");
include($_SERVER['DOCUMENT_ROOT']."/includes/lang_set.php");

$header_link_prefix = "../";

$target_regions = array('BH', 'CN', 'CU', 'ET', 'IR', 'MM', 'SA', 'SY', 'TN', 'TM', 'UZ', 'VN', 'YE');

$blank_region_name = "??";

$all_days_in_range = array();
$all_months_in_range = array();
$peak_day_logins_results_html = "";
$average_day_logins_results_html = "";
$total_monthly_logins_results_html = "";
$total_monthly_requests_results_html = "";


//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "reports"))
{
    // We use 404 instead of 403 so as to not leak the existence of this file.
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
    // 404.php exits
}

// process filter form

while ($_GET['filter_submit'] == "1")
{
    $filter_date_from = (strlen($_GET['filter_date_from']) ? trim($_GET['filter_date_from']) : Null);
    $filter_date_to = (strlen($_GET['filter_date_to']) ? trim($_GET['filter_date_to']) : Null);
    
    if (strlen($filter_date_from))
    {
        $filter_date_from = substr($filter_date_from, 0, strlen('YYYY-MM'));
    }

    if (strlen($filter_date_to))
    {
        $filter_date_to = substr($filter_date_to, 0, strlen('YYYY-MM'));
    }

    //
    // Create the WHERE clauses that will be used by all queries below.
    //

    $common_where_clause = "";
    $common_where_clause_params = array();
    $common_date_to_clause = "";
    $common_date_from_clause = "";

    // Limit the regions to query about.
    $region_limit_from_clause = " (region IN ('".join("','", $target_regions)."')) ";
    $common_where_clause .= $region_limit_from_clause;
    
    if (!is_null($filter_date_from))
    {
        if (strlen($common_where_clause) > 0)
        {
            $common_where_clause .= " AND ";
        }

        $common_date_from_clause = "FROM_UNIXTIME(date, '%Y-%m') >= :filter_date_from";
        $common_where_clause .= $common_date_from_clause;
        $common_where_clause_params[':filter_date_from'] = $filter_date_from;
    }

    if (!is_null($filter_date_to))
    {
        if (strlen($common_where_clause) > 0)
        {
            $common_where_clause .= " AND ";
        }

        $common_date_to_clause = "FROM_UNIXTIME(date, '%Y-%m') <= :filter_date_to";
        $common_where_clause .= $common_date_to_clause;
        $common_where_clause_params[':filter_date_to'] = $filter_date_to;
    }
    
    // An array of all dates in the desired range will be needed later.
    $all_days_in_range =
        get_seq_dates(
            $config,
            "(SELECT FROM_UNIXTIME(MIN(date), '%Y-%m-%d') FROM stat_region ".(strlen($common_date_from_clause) ? " WHERE ".$common_date_from_clause : "").")",
            "(SELECT FROM_UNIXTIME(MAX(date), '%Y-%m-%d') FROM stat_region ".(strlen($common_date_to_clause) ? " WHERE ".$common_date_to_clause : "").")",
            strlen($common_date_from_clause) ? $common_where_clause_params : array(),
            false);
    $all_months_in_range =
        get_seq_dates(
            $config,
            "(SELECT FROM_UNIXTIME(MIN(date), '%Y-%m-%d') FROM stat_region ".(strlen($common_date_from_clause) ? " WHERE ".$common_date_from_clause : "").")",
            "(SELECT FROM_UNIXTIME(MAX(date), '%Y-%m-%d') FROM stat_region ".(strlen($common_date_to_clause) ? " WHERE ".$common_date_to_clause : "").")",
            strlen($common_date_from_clause) ? $common_where_clause_params : array(),
            true);

    //
    // "Peak Day Psiphon Users per month by Country"
    // "(The number of unique, cookie-based Psiphon sessions logged on whichever
    // day, during a particular month, had the greatest number of sessions)"
    //

    $query =
        "SELECT MAX(count) AS count, DATE_FORMAT(day, '%Y-%m') AS month, region FROM (".
            "SELECT ".
                "FROM_UNIXTIME(date, '%Y-%m-%d') AS day, ".
                "SUM(COALESCE(cnt, 0)) AS count, ".
                "region ".
            "FROM stat_region ".
            (strlen($common_where_clause) ? "WHERE ".$common_where_clause : "")." ".
            "GROUP BY day, region ".
        ") AS daily_sums ".
        "GROUP BY region, month";
        
    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    process_peak_day_logins_results($result);

    //
    // "Average Daily Psiphon Users per month by Country"
    // "(The average, throughout a particular month, of the number of unique
    // cookie-based Psiphon sessions logged in a single day)"
    //
    
    $query =
        "SELECT ".
            "FROM_UNIXTIME(date, '%Y-%m-%d') AS day, ".
            "SUM(COALESCE(cnt, 0)) AS count, ".
            "region ".
        "FROM stat_region ".
        (strlen($common_where_clause) ? "WHERE ".$common_where_clause : "")." ".
        "GROUP BY day, region ";
        
    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    process_average_day_logins_results($result);

    //
    // "Total Monthly Psiphon Users by Country"
    // "(The total number of unique cookie-based Psiphon sessions logged during
    // a particular month)"
    //
    
    $query =
        "SELECT SUM(count) AS count, DATE_FORMAT(day, '%Y-%m') AS month, region FROM (".
            "SELECT ".
                "FROM_UNIXTIME(date, '%Y-%m-%d') AS day, ".
                "SUM(COALESCE(cnt, 0)) AS count, ".
                "region ".
            "FROM stat_region ".
            (strlen($common_where_clause) ? "WHERE ".$common_where_clause : "")." ".
            "GROUP BY day, region ".
        ") AS daily_sums ".
        "GROUP BY region, month";
        
    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    process_total_monthly_logins_results($result);

    //
    // "Total Monthly Psiphon Usage by Country"
    // "(The total number of page-views served through Psiphon during a
    // particular month)"
    //
    
    $query =
        "SELECT SUM(count) AS count, DATE_FORMAT(day, '%Y-%m') AS month, region FROM (".
            "SELECT ".
                "FROM_UNIXTIME(date, '%Y-%m-%d') AS day, ".
                "SUM(COALESCE(cnt, 0)) AS count, ".
                "region ".
            "FROM stat_site ".
            (strlen($common_where_clause) ? "WHERE ".$common_where_clause : "")." ".
            "GROUP BY day, region ".
        ") AS daily_sums ".
        "GROUP BY region, month";
        
    $result = db_query_execute($config, $query, convert_null_array($common_where_clause_params));

    process_total_monthly_requests_results($result);

    break;
}


function process_peak_day_logins_results($result)
{
    global $peak_day_logins_results_html;
    global $target_regions;
    global $all_months_in_range;
    
    $results = array();
    
    while ($record = db_fetch_result($config, $result))
    {
        if (!strlen($record['region']))
            $record['region'] = $blank_region_name;
            
        if (!in_array($record['region'], $target_regions))
        {
            // Not a target region.
            continue;
        }
        
        $record = escape_html($record);
            
        if (!array_key_exists($record['region'], $results))
            $results[$record['region']] = array();
            
        $results[$record['region']][$record['month']] = $record['count'];
    }
    
    $region_table_html = "<table class='brd' style='width:100%;'><tr><th>&nbsp;</th></tr>\n";
    $region_table_html .= "<tr><th align='left'>".join("</th></tr>\n<tr><th align='left'>", array_map("full_region_name", $target_regions))."</th></tr>\n";
    $region_table_html .= "</table>\n";

    // Some regions may be missing dates, so we'll need to fill them in.
    // We're making an assumption that there'll be a result from *some* region
    // for each month.
    fill_in_missing_regions($results);
    fill_in_missing_dates($results, $all_months_in_range);

    $all_dates = get_all_dates($results);
    $all_dates_row_html = "<tr><th>".join("</th>\n<th>", $all_dates)."</th></tr>\n";
        
    $results_html = "";
    foreach ($target_regions as $region)
    {
        $results_html .= "<tr><td align='right'>".join("</td>\n<td align='right'>", $results[$region])."</td></tr>\n";
    }

    // The actual results are put into a separate sub-table. This allows for easy
    // copy-and-paste-into-spreadsheet.
    $peak_day_logins_results_html = "
<table>
    <tr>
       <td>
           $region_table_html
       </td>
       <td>
           <table class='brd' style='width:100%;'>
                $all_dates_row_html
                $results_html
           </table>
       </td>
   </tr>
</table>
";
}

function process_average_day_logins_results($result)
{
    global $average_day_logins_results_html;
    global $target_regions;
    global $all_days_in_range;
    
    $results = array();
    
    while ($record = db_fetch_result($config, $result))
    {
        if (!strlen($record['region']))
            $record['region'] = $blank_region_name;
            
        if (!in_array($record['region'], $target_regions))
        {
            // Not a target region.
            continue;
        }
        
        $record = escape_html($record);
            
        if (!array_key_exists($record['region'], $results))
            $results[$record['region']] = array();
            
        $results[$record['region']][$record['day']] = $record['count'];
    }
    
    $region_table_html = "<table class='brd' style='width:100%;'><tr><th>&nbsp;</th></tr>\n";
    $region_table_html .= "<tr><th align='left'>".join("</th></tr>\n<tr><th align='left'>", array_map("full_region_name", $target_regions))."</th></tr>\n";
    $region_table_html .= "</table>\n";
    
    fill_in_missing_regions($results);
    fill_in_missing_dates($results, $all_days_in_range);
    
    // The data from the query is raw, so we'll need to calculate monthly averages from it.
    $averages = array();
    foreach ($results as $region_code => $region_results)
    {
        $averages[$region_code] = array();
        
        $current_month_results = array();
        $current_month = "";
        
        foreach ($region_results as $date => $count)
        {
            list($year, $month, $day) = sscanf($date, "%4c-%2c-%2c");
            
            $month = "$year-$month";
            
            if ($month != $current_month)
            {
                // Into a new month. Calculate results for previous one.
                if (strlen($current_month))
                {
                    $averages[$region_code][$current_month] = ceil(array_sum($current_month_results) / count($current_month_results));
                }
                // else there isn't really a current month. Just the initial value.

                $current_month = $month;
                $current_month_results = array();
            }
            
            $current_month_results[] = $count;
        }
        
        if (count($current_month_results) > 0)
            $averages[$region_code][$current_month] = ceil(array_sum($current_month_results) / count($current_month_results));
    }
    
    $all_dates = get_all_dates($averages);
    $all_dates_row_html = "<tr><th>".join("</th>\n<th>", $all_dates)."</th></tr>\n";
    
    $results_html = "";
    foreach ($target_regions as $region)
    {
        $results_html .= "<tr><td align='right'>".join("</td>\n<td align='right'>", $averages[$region])."</td></tr>\n";
    }

    // The actual results are put into a separate sub-table. This allows for easy
    // copy-and-paste-into-spreadsheet.
    $average_day_logins_results_html = "
<table>
    <tr>
       <td>
           $region_table_html
       </td>
       <td>
           <table class='brd' style='width:100%;'>
                $all_dates_row_html
                $results_html
           </table>
       </td>
   </tr>
</table>
";
}

function process_total_monthly_logins_results($result)
{
    global $total_monthly_logins_results_html;
    global $target_regions;
    global $all_months_in_range;
    
    $results = array();
    
    while ($record = db_fetch_result($config, $result))
    {
        if (!strlen($record['region']))
            $record['region'] = $blank_region_name;
            
        if (!in_array($record['region'], $target_regions))
        {
            // Not a target region.
            continue;
        }
        
        $record = escape_html($record);
            
        if (!array_key_exists($record['region'], $results))
            $results[$record['region']] = array();
            
        $results[$record['region']][$record['month']] = $record['count'];
    }
    
    $region_table_html = "<table class='brd' style='width:100%;'><tr><th>&nbsp;</th></tr>\n";
    $region_table_html .= "<tr><th align='left'>".join("</th></tr>\n<tr><th align='left'>", array_map("full_region_name", $target_regions))."</th></tr>\n";
    $region_table_html .= "</table>\n";

    // Some regions may be missing dates, so we'll need to fill them in.
    // We're making an assumption that there'll be a result from *some* region
    // for each month.
    fill_in_missing_regions($results);
    fill_in_missing_dates($results, $all_months_in_range);
    
    $all_dates = get_all_dates($results);
    $all_dates_row_html = "<tr><th>".join("</th>\n<th>", $all_dates)."</th></tr>\n";
    
    $results_html = "";
    foreach ($target_regions as $region)
    {
        $results_html .= "<tr><td align='right'>".join("</td>\n<td align='right'>", $results[$region])."</td></tr>\n";
    }

    // The actual results are put into a separate sub-table. This allows for easy
    // copy-and-paste-into-spreadsheet.
    $total_monthly_logins_results_html = "
<table>
    <tr>
       <td>
           $region_table_html
       </td>
       <td>
           <table class='brd' style='width:100%;'>
                $all_dates_row_html
                $results_html
           </table>
       </td>
   </tr>
</table>
";
}

function process_total_monthly_requests_results($result)
{
    global $total_monthly_requests_results_html;
    global $target_regions;
    global $all_months_in_range;
    
    $results = array();
    
    while ($record = db_fetch_result($config, $result))
    {
        if (!strlen($record['region']))
            $record['region'] = $blank_region_name;
            
        if (!in_array($record['region'], $target_regions))
        {
            // Not a target region.
            continue;
        }
        
        $record = escape_html($record);
            
        if (!array_key_exists($record['region'], $results))
            $results[$record['region']] = array();
            
        $results[$record['region']][$record['month']] = $record['count'];
    }
    
    $region_table_html = "<table class='brd' style='width:100%;'><tr><th>&nbsp;</th></tr>\n";
    $region_table_html .= "<tr><th align='left'>".join("</th></tr>\n<tr><th align='left'>", array_map("full_region_name", $target_regions))."</th></tr>\n";
    $region_table_html .= "</table>\n";

    // Some regions may be missing dates, so we'll need to fill them in.
    // We're making an assumption that there'll be a result from *some* region
    // for each month.
    fill_in_missing_regions($results);
    fill_in_missing_dates($results, $all_months_in_range);
    
    $all_dates = get_all_dates($results);
    $all_dates_row_html = "<tr><th>".join("</th>\n<th>", $all_dates)."</th></tr>\n";
    
    $results_html = "";
    foreach ($target_regions as $region)
    {
        $results_html .= "<tr><td align='right'>".join("</td>\n<td align='right'>", $results[$region])."</td></tr>\n";
    }

    // The actual results are put into a separate sub-table. This allows for easy
    // copy-and-paste-into-spreadsheet.
    $total_monthly_requests_results_html = "
<table>
    <tr>
       <td>
           $region_table_html
       </td>
       <td>
           <table class='brd' style='width:100%;'>
                $all_dates_row_html
                $results_html
           </table>
       </td>
   </tr>
</table>
";
}

// Dates can be SQL strings or expressions
function get_seq_dates($config, $date_from, $date_to, $params, $months=false)
{
    $query =
        $months ?
            "SELECT DATE_FORMAT(SEQ.SeqDate, '%Y-%m') AS seq " :
            "SELECT SEQ.SeqDate AS seq ";
            
    $query .= 
        "FROM ( SELECT DATE_ADD($date_from, INTERVAL (TEN_THOUSANDS.SeqValue + THOUSANDS.SeqValue + HUNDREDS.SeqValue + TENS.SeqValue + ONES.SeqValue) DAY) AS SeqDate FROM (SELECT 0 SeqValue UNION ALL SELECT 1 SeqValue UNION ALL SELECT 2 SeqValue UNION ALL SELECT 3 SeqValue UNION ALL SELECT 4 SeqValue UNION ALL SELECT 5 SeqValue UNION ALL SELECT 6 SeqValue UNION ALL SELECT 7 SeqValue UNION ALL SELECT 8 SeqValue UNION ALL SELECT 9 SeqValue) AS ONES CROSS JOIN (SELECT 0 SeqValue UNION ALL SELECT 10 SeqValue UNION ALL SELECT 20 SeqValue UNION ALL SELECT 30 SeqValue UNION ALL SELECT 40 SeqValue UNION ALL SELECT 50 SeqValue UNION ALL SELECT 60 SeqValue UNION ALL SELECT 70 SeqValue UNION ALL SELECT 80 SeqValue UNION ALL SELECT 90 SeqValue) AS TENS CROSS JOIN (SELECT 0 SeqValue UNION ALL SELECT 100 SeqValue UNION ALL SELECT 200 SeqValue UNION ALL SELECT 300 SeqValue UNION ALL SELECT 400 SeqValue UNION ALL SELECT 500 SeqValue UNION ALL SELECT 600 SeqValue UNION ALL SELECT 700 SeqValue UNION ALL SELECT 800 SeqValue UNION ALL SELECT 900 SeqValue) AS HUNDREDS CROSS JOIN (SELECT 0 SeqValue UNION ALL SELECT 1000 SeqValue UNION ALL SELECT 2000 SeqValue UNION ALL SELECT 3000 SeqValue UNION ALL SELECT 4000 SeqValue UNION ALL SELECT 5000 SeqValue UNION ALL SELECT 6000 SeqValue UNION ALL SELECT 7000 SeqValue UNION ALL SELECT 8000 SeqValue UNION ALL SELECT 9000 SeqValue) AS THOUSANDS CROSS JOIN (SELECT 0 SeqValue UNION ALL SELECT 10000 SeqValue UNION ALL SELECT 20000 SeqValue UNION ALL SELECT 30000 SeqValue UNION ALL SELECT 40000 SeqValue UNION ALL SELECT 50000 SeqValue UNION ALL SELECT 60000 SeqValue UNION ALL SELECT 70000 SeqValue UNION ALL SELECT 80000 SeqValue UNION ALL SELECT 90000 SeqValue) AS TEN_THOUSANDS WHERE DATE_ADD($date_from, INTERVAL (TEN_THOUSANDS.SeqValue + THOUSANDS.SeqValue + HUNDREDS.SeqValue + TENS.SeqValue + ONES.SeqValue) DAY) <= $date_to ) AS SEQ ";
        
    $query .=
        $months ?
            "GROUP BY seq " :
            "";
        
    $result = db_query_execute($config, $query, convert_null_array($params));

    $dates = array();
    while ($record = db_fetch_result($config, $result))
    {
        $dates[] = $record['seq'];
    }
    
    return $dates;
}


// Assumes $results looks like [{region:[{date:count}]}]
function get_all_dates($results)
{
    $all_dates = array();
    
    foreach ($results as $region_result)
    {
        $all_dates = array_merge($all_dates, array_keys($region_result));
    }

    $all_dates = array_unique($all_dates);
    sort($all_dates);
    return $all_dates;
}

function target_region_sort($a, $b)
{
    global $target_regions;
    return array_search($a, $target_regions) > array_search($b, $target_regions) ? 1 : -1;
}
    
function fill_in_missing_regions(&$results)
{
    global $target_regions;
    foreach ($target_regions as $region)
    {
        if (!isset($results[$region]))
        {
            $results[$region] = array();
        }
    }
    
    uksort($results, "target_region_sort");
}

// Any mising dates will be given a count of zero. The date sub-arrays will also
// be sorted.
function fill_in_missing_dates(&$results, $all_dates)
{
    foreach ($results as $region => $result)
    {
        foreach ($all_dates as $date)
        {
            if (!array_key_exists($date, $result))
            {
                $results[$region][$date] = 0;
            }
        }
        
        ksort($results[$region]);
    }
}

function full_region_name($region_code)
{
    return get_country_name_from_country_code($region_code)." (".$region_code.")";
}

function region_name_sort_callback($region_name1, $region_name2)
{
    global $unknown_region_name;
    // Make sure BLANK sorts to the bottom.
    if ($region_name1 == $unknown_region_name)
    {
        return 1;
    }
    else if ($region_name2 == $unknown_region_name)
    {
        return -1;
    }

    return strcasecmp($region_name1, $region_name2);
}


// Screen title. Must come before the header.php include.
$title = 'Usage Report Data';

// include menu side-bar
include($_SERVER['DOCUMENT_ROOT'].'/header.php');

?>

<?/*
Output forms.
Hidden '*_submit' fields are used to determine when/which form is submitted.
*/?>

<?/* The filtering form */?>
<form name='filter' method='get' action='<?=getenv('SCRIPT_NAME')?>'>
    <?/* Because the method for this form is GET (and the effects are read-only),
         we don't use the anti-CRSF field.
        <input name='csrf_token' value='<?=$record_user['csrf_token']?>' type='hidden'>*/?>
    <input name='filter_submit' value='1' type='hidden'>

    <table class='outline vertspace'>
        <tr class='nowrap'>
            <ul>
                <li>This page produces the results required to fill in the Usage monthly reporting spreadsheet.</li>
                <li>The desired results can easily be selected, copied, and pasted directly into the spreadsheet.</li>
            </ul>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Month From:</th>
            <td align='left'><input type=text name='filter_date_from' value='<?=escape_html($filter_date_from)?>'> YYYY-MM or blank</input></td>
        </tr>
        <tr class='nowrap'>
            <th align='right'>Month To:</th>
            <td align='left'><input type=text name='filter_date_to' value='<?=escape_html($filter_date_to)?>'> YYYY-MM or blank</input></td>
        </tr>

        <tr>
            <td><input type='submit' value='Submit'></td>
            <!--uncomment to get the box the full width of the screen <td></td><td width='90%'></td>-->
        </tr>

    </table>

</form>
<br/>

<? if ($_GET['filter_submit'] == '1') { ?>

<h3>Peak Day Psiphon Users per month by Country</h3>
<p>(The number of unique, cookie-based Psiphon sessions logged on whichever day, during a particular month, had the greatest number of sessions)</p>

<?= $peak_day_logins_results_html ?>

<br/>
<h3>Average Daily Psiphon Users per month by Country</h3>
<p>(The average, throughout a particular month, of the number of unique cookie-based Psiphon sessions logged in a single day)</p>

<?= $average_day_logins_results_html ?>

<br/>
<h3>Total Monthly Psiphon Users by Country</h3>
<p>(The total number of unique cookie-based Psiphon sessions logged during a particular month)</p>

<?= $total_monthly_logins_results_html ?>

<br/>
<h3>Total Monthly Psiphon Usage by Country</h3>
<p>(The total number of page-views served through Psiphon during a particular month)</p>

<?= $total_monthly_requests_results_html ?>

<? } /*if ($_GET['filter_submit'] == '1')*/ ?>

<?
// output footer (e.g., copyright)
include($_SERVER['DOCUMENT_ROOT'].'/footer.php');
?>
