<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// anonymous.php - the anonymous browsing configuration screen

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "anonymous"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

// process delete get request

while ($_GET["delete"] = (int)$_GET["delete"])
{
    $query="DELETE FROM anonymous_browsing WHERE id = :id";
    $result = db_query_execute($config, $query, convert_null_array(array(":id" => $_GET['delete'])));

    if (!db_num_rows_affected($result))
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }

    $processing_results[] = ___("Deleted guest browsing URL");
    break;
}

// process form

while ($_POST[anonymous_submit] == "1")
{
    // validation flag: check all fields and output errors for each at once

    $valid = true;

    // check home page

    $home_page = trim($_POST["home_page"]);

    if (strlen($home_page) <= 0 || !is_url($home_page))
    {
        $processing_errors[] = ___("Error: Invalid home page format");
        if ($valid) $curs = "anonymous.home_page";
        $valid = false;
    }

    // check proxy selection

    $query = "SELECT * FROM proxy WHERE id = :proxy_id";

    $result = db_query_execute($config, $query, convert_null_array(array(":proxy_id" => $_POST[proxy])));

    if (!($proxy_record=escape_html(db_fetch_result($config, $result))) || !$proxy_record['hostname'])
    {
         $processing_errors[] = ___("Error: no proxy selected");
         if ($valid) $curs = "anonymous.proxy";
         $valid = false;
    }

    // check lang

    if (!in_array($_POST["anonymous_lang"], array_keys($config["lang"])))
    {
         $processing_errors[] = ___("Error: No language selected");
         if ($valid) $curs = "anonymous.home_page";
         $valid = false;
    }

    // done validating inputs

    if (!$valid)
    {
        break;
    }

    // unguessable token

    $token = bin2hex(secure_rand());

    // check locked

    $home_page_locked = ($_POST[home_page_locked] ? 1 : 0);

    // insert new anonymous browsing record

    $query = "INSERT INTO anonymous_browsing (token, home_page, lang, proxy, home_page_locked, created) ".
             "VALUES (:token, :home_page, :lang, :proxy, :home_page_locked, UNIX_TIMESTAMP())";

    db_query_execute(
        $config,
        $query,
        convert_null_array(array(
            ":token" => $token,
            ":home_page" => $home_page,
            ":lang" => $_POST[anonymous_lang],
            ":proxy" => $_POST[proxy],
            ":home_page_locked" => $home_page_locked)));

    $anonymous_success = 1;

    // Output result
    $processing_results[] = ___("Created guest browsing URL: ").make_anonymous_browsing_url($config, $proxy_record['id'], $token);

    break;
}

// Assemble data for list display

$query = "SELECT anonymous_browsing.*, FROM_UNIXTIME(anonymous_browsing.created, '".___("%Y-%m-%d %H:%i:%S")."') AS created_formatted, ".proxy_label_sql("proxy")." AS proxy_label FROM anonymous_browsing JOIN proxy ON anonymous_browsing.proxy = proxy.id ORDER BY anonymous_browsing.created DESC";
$result = db_query_execute($config, $query, array());

// output one row per user

$rows = array();
while ($record=escape_html(db_fetch_result($config, $result)))
{
    // Get associated proxy record

    $record[url] = make_anonymous_browsing_url($config, $record['proxy'], $record['token']);
    $rows[] = $record;
}

db_free_result($result);

// Screen title. Must come before the header.php include.
$title = ___("Guest Browsing");

// include menu side-bar
include($_SERVER[DOCUMENT_ROOT]."/header.php");

// Output the check_all javascript handler.
echo_check_all_handler();

// debug
//$x = print_r($_POST, TRUE);
//echo "<pre>{$x}</pre>";

?>

<?/* Presentation Layer */?>

<table>
    <tr><td>

        <?/* The new record form */?>
        <form name="anonymous" method="post" action="<?=getenv("SCRIPT_NAME")?>">
            <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden">
            <input name="anonymous_submit" value="1" type="hidden">

            <table cellpadding="2" cellspacing="0" border="0" style="border: thin solid gray; border-spacing: 2px; ">

                <tr>
                    <th><?=___("Home page")?></th>
                    <th><?=___("Locked")?></th>
                    <th><?=___("Proxy")?></th>
                    <th><?=___("Language")?></th>
                </tr>

                <tr>
                    <td>
                        <?/* Populate home page text. Don't restore value after successful operation. */?>
                        <input type="text" name="home_page" value="<?=$anonymous_success ? "" : escape_html($_POST[home_page])?>" size="35" maxlength="255">
                    </td>
                    <td nowrap valign="middle">
                        <?/* Populate locked check box. Don't restore check after successful operation. */?>
                        <input type="checkbox" name="home_page_locked" <?=(($_POST[home_page_locked] && !$anonymous_success) ? "checked=1" : "")?> ><?=___("Locked")?>
                    </td>
                    <td>
                        <?/* Populate proxy combo box. Don't restore proxy selection after successful operation. */?>
                        <?=echo_proxy_combo($config, "proxy", $anonymous_success ? NULL : $_POST[proxy], 1)?>
                    </td>
                    <td>
                        <?/* Populate language combo box. Set to user's lang by default or after success. */?>
                        <?=echo_lang_combo($config, "anonymous_lang", ($_POST[anonymous_lang] && !$anonymous_success) ? $_POST[anonymous_lang] : $record_user[lang])?>
                    </td>
                    <td><input type="submit" value="<?=___("Add")?>"></td>
                    <td width="90%">&nbsp;</td>
                </tr>

            </table>
        </form>

    </td></tr>

    <tr><td>&nbsp;</td></tr>

    <tr><td>

        <?/* List */?>

        <table cellpadding="2" cellspacing="0" border="0" name="user_table" class="brd">

            <tr>
                <th><?=___("Home page")?></th>
                <th><?=___("Locked")?></th>
                <th><?=___("Proxy")?></th>
                <th><?=___("Language")?></th>
                <th><?=___("URL")?></th>
                <th><?=___("Created")?></th>
                <th></th>
            </tr>

            <?foreach ($rows as $row){?>
                <tr>
                    <td><?=$row[home_page]?></td>
                    <td><?=($row[home_page_locked] ? ___("yes") : ___("no"))?></td>
                    <td><?=$row[proxy_label]?></td>
                    <td><?=$config["lang"][$row[lang]]["title"]?></td>
                    <td><a href="<?=$row[url]?>"><?=$row[url]?></a></td>
                    <td><?=$row[created_formatted]?></td>
                    <td><a href="<?=getenv("SCRIPT_NAME")."?delete={$row[id]}"?>">X</a></td>
                </tr>
            <?}?>

        </table>
    </td></tr>
</table>

<?
// output footer (e.g., copyright)
include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>
