<?php
/*
   Psiphon Circumvention Platform
   Copyright (C) 2009 Psiphon Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

include_once($_SERVER[DOCUMENT_ROOT] . "/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT] . "/check_user.php");
include($_SERVER[DOCUMENT_ROOT] . "/includes/lang_set.php");

function validate_bookmark_url($url)
{
    $url = trim($url);
    if ($url &&
            substr($url, 0, 7) != "http://" &&
            substr($url, 0, 8) != "https://" &&
            substr($url, 0, 6) != "ftp://" )
    {
        return "http://" . $url; 
    }
    return $url;
}

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "bookmarks"))
{
    include($_SERVER[DOCUMENT_ROOT] . "/http-errors/403.php");
    // 403.php exits
}

$title = ___("Bookmarks");
$bookmarks_continue = false;
$show_add_form = false;

if ($_GET["ret"])
{
    $ret = $_GET["ret"];
}
else
{
    $ret = getenv("SCRIPT_NAME");
}
if ($_GET["bookmark_url"] || $_GET["add"] == 1)
{
    $bookmark_url = validate_bookmark_url($_GET["bookmark_url"]);
    $bookmark_title="";
    $show_add_form = true;
}
if($_POST["mode"] == "ADD" || $_POST["mode"] == "EDIT")
{
    $bookmark_url = validate_bookmark_url($_POST["url"]);
    $bookmark_title = trim($_POST["title"]);
}

if ($_GET["bookmark_url"] || $_GET["edit_bookmark_id"])
{
    $curs = "bookmark.title";
    $title = ___("Bookmark");
}

while ($_POST["mode"] == "ADD")
{
    if (!$bookmark_url)
    {
        $curs = "bookmark.url";
        $processing_errors[] = ___("URL can't be empty");
        $show_add_form = true;
        break;
    }
    $query = "INSERT IGNORE INTO bookmark (user, url, title) VALUES (:user_id, :url, :title)";

    $stmt = db_query_execute($config, $query, 
            convert_null_array(
                array(
                    ":user_id" => $record_user['id'],
                    ":url" => $bookmark_url, 
                    ":title" => $bookmark_title
                    )));
    if(db_num_rows_affected($stmt) == 0)
    {
        $processing_errors[] = ___("Bookmark already exists");
        $curs = "bookmark.url";
        $show_add_form = true;
        break;
    }
    $last_affected_id = db_get_last_insert_id($config);
    $processing_results[] = ___("Link saved");
    $bookmarks_continue = true;
    unset($_GET);
    unset($_POST);
    break;
}
while ($_GET['delete_bookmark_id'])
{
    $query = "DELETE FROM bookmark WHERE user = :user_id AND id = :id";

    db_query_execute($config, $query, 
            convert_null_array(array(":user_id" => $record_user['id'], ":id" => $_GET['delete_bookmark_id'])));
    break;
}

while ($_POST["mode"] == "EDIT")
{
    if (!$bookmark_url)
    {
        $curs = "bookmark.url";
        $processing_errors[] = ___("URL can't be empty");
        break;
    }

    // NOTE: Because we don't have transactions, there is a race condition between
    // the url conflict check and the insert/update attempt. If a conflict is
    // introduced between the check and the insert/update, a DB error will occur.
    // This is unfortunate but acceptable (at this time).

    if (db_check_if_value_exists($config, "bookmark", "url", $bookmark_url, "user = :user_id AND id != :bookmark_id", 
                array(":user_id" => $record_user['id'], ":bookmark_id" => $_POST["bookmark_id"])))
    {
        $processing_errors[] = ___("Bookmark already exists");
        $curs = "bookmark.url";
        break;
    }

    $query = "UPDATE bookmark SET url = :url, title = :title WHERE user = :user_id AND id = :bookmark_id";

    db_query_execute($config, $query, 
            convert_null_array(
                array(
                    ":url" => $bookmark_url, 
                    ":title" => $bookmark_title,
                    ":user_id" => $record_user["id"],
                    ":bookmark_id" => $_POST["bookmark_id"]
                    )));
    $last_affected_id = $_POST["bookmark_id"];
    $processing_results[] = ___("Link saved");
    break;
}

include($_SERVER[DOCUMENT_ROOT] . "/header.php");

if ($bookmarks_continue)
{
    echo "<a href=\"{$ret}\">" . ___("Continue") . "</a><br><br>\n";
}
?>

<form name="bookmark" method="post" action="<?=getenv("SCRIPT_NAME")."?ret=".urlencode($_GET["ret"])?>">
<input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden">
<input name="mode" value="ADD" type="hidden">
<table cellpadding="2" cellspacing="0" border="0" class="brd">
<tr>
<th><?=___("Link")?></th>
<th><a href="<?=getenv("SCRIPT_NAME")?>?add=1"><?=___("Add")?></a></th>
</tr>

<?php
if ($show_add_form)
{
?>
        <tr>
        <td align="right"><input type="hidden" name="id" value="add">
        <?=___("Title")?> 
        <input type="text" name="title" maxlength="64" size="60" value="<?=htmlentities($bookmark_title, ENT_QUOTES, charset())?>"><br>
        <?=___("Link")?> 
        <input type="text" name="url" maxlength="255" size="60" value="<?=htmlentities($bookmark_url, ENT_QUOTES, charset())?>"></td>
        <td valign="middle"><input type="submit" value="<?=___("Add")?>"> &nbsp; <a href="<?=$ret?>"><?=___("Cancel")?></a></td>
        </tr>

<?php
}

$query = "SELECT * FROM bookmark WHERE user = :user_id ORDER BY url";
$result = db_query_execute($config, $query, 
        convert_null_array(array(":user_id" => $record_user["id"])));

while ($record = db_fetch_result($config, $result))
{
    $edit_form = false;
    echo "<tr>\n";

    if ($record["id"] == $_POST["bookmark_id"] || $record["id"] == $last_affected_id)
    {
        $_p = "<b>";
        $_s = "</b>";
    }
    else
    {
        $_p = $_s = "";
    }

    if($_GET["edit_bookmark_id"] == $record["id"])
    {
        $edit_form = true;
        $bookmark_title = $record["title"];
        $bookmark_url = $record["url"];
    }
    if($_POST["bookmark_id"] == $record["id"] && $_POST["mode"] == "EDIT" && !empty($processing_errors))
    {
        $edit_form = true;
    }
    if ($edit_form)
    {
        echo "<td align=\"right\">";
        echo ___("Title");
        echo "<input type=\"text\" name=\"title\" maxlength=\"64\" size=\"60\" value=\"" . 
            htmlentities($bookmark_title, ENT_QUOTES, charset()) . "\"><br>";
        echo ___("Link");
        echo "<input type=\"text\" name=\"url\" maxlength=\"255\" size=\"60\" value=\"" . 
            htmlentities($bookmark_url, ENT_QUOTES, charset()) . "\">";
        echo "</td>\n";
        echo "<td>";
        echo "<input type=\"submit\" value=\"" . ___("Update") . "\"> &nbsp; <a href=\"" . 
            getenv("SCRIPT_NAME") . "\">" . ___("Cancel") . "</a>";
        echo "<input name=\"mode\" value=\"EDIT\" type=\"hidden\">";
        echo "<input name=\"bookmark_id\" value=\"{$record["id"]}\" type=\"hidden\">";
        echo "</td>\n";
    }
    else
    {
        echo "<td><a href=\"" . make_proxied_url($record["url"]) . "\">{$_p}";
        if (strlen($record["title"]))
        {
            echo htmlentities($record["title"], ENT_QUOTES, charset());
        }
        else
        {
            echo htmlentities($record["url"], ENT_QUOTES, charset());
        }
        echo "{$_s}</a></td>\n";
        echo "<td><a href=\"" . getenv("SCRIPT_NAME") . "?edit_bookmark_id=" . 
            $record["id"] . "\">{$_p}" . ___("Edit") . "{$_s}</a> &nbsp;";
        echo "<a href=\"" . getenv("SCRIPT_NAME") . "?delete_bookmark_id=" . 
            $record["id"] . "\">{$_p}" . ___("Delete") . "{$_s}</a>";
        echo "</td>\n";
    }

    echo "</tr>\n\n";
}
?>

</table>

</form>

<?php
include($_SERVER[DOCUMENT_ROOT] . "/footer.php");
?>
