<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009-2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

include_once($_SERVER[DOCUMENT_ROOT]."/includes/database_helpers.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/user_class_helpers.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/cr.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/geoip_helpers.php");

// Set record_user[last_lang] early as it's referenced in ___()

if (strlen($_POST["login_lang"]) && array_key_exists($_POST["login_lang"], $config["lang"]))
{
    $record_user["last_lang"] = $_POST["login_lang"];
    $lang_selected = True;
}

// Get login_url for this proxy for use in Reset Password link
// This code is adapted from chk.php

$query = "SELECT * FROM proxy WHERE hostname = :hostname";
$result = db_query_execute($config, $query, convert_null_array(array(":hostname" => $_SERVER[HTTP_HOST])));
$proxy_record = db_fetch_result($config, $result);

if (!$proxy_record)
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}

$login_url = $proxy_record['login_url'];

// === Process login form ===
// User has submitted the login form rendered below

while ($_POST["login_id"] || $_POST["login_pass"])
{
    // === Normal login ===

    if (!strlen($_POST[login_id]) || !strlen($_POST[login_pass]))
    {
        $login_msg=___("Wrong username or password");
        break;
    }

    $stay_logged_in = @$_POST['stay_logged_in'];

    // - Find user with matching username (password check done after, with salt)
    // - Login delay: if login_delay is not 0, don't allow login
    //   if not passed the delay time.  The delay time is set
    //   at failed-login time (see below).
    // - Anonymous Users are excluded from login
    $login_query = "SELECT id, pass, pass_salt, grp, ".
                        "(SELECT proxy.id FROM proxy WHERE proxy.hostname = :hostname) AS proxy ".
                   "FROM user ".
                   "WHERE (email = :email OR uname = :uname) ".
                   "      AND grp <> ".get_anonymous_user_class_id()." ".
                   "      AND (login_delay = 0 or login_delay < UNIX_TIMESTAMP()) ".
                   "      AND NOT locked";

    $login_query_params = array(
                            ":email" => $_POST['login_id'],
                            ":uname" => $_POST['login_id'],
                            ":hostname" => getenv("HTTP_HOST"));

    $login_result = db_query_execute($config, $login_query, convert_null_array($login_query_params));

    $user_found = false;
    $pass_match = false;

    $login_record = db_fetch_result($config, $login_result);
    if ($login_record)
    {
        // Backwards compatible with old, MD5 and non-salted hashed passwords.
        // Explicit length checks prevent unnecessary crypto calls
        //
        // (Is there a timing attack here, if == exits early?  Attacker that knows salt could
        // select login_pass in such a way as to probe the 1st, then 2nd, etc. byte of the
        // hashed password.  But failed login delay makes this intractable.)

        $user_found = true;
        if ((strlen($login_record['pass']) == 40 && $login_record['pass'] == sha1($_POST['login_pass'].$login_record['pass_salt'])) ||
            (strlen($login_record['pass']) == 32 && $login_record['pass'] == md5($_POST['login_pass'])))
        {
            $pass_match = true;
        }
    }

    if (!$user_found)
    {
        $login_msg = ___("Wrong username or password");
    }
    else if (!$pass_match)
    {
        // Valid user/wrong password, so invalid credential or login delay in effect

        // - Update failed login delay: delay time is a function of number of
        //   consecutive failed logins, with a maximum delay time (e.g., 30 seconds)
        // - Note the delay is not in the response to the attacker, but in the time
        //   before a valid login can succeed.  So, no server resource is consumed
        //   by the delay, and the attacker doesn't gain information (that the username
        //   must be valid as a delay is in effect).
        // - There's a DoS side-effect in that an attacker is free to inflate
        //   the delay time for any known or guessable username.
        // - If the correct credential was actually entered, don't increase the
        //   delay (although still not logging in)
        $delay_query =
           "UPDATE user SET ".
           "  login_delay = UNIX_TIMESTAMP() + ".
           "    least(consecutive_failed_logins, '{$config['max_login_delay']}'), ". // don't need to parameterize the config value
           "  consecutive_failed_logins = consecutive_failed_logins + 1 ".
           "WHERE uname = :uname";

        db_query_execute($config, $delay_query, convert_null_array(array(":uname" => $_POST['login_id'])));

        $login_msg = ___("Wrong username or password");
    }
    else
    {
        if ($config["no_proxy"] || user_has_permission($config, $login_record, "other_nodes"))
        {
            // Do nothing.
            // Either the system is configured to not use proxies, or the current
            // user has permission to login from anywhere (not just specific
            // proxies) and administer all proxies.
        }
        else
        {
            // The user has no special permissions and so must log in from one of
            // the proxies associated with his user record.

            $login_query = "SELECT TRUE ".
                           "FROM userproxy JOIN proxy ON userproxy.proxyid = proxy.id ".
                           "WHERE userproxy.userid = :userid ".
                            "AND proxy.hostname = :hostname ";

            $login_query_params = array(":userid" => $login_record["id"],
                                        ":hostname" => getenv("HTTP_HOST"));

            $login_result = db_query_execute(
                                $config,
                                $login_query,
                                convert_null_array($login_query_params));

            if (!db_fetch_result($config, $login_result))
            {
                $login_msg = ___("Wrong username or password");
                break;
            }
        }

        // Password salt patch-up
        //
        // Old code used MD5 and didn't salt passwords; pass_salt database field will be NULL
        // in this case --> generate a salt and store a new hash since we have the plaintext
        // password at this time.

        if (!$login_record['pass_salt'] || strlen($login_record['pass_salt']) == 0)
        {
            $pass_salt = bin2hex(secure_rand());

            $query = "UPDATE user ".
                     "SET pass = :pass, pass_salt = :pass_salt ".
                     "WHERE id = :user_id";

            db_query_execute(
                $config,
                $query,
                convert_null_array(array(
                    ":pass" => sha1($_POST['login_pass'].$pass_salt),
                    ":pass_salt" => $pass_salt,
                    ":user_id" => $login_record['id'])));
        }

        // CSRF defense
        //
        // For a good overview of CSRF, read this paper:
        // http://www.google.ca/search?q=Cross-Site+Request+Forgeries+Exploitation+and+Prevention
        //
        // A secret token is generated and stored in the user record.  Each form
        // the user submits in this session must include this token value as a
        // hidden value field.  This token must never be placed in a cookie.

        $csrf_token = bin2hex(secure_rand());

        // generate secure session ID and assign to user record and browser cookie

        $session_id = bin2hex(secure_rand());

        // GeoIP lookup and stats update

        $login_region = lookup_region($config);
        update_login_stats($config, $login_region, $login_record['proxy'], $login_record['grp']);


        $can_stay_logged_in = user_has_permission($config, $login_record, 'stay_logged_in');

        // - Set user session (cookie), last login time, last login region
        // - Reset failed login data to no delay
        // - Set user's current proxy to the current proxy.
        $login_query =
            "UPDATE user ".
            "SET ".
            "  session = :session_id, ".
            "  csrf_token = :csrf_token, ".
            "  last = UNIX_TIMESTAMP(), ". // This may be redundant with an update done in mod_psiphon
            "  last_reg = :last_reg, ".
            "  login_delay = 0, ".
            "  consecutive_failed_logins = 0, ".
            '  stay_logged_in = '.($can_stay_logged_in && $stay_logged_in ? 1 : 0).', '.
            "  current_proxy = :proxy_id ".
            "WHERE user.id = :user_id";

        db_query_execute(
            $config,
            $login_query,
            convert_null_array(array(
                ":session_id" => $session_id,
                ":csrf_token" => $csrf_token,
                ":last_reg" => $login_region,
                ":user_id" => $login_record['id'],
                ":proxy_id" => $login_record['proxy'])));


        if ($lang_selected)
        {
            $update_language_query =
                "UPDATE user SET last_lang = :last_lang WHERE id = :user_id";

            db_query_execute(
                $config,
                $update_language_query,
                convert_null_array(array(
                    ":last_lang" => $record_user["last_lang"],
                    ":user_id" => $login_record['id'])));
        }

        $_COOKIE[p] = $session_id;

        // Set cookie
        $cookie_str = "Set-Cookie: p={$_COOKIE[p]}; path=/;";
        if ($can_stay_logged_in && $stay_logged_in)
        {
            $cookie_str .= " expires=". gmdate('D, d-M-Y H:i:s', time() + $config['stay_logged_in_ttl']);
        }

        header($cookie_str, false);

        // TODO: need clarification on what this code is for...
        $login_query = "DELETE FROM cookie WHERE user = :user_id AND (expire = -1 OR expire < ".(date("U") + date("Z")).")";

        db_query_execute($config, $login_query, convert_null_array(array(":user_id" => $login_record['id'])));
    }

    break;
}

// === Process logged in user ===
// - load $record_user, which is used in files that include check_user.php
// - update last activity
// - CSRF check

if ($_COOKIE[p])
{
    $record_user = lookup_user($config,
                               "session is not NULL and session = :session_id and !locked",
                               array(":session_id" => $_COOKIE[p]));
    if ($record_user)
    {
        // CSRF check (see comment above)
        // we can only check POST requests, since we can't assume GETs are for
        // form submission... so only use GET to retrieve data, not modify
        // data (consistent with RFC 2616 HTTP/1.1 recommendation)
        //
        // The !$csrf_token check excludes the login form case, as there's
        // no session and no embedded secret token yet.
        //
        // $do_get_csrf is a flag set by a check_user.php includer.  In most
        // cases, we don't include the CSRF token in GET requests since a GET
        // is idempotent and doesn't leak information.

        if (!$csrf_token &&
            ((count($_POST) > 0 && $_POST['csrf_token'] != $record_user['csrf_token']) ||
             ($do_get_csrf && count($_GET) > 0 && $_GET['csrf_token'] != $record_user['csrf_token'])))
        {
            // TODO: log suspected attack event
            include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        }

        $_SERVER["psiphon_uid"] = $record_user[id];

        // update last_activity, which is used as a session timeout
        // since check_user.php is included at the top of all management screens,
        // this update should keep alive sessions with management activity.

        $update_sql = "UPDATE user SET last_activity = UNIX_TIMESTAMP() WHERE id = :user_id";

        db_query_execute($config, $update_sql, convert_null_array(array(":user_id" => $record_user['id'])));

        $record_user["perm"] = $config["grp"][$record_user["grp"]];

        // Check if the user is coming from an acceptable proxy.
        if (FALSE === array_search($record_user['current_proxy'], $record_user['assigned_proxies']))
        {
            if (user_has_permission($config, $record_user, "other_nodes"))
            {
                // If the user has the other_nodes permission, he can come from anywhere (proxy or not).
            }
            else // ordinary users must come from their proxy
            {
                // Ordinary user must come from his assigned proxy.
                include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
                // 403.php exits
            }
        }
    }
}

// === Process usage terms form ===
// If the user agreed, update the user record to reflect this

if ($_POST['terms_submit_agree'] && $record_user)
{
    $terms_query = "UPDATE user SET agreed_to_usage_terms = 1 WHERE id = :user_id";

    db_query_execute($config, $terms_query, convert_null_array(array(":user_id" => $record_user['id'])));

    // refresh the record_user value (referenced in logic below)
    $record_user['agreed_to_usage_terms'] = 1;
}

// === Display login screen ===
// The user has no cookie, so no session, and must complete the login form

if (!$_COOKIE[p] || !$record_user)
{
   if (!$login_curs && $_POST[login_id]) $login_curs="login_pass";
   else $login_curs="login_id";

   header("Content-type: text/html; charset=".charset());
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
    <head>
        <title><?=___("Login")?></title>
        <?=echo_css()?>
    </head>

    <body style="direction: <?=direction()?>">

            <table><tr>
                <td style="width:50%">
<?php
    include($_SERVER[DOCUMENT_ROOT]."/images/psiphonlogo.html");
?>
                </td>
                <td><font class="sixteenbluebold"><?=___("Log into Psiphon")?></font></td>
            </tr></table>
            <br>

            <? if ($login_msg) echo "<font class=\"processing_error\">{$login_msg}</font><br><br>\n\n"; ?>

            <form name="login" method="post" action="<?=escape_html(getenv("REQUEST_URI"))?>">
            <input type="hidden" name="login_lang" value="<?=$_POST["login_lang"]?>"/>
            <table border="0" cellpadding="2" cellspacing="0">

                <tr>
                    <td><label for="email"><font class="fourteenblue"><?=___("Username (or e-mail)")?></font></label></td>
                    <td><input type="text" name="login_id" value="<?=escape_html($_POST[login_id])?>" size="30" maxlength="64" id="email"></td>
                </tr>
                <tr>
                    <td><label for="pass"><font class="fourteenblue"><?=___("Password")?></font></label></td>
                    <td><input type="password" name="login_pass" size="30" id="pass"></td>
                </tr>
                <tr>
                    <td><label for="stay_logged"><font class="fourteenblue"><?=___("Stay logged in")?></font></label></td>
                    <td><input type="checkbox" name="stay_logged_in" id="stay_logged" value="1"></td>
                </tr>
                <tr><td></td></tr>
                <tr>
                    <td></td>
                    <td colspan="2" align="left"><input type="submit" value="<?=___("Login")?>"></td>
                </tr>
                <tr><td></td></tr>
                <tr>
                    <td></td>
                    <td colspan="2" align="left"><label><font class="fourteenblue"><a href="/reset_password.php?lu=<?=$login_url?><?=strlen($record_user["last_lang"]) ? "&amp;reset_lang=".$record_user["last_lang"] : ""?>"><?=___("Reset password")?></a></font></label></td>
                </tr>

            </table>
            </form>

            <br>
            <br>

            <?echo_language_buttons($config, $record_user, "login_lang", 6);?>

        <script language="javascript" type="text/javascript">
        <!--
        document.login.<?=$login_curs?>.focus();
        //-->
        </script>
    </body>
</html>
<?
    // Don't proceed with the screen that included check_user.php...

    exit(0);

}

// === Usage Terms ===
// Display the usage terms, if user hasn't agreed to terms of use.
// When the user agrees, update their record with the agreed flag. If the
// user does not agree, they are redirected to logout.php.

if (!$dont_require_terms &&
    $_COOKIE[p] &&
    $record_user &&
    !$record_user['agreed_to_usage_terms'])
{
    if ($lang_selected)
    {
        // record_user has been refreshed by lookup_user, so we need to reset it based on login_lang
        $record_user["last_lang"] = $_POST["login_lang"];

        $update_language_query =
            "UPDATE user SET last_lang = :last_lang WHERE id = :user_id";

        db_query_execute(
            $config,
            $update_language_query,
            convert_null_array(array(
                ":last_lang" => $record_user["last_lang"],
                ":user_id" => $record_user['id'])));
    }

    header("Content-type: text/html; charset=".charset());
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
    <head>
        <title><?=___("Welcome to Psiphon!")?></title>
        <?=echo_css()?>
    </head>

    <body style="direction: <?=direction()?>">
        <table style>
            <tr>
                <table><tr>
                    <td style="width:50%">
<?php
    include($_SERVER[DOCUMENT_ROOT]."/images/psiphonlogo.html");
?>
                    </td>
                    <td><font class="sixteenbluebold"><?=___("Welcome to Psiphon!")?></font></td>
                </tr></table>
            </tr>
            <tr>
                <td><div style="width:40em;"><font class="fourteenblue"><br><?=___("<p>Psiphon is a web-based proxy system that enables you to view
Internet content that might be inaccessible in your region.  To use
Psiphon: </p>
<ol>
 <li>Read and agree to <b>Psiphon's Terms of Service</b></li>
<br>
<li>Enter the address of the site you would like to visit into the
<b>Psiphon blue bar</b>, which will appear at the top of your browser
window.  Click <b>\"Go\"</b>.   (Repeat this for each new site you wish
to visit.)</li>
<br>
<li>To ensure you have consistent access to the Psiphon service, you
may wish to create a Psiphon account.  To do so, simply click the
<b>\"Create Account\"</b> link on the blue bar and follow the
instructions.</li>
</ol>
<p>Before using Psiphon, please read and agree to the Terms of Service.</p>
<p><i><b>Please note</b>, Psiphon is designed to provide you with open
access to online content.  Psiphon does not increase your online
privacy, and should not be considered or used as an online security
tool.</i></p>")?><br><br></font></td>
            </tr>
            <tr>
                <td><textarea class="blueback" name="terms" cols="80" rows="8" readonly><?=___("Psiphon Inc. - Terms of Use and Privacy Statement

Privacy

Psiphon collects information that you submit as part of the account creation process: a username (this should not be your real name) a password, and optionally, an e-mail address (which should be from a provider such as GMail that offers SSL and is hosted outside of your censoring country/region).

Psiphon does not store usage information in a manner that allows Psiphon to correlate an individual user’s Psiphon account or IP address with a visit to any individual web site. Psiphon records the date/time of account creation, the date/time and country/region of each user's last login. Psiphon also records the number of invitations a user sends as well as the number of invitations that are accepted.

Psiphon logs aggregate usage statistics on a per in-proxy as well as country/region basis. Psiphon collects domain names visited through the service, the number of visit per domain name, and the number of logins per year/month/day. Psiphon maintains server logs collected on the inproxy (which contain users' IP addresses and obfuscated requested URLs) and server (which contain the inproxy IP addreses and requested URLs) as well as sendmail logs that record the e-mail addresses of invitation recipients.

When necessary in order to troubleshoot its services, Psiphon may temporarily alter is standard practices in order to record usage information in a way that correlates a user’s account with the websites that he or she visits. This will only be done for a limited period of time and information retained in this way will be deleted after it has been analyzed.  Please read our Privacy Policy and Design Overview paper for more information.

Use of Psiphon

By accessing and invoking the Psiphon service, the user warrants that he / she will abide by the following terms of use:

• The user will not utilize Psiphon in ways that are prohibited or restricted by Canadian law including, but not limited to, accessing child pornography, engaging in fraud.

• The user will not utilize Psiphon in a manner that contravenes the right of other users to access the Psiphon service, by publicly disclosing in-proxy addresses and/or exploiting or otherwise over-loading Psiphon servers.

Acceptance of Risk

The user agrees that a degree of risk is associated with the use of any service provided by Psiphon Inc., and accepts that risk as a result of the usage of Psiphon. We strongly recommend that you read our Design Overview document to help you decide if Psiphon is right for you.

Psiphon is intended for non-technical users seeking access to denied content. Psiphon traffic is encrypted to avoid blocking but is unencrypted at the in-proxy level before it is re-encrypted and either passed on to the Psiphon server (for outgoing requests), or the end user (for returned requests).

Psiphon should not be construed as a way of enhancing the privacy or security of your Internet connection. Psiphon does not provide anonymity and does not provide protection from traffic analysis attacks. Psiphon does not disguise the fact that you are using Psiphon.

If you are engaged in politically sensitive behaviour and are at risk of high levels of surveillance, be aware that Psiphon does not have ability to defend you from such attacks.

Limitations of Liability

The user agrees that by utilizing Psiphon, he / she is using the service “as is” and accepts all consequences of doing so. Under no circumstances will Psiphon be held responsible for any way - for direct, indirect and consequential damages - by the user or any third party, even if Psiphon has been advised of the possibility that such damages may occur. Such damages include, but ARE NOT limited to: all personal damages such as reputation; lost data; lost business revenue; failure to realize expected profits or savings or other commercial or economic loss of any kind arising out of the use, or inability to use, the software.

Last updated: 2009-06-03")?></textarea></td>
            </tr>
            <br><br>
            <tr><td>
                <table>
                    <tr>
                        <td>
                            <form name="terms" method="post" action="<?=escape_html(getenv("REQUEST_URI"))?>">
                                <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden">
                                <input name="terms_submit_agree" value="1" type="hidden">
                                <input type="submit" value="<?=___("Agree")?>">
                            </form>
                        </td>
                        <td>
                            <form name="terms" method="post" action="/logout.php?lu=<?=$login_url?>">
                                <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden">
                                <input type="submit" value="<?=___("Don't Agree")?>">
                            </form>
                        </td>
                    </tr>
                </table>
            </td></tr>

        </table>

        <br>
        <br>

        <?echo_language_buttons($config, $record_user, "login_lang", 6);?>

    </body>
</html>
<?
    // Don't proceed with the screen that included check_user.php...

    exit(0);
}

// Proceed with the screens that include check_user.php...

?>
