<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// e.php - email verification
//
// expects an argument of the form token=9dc3a577a7246a0ef823b6dfb072f65e
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/cr.php");

$user_record = lookup_user(
                $config,
                "email_candidate_token = :email_candidate_token and email_candidate_date >= UNIX_TIMESTAMP() - '{$config["email_candidate_ttl"]}'",
                array(":email_candidate_token" => $_GET[token]));

if (!$user_record)
{
    // sleep to slow down brute force attacks
    // no specific error for bad token, just HTTP 404
    sleep(1);
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}

// for lang.php/msg

$record_user = $user_record;

// check accessing proxy matches proxy specified in user record

$param_string = "";
$param_array = array();
string_list_to_params(join(",", $user_record['assigned_proxies']), ",", "ap_", $param_string, $param_array);

$query = "SELECT id, hostname FROM proxy WHERE id IN (".$param_string.")";

$result = db_query_execute($config, $query, convert_null_array($param_array));

$proxy_valid = False;
while ($proxy_record = db_fetch_result($config, $result))
{
    if ($proxy_record['hostname'] == getenv("HTTP_HOST"))
    {
        $proxy_valid = True;
        break;
    }
}
db_free_result($result);

if (!$proxy_valid)
{
    // sleep to slow down brute force attacks
    // no specific error for bad token, just HTTP 404
    sleep(1);
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}

// Promote the email candidate to current email address.  Since the email
// field is subject to uniqueness constraints and the candidate is not (to
// prevent email probing attacks when creating account), there's a chance
// the promotion will encounter a conflict due to violation of this constraint.
//
// Because we assume that the user owns his email address if it's able to validate
// it, we will deal with uniqueness conflicts by pre-clearing that email address 
// from another other records where it may appear. That way a user can re-use his 
// address.
//
// However, clearing the email address and creating a new account essentially
// destroys the old account (the user may have used their email to log into it),
// losing bookmarks and accrued invites (etc.). So we'll detect if another account
// with the email exists, tell the user, and prompt to ensure the user really
// wants to proceed with the verification/clearing.

// Look for an account that already has that email address.
$conflicting_user_record = Null;
if (!$_POST['submitted'])
{
    $conflicting_user_record = lookup_user(
                                $config,
                                "email = :email",
                                array(":email" => $user_record['email_candidate']));
}

// If there's no conflict, or if the user has decided to verify, do the verification work.
if (!$conflicting_user_record || $_POST['submitted'])
{
    // Do a pre-emptive clearing of the email address from any other user records.

    $query = "UPDATE user SET email = NULL WHERE email = :email";
    db_query_execute($config, $query, convert_null_array(array(":email" => $user_record['email_candidate'])));

    // Promote the candidate email (we know there'll be no uniqueness conflict
    // (ignoring non-transactional tomfoolery)).

    $query = "UPDATE IGNORE user SET ".
                "email = :email, ".
                "email_candidate = NULL, ".
                "email_candidate_token = NULL ".
             "WHERE id = :user_id";

    db_query_execute(
        $config,
        $query,
        convert_null_array(array(
            ":email" => $user_record['email_candidate'],
            ":user_id" => $user_record['id'])));

    $processing_results[] = sprintf(___("Thank you.  Your e-mail address, %s, has been verified."), $user_record['email_candidate']);

    // send (enqueue) welcome email to user's now-verified email address at this time

    enqueue_welcome_email($config, $user_record['id']);
}

// If the user is logged in, show them a browsing link. If not, show them a login link.
$user_logged_in =
    (!empty($user_record['session'])
     && $user_record['session'] == $_COOKIE['p']
     && !$user_record['locked']
     && $user_record['agreed_to_usage_terms']);
//echo "<pre>";print_r($user_record);echo "\n";print_r($_COOKIE);exit();
?>
<?/*
 *
 * Presentation Layer
 *
 */?>
<?=header("Content-type: text/html; charset=".charset());?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
    <head>
        <title><?=___("E-mail Verification")?></title>
        <?=echo_css()?>
    </head>
    <body style="direction: <?=direction()?>" class="sst">
        <?=echo_processing_results_and_errors();?>

        <? if ($conflicting_user_record) { /* show a prompt to the user before verifying */ ?>
            <p><?=___("The e-mail address that you are verifying is already associated with an existing account.<br/>If you choose to Verify, the address will be removed from that other account.<br/>Alternatively, you can log into Psiphon with the other account.")?></p>
            <form name="profile" method="post" action="<?=getenv("REQUEST_URI")?>">
                <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden"/>
                <input name="submitted" value="1" type="hidden"/>
                <input type="submit" value="<?=___("Verify")?>"/>
                &nbsp;
                <a href="<?=make_home_url($config, $conflicting_user_record['current_proxy'])?>"><?=___("Log in with other account")?></a>
            </form>
        <? } else { ?>
            <? if ($user_logged_in) { ?>
                <a href="<?=make_proxied_url("")?>"><?=___("Browse")?></a><br><br>
            <? } else { ?>
                <a href="<?=make_home_url($config, $proxy_record['id'])?>"><?=___("Login")?></a><br><br>
            <? } ?>
        <? } ?>
    </body>
</html>
