<?php
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

include_once($_SERVER[DOCUMENT_ROOT]."/includes/sendmail.php");

// Helper: enqueue an email which will be sent by email daemon
function enqueue_email(
            $config,
            $email,
            $subject,
            $body,
            $lang)
{
    // Send this email.
    
    if (!smail_plain(
            escape_smtp($email),
            "",
            escape_smtp($config["invitation_sender"]),
            "",
            "",
            escape_smtp($subject),
            escape_smtp($body),
            charset($lang)))
    {
        return false;
    }

    // TODO: none of the code that triggers email checks if it succeeds.  It never did -- and can never
    // know for sure if it's delivered; we could at least report an error to the user if the enqueue fails;
    // or is this an internal issue?  Do we need to log something to syslog here or will the email
    // daemon setup take care of that?

    return true;
}

// Helper: enqueue email verification email
function enqueue_email_candidate_email(
            $config,
            $username,
            $user_proxy_id,
            $user_lang,
            $email_candidate,
            $email_candidate_token)
{
    /* There are two different emails that may be sent. If the specified email
     * address *is not* in use, then a very simple verification email is sent.
     * If the specified email address *is* is in , then the email will also tell
     * the user that they already have an account, that they can log in using that
     * email address, and which proxy they can use to log in through.
     */

    // Look for a pre-existing user with the specified email address.
    $user_record = lookup_user($config, "user.email = :email", array(":email" => $email_candidate));

    $url = make_email_verification_url($config, $user_proxy_id, $email_candidate_token);

    // generate the email subject and body

    $subject = ___("Verify your Psiphon e-mail address", NULL, $user_lang);

    $body = "";
    if (!$user_record)
    {
        $body = sprintf(___("Your e-mail address (%s) has been registered with the Psiphon account %s.  
If this is correct, please activate the registration by clicking the following link: %s.", NULL, $user_lang),
                        $email_candidate, $username, $url);
    }
    else
    {
        $body = sprintf(
                    ___("Your e-mail address (%s) has been registered with the Psiphon account %s.  
If this is correct, please activate the registration by clicking the following link: %s.

Note that this e-mail address is currently associated with another account. If you verify it, you will be removing it from that other account. If instead you wish to log in using the other account, you may do so by logging in at:
%s", NULL, $user_lang),
                    $email_candidate,
                    $username,
                    $url,
                    make_home_url($config, $user_record['current_proxy']));
    }

    // enqueue email

    enqueue_email($config, $email_candidate, $subject, $body, $user_lang);
}

// Helper: enqueue welcome email
function enqueue_welcome_email($config, $user_id)
{
    // welcome email is only sent once in account lifetime, so check
    // if it's already sent

    $user_record = lookup_user($config, "user.id = :user_id", array(":user_id" => $user_id));
    if (!$user_record)
    {
        // user id should be valid
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
    }

    if ($user_record[welcome_email_sent])
    {
        return;
    }

    $url = make_home_url($config, $user_record['current_proxy']);

    // generate the email subject and body

    $subject = ___("Welcome to your Psiphon account", NULL, $user_record[last_lang]);
    $body = sprintf(___("Welcome to your Psiphon account!

Do not share this link or your Psiphon account login details on any public forum.  This will help protect Psiphon from being blocked in your region.

You may encounter an \"invalid security certificate\" warning when you activate or access your Psiphon account.  Please select \"continue\" or \"add an exception\" to enable your browser to establish a secure connection to Psiphon.

You must use the following link to login to Psiphon:

%s

The Psiphon Team", NULL, $user_record[last_lang]), $url);

    // enqueue email

    enqueue_email($config, $user_record[email], $subject, $body, $user_record[last_lang]);

    // mark welcome as sent so we don't send it again
    // (usual transaction disclaimer here: email enqueue can succeed
    // and then this flag set fails)

    $query = "UPDATE user SET welcome_email_sent = 1 WHERE id = :user_id";
    db_query_execute($config, $query, convert_null_array(array(":user_id" => $user_id)));
}

?>
