<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

if($config["geoip_type"] == "city")
{
    include($_SERVER[DOCUMENT_ROOT]."/includes/geoipcity.inc");
}
else //if($config["geoip_type"] == "country")
{
    include($_SERVER[DOCUMENT_ROOT]."/includes/geoip.inc");
}

function lookup_region($config)
{
    // get IP from in-proxy custom header or remote host if no header

    if (!($ip_address = $_SERVER["HTTP_X_FORWARDED_FOR"]))
    {
        $ip_address = $_SERVER["REMOTE_ADDR"];
    }

    // GeoIP lookup

    $geoip_file = $config["geoip_data_file"];

    if (file_exists($geoip_file))
    {
        if($config["geoip_type"] == "city"  && function_exists("geoip_record_by_addr"))
        {
            $gi = geoip_open($geoip_file, GEOIP_STANDARD);
            $gi_record = geoip_record_by_addr($gi, $ip_address);
            $last_region = $gi_record->country_code;
            geoip_close($gi);
        }
        else if($config["geoip_type"] == "country"  && function_exists("geoip_country_code_by_addr"))
        {
            $gi = geoip_open($geoip_file, GEOIP_STANDARD);
            $last_region = geoip_country_code_by_addr($gi, $ip_address);
            geoip_close($gi);
        }
    }
    else
    {
        $last_region = "";
    }

    return $last_region;
}

function update_login_stats($config, $last_region, $proxy, $grp)
{
    // update login stats
    //
    // This stats entry is unique for the region, proxy, current day, and user group.  If an
    // existing record is found for that key, the counter is incremented.  Otherwise, a new
    // record is created.
    // A snapshot of the proxy's notes is taken so that reporting based on note keyword
    // matching will reflect the keywords in place on the day of the event.  Note
    // the COALESCE as certain users may not have a valid assigned proxy id.

    $date = mktime(0, 0, 1, date("m"), date("j"), date("Y"));

    $query = "UPDATE stat_region SET cnt=cnt+1, proxy_notes = (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')) ".
             "WHERE date = :date AND region = :last_region AND proxy = :proxy_id AND grp = :grp";
    $result = db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));

    if (!db_num_rows_affected($result))
    {
        $query = "INSERT INTO stat_region (date, region, proxy, proxy_notes, grp, cnt) ".
                 "VALUES (:date, :last_region, :proxy_id, (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')), :grp, 1)";
        db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));
    }
}

function update_site_request_stats($config, $domain, $last_region, $proxy, $grp)
{
    // update site request stats
    //
    // NOTE: this stat is now *page views*
    //
    // See comment above.  Has an additional key attribute, the site visited.

    $date = mktime(0, 0, 1, date("m"), date("j"), date("Y"));

    $query = "UPDATE stat_site SET cnt=cnt+1, proxy_notes = (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')) ".
             "WHERE date = :date AND domain = :domain AND region = :last_region AND proxy = :proxy_id AND grp = :grp";
    $result = db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":domain" => $domain, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));

    if (!db_num_rows_affected($result))
    {
        $query = "INSERT INTO stat_site (date, domain, region, proxy, proxy_notes, grp, cnt) ".
                 "VALUES (:date, :domain, :last_region, :proxy_id, (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')), :grp, 1)";
        db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":domain" => $domain, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));
    }
}

function update_blue_bar_submit_stats($config, $domain, $last_region, $proxy, $grp)
{
    // update blue bar submit stats
    //
    // See comment above.  Has an additional key attribute, the site visited.

    $date = mktime(0, 0, 1, date("m"), date("j"), date("Y"));

    $query = "UPDATE stat_blue_bar_submit SET cnt=cnt+1, proxy_notes = (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')) ".
             "WHERE date = :date AND domain = :domain AND region = :last_region AND proxy = :proxy_id AND grp = :grp";
    $result = db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":domain" => $domain, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));

    if (!db_num_rows_affected($result))
    {
        $query = "INSERT INTO stat_blue_bar_submit (date, domain, region, proxy, proxy_notes, grp, cnt) ".
                 "VALUES (:date, :domain, :last_region, :proxy_id, (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')), :grp, 1)";
        db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":domain" => $domain, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));
    }
}

function update_account_created_stats($config, $last_region, $proxy, $grp)
{
    // update account created stats
    //
    // See comment above.

    $date = mktime(0, 0, 1, date("m"), date("j"), date("Y"));

    $query = "UPDATE stat_account_created SET cnt=cnt+1, proxy_notes = (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')) ".
             "WHERE date = :date AND region = :last_region AND proxy = :proxy_id AND grp = :grp";
    $result = db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));

    if (!db_num_rows_affected($result))
    {
        $query = "INSERT INTO stat_account_created (date, region, proxy, proxy_notes, grp, cnt) ".
                 "VALUES (:date, :last_region, :proxy_id, (SELECT COALESCE((SELECT notes FROM proxy where id = :proxy_id),'')), :grp, 1)";
        db_query_execute($config, $query, convert_null_array(array(":date" => $date, ":last_region" => $last_region, ":proxy_id" => $proxy, ":grp" => $grp)));
    }
}

function get_country_name_from_country_code($country_code)
{
    $geoip = new GeoIP();

    $country_number = $geoip->GEOIP_COUNTRY_CODE_TO_NUMBER[$country_code];
    if (is_null($country_number))
    {
        return ___("Unknown region");
    }

    $country_name = $geoip->GEOIP_COUNTRY_NAMES[$country_number];
    if (is_null($country_name))
    {
        return ___("Unknown region");
    }

    return $country_name;
}

?>
