<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*********************************************************************************
NAME
	smail(), smail_html_only(), smail_html() -- send mail functions

SYNOPSIS
	include("/home/httpd/common/sendmail.php");

	bool smail_plain($to_addr, $to_name, $from_addr, $from_name, $reply_to, $subject, $body);
	bool  smail_html($to_addr, $to_name, $from_addr, $from_name, $reply_to, $subject, $body);
	bool  smail_both($to_addr, $to_name, $from_addr, $from_name, $reply_to, $subject, $body, $body_html);


DESCRIPTION
	The functions smail() sends an email to address {$to_addr}, adding {$to_name} into "To: "
	header line. Sender's address should be placed in {$from_addr}, {$from_name} will be added
	to "From: " header line. {$from_addr} field MUST contain verified address. If you don't
        have any - place your own.

	Only one recipient can be specified.

	{$to_name} and {$from_name} fields are optional (may contain an empty string). This fields
	CANNOT contain any "new line", "TAB" or "cartrige return" characters. It's better to
	keep them less than 60 bytes.

	{$reply_to} field may contain address recipient can reply to, this field is optional and can be
	empty.

	{$subject} field should contain subject of message, this field is optional. It CANNOT
	contain any "new line", "TAB" or "cartrige return" characters. It's better to keep
	this fields less than 60 bytes.

	{$body} filed contains plain text message body.
	{$body_html} filed contains html text message body.

	There is header "X-url-submited-from: ". It contains url of page from which this function
	was called. This header will be added to message only in case this function was called from
	script called from some web-page. In case of CLI, this header will be skipped. This header
	will be added automatically.

RETURN VALUES
	Upon successful completion this functions return TRUE, otherwise, FALSE is returned.

NOTES
	If you got TRUE, it means that only local mail server have accepted this message for
	delivery, and there is no warranty for this message will be delivered. In case of
	error, email with error message will be returned to address specified in {$from_addr} field.

SEE ALSO
	DARPA Internet Request For Comments RFC819, RFC821, RFC822.

BUGS
	If you found any, please, let me know: mailto:v@ezd.com

*********************************************************************************/

$smtp_host = "localhost";
$smtp_port = 25;
$smtp_timeout = 3;
$smtp_max_line = 256;

if (!empty($_SERVER["HTTP_HOST"]))
{
    $smtp_helo = $_SERVER["HTTP_HOST"];
}
else if (!empty($_SERVER["SERVER_NAME"]))
{
    $smtp_helo = $_SERVER["SERVER_NAME"];
}
else if (!empty($_SERVER["HOST"]))
{
    $smtp_helo = $_SERVER["HOST"];
}
else
{
    $smtp_helo = "host.domain";
}


include_once($_SERVER[DOCUMENT_ROOT]."/includes/validators.php");

function smail_clean_header($str)
{
    $str = str_replace("\n", " ", $str);
    $str = str_replace("\r", " ", $str);
    $str = str_replace("\"", "'", $str);
    $str = str_replace("<", "(", $str);
    $str = str_replace(">", ")", $str);
    return($str);
}

function smail_plain($smtp_to_addr, $smtp_to_name, $smtp_from_addr, $smtp_from_name, $smtp_reply_addr, $smtp_subject, $m_body, $encoding="")
{
    global $smtp_host;
    global $smtp_port;
    global $smtp_timeout;
    global $smtp_max_line;
    global $smtp_helo;

    if (!is_email($smtp_to_addr))
    {
        syslog(LOG_INFO, "{$smtp_helo}: smail(): bad recipient address: {$smtp_to_addr}");
        return(false);
    }

    if (!is_email($smtp_from_addr))
    {
        syslog(LOG_INFO, "{$smtp_helo}: smail(): bad sender address: {$smtp_from_addr}");
        return(false);
    }

    if (!$smtp_reply_addr || !is_email($smtp_reply_addr))
    {
        $smtp_reply_addr=$smtp_from_addr;
    }

    $smtp_to_name = smail_clean_header($smtp_to_name);
    $smtp_from_name = smail_clean_header($smtp_from_name);
    $subject = smail_clean_header($subject);

    if (!($fp = @fsockopen($smtp_host, $smtp_port, $errno, $errstr, $smtp_timeout)))
    {
        syslog(LOG_ERR, "{$smtp_helo}: smail() error in fsockopen(): {$errstr} ({$errno})");
        return(false);
    }

    $r = false;
    while (1)
    {
        $s = fgets($fp, $smtp_max_line);
        if (strncmp($s, "220 ", 4))
        {
            syslog(LOG_INFO, "{$smtp_helo}: smail() error in greeting: {$s}");
            break;
        }

        fputs($fp, "helo {$smtp_helo}\r\n");
        $s = fgets($fp, $smtp_max_line);
        if (strncmp($s, "250 ", 4)) 
        {
            syslog(LOG_INFO, "{$smtp_helo}: smail() error in \"helo {$smtp_helo}\": {$s}");
            break;
        }

        fputs($fp, "mail from: {$smtp_from_addr}\r\n");
        $s = fgets($fp, $smtp_max_line);
        if (strncmp($s, "250 ", 4)) 
        {
            syslog(LOG_INFO, "{$smtp_helo}: smail() error in \"mail from: {$smtp_from_addr}\": {$s}");
            break;
        }

        fputs($fp, "rcpt to: {$smtp_to_addr}\r\n");
        $s = fgets($fp, $smtp_max_line);
        if (strncmp($s, "250 ", 4)) 
        {
            syslog(LOG_INFO, "{$smtp_helo}: smail() error in \"rcpt to: {$smtp_to_addr}\": {$s}");
            break;
        }

        fputs($fp, "data\r\n");
        $s = fgets($fp, $smtp_max_line);
        if (strncmp($s, "354 ", 4)) 
        {
            syslog(LOG_INFO, "{$smtp_helo}: smail() error in \"data\": {$s}");
            break;
        }

        if (strlen($smtp_to_name))
        {
            fputs($fp, "To: \"{$smtp_to_name}\" <{$smtp_to_addr}>\r\n");
        }
        else
        {
            fputs($fp, "To: {$smtp_to_addr}\r\n");
        }

        if (strlen($smtp_from_name))
        {
            fputs($fp, "From: \"{$smtp_from_name}\" <{$smtp_reply_addr}>\r\n");
        }
        else
        {
            fputs($fp, "From: {$smtp_reply_addr}\r\n");
        }

        if ($encoding)
        {
            fputs($fp, "Content-type: text/plain; charset=\"{$encoding}\"\r\n");
        }

        if ($_SERVER[HTTP_HOST] && $_SERVER[REQUEST_URI])
        {
            fputs($fp, "X-url-submited-from: http://{$_SERVER[HTTP_HOST]}{$_SERVER[REQUEST_URI]}\r\n");
        }

        if ($smtp_subject)
        {
            fputs($fp, "Subject: {$smtp_subject}\r\n");
        }        
        fputs($fp, "\r\n");

        fputs($fp, $m_body);
        fputs($fp, "\r\n.\r\n");

        $s = fgets($fp, $smtp_max_line);
        if (strncmp($s, "250 ", 4)) 
        {
            syslog(LOG_INFO, "{$smtp_helo}: smail() error after \".\": {$s}");
            break;
        }

        $r = true;
        break;
    }
    
    fputs($fp, "quit\r\n");
    fclose($fp);
    return($r);
}

?>
