<?php
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function get_proxy_host($config, $proxy_id)
{
    $proxy_record = lookup_proxy($config, $proxy_id);

    if (!$proxy_record)
    {
        // there are some possible fringe cases where this will happen. since
        // we don't enforce FK constraints, for example, you can have guest
        // browsing records for non-existant proxies
        //http://en.wikipedia.org/wiki/Top_level_domain#Reserved_domains
        return "example.com";
    }

    if ($proxy_record['hostname'] && strlen($proxy_record['hostname']) > 0)
    {
        return $proxy_record['hostname'];
    }
    return "example.com";
    
}

function get_proxy_login_url($config, $proxy_id)
{
    $proxy_record = lookup_proxy($config, $proxy_id);

    if (!$proxy_record)
    {
        // there are some possible fringe cases where this will happen. since
        // we don't enforce FK constraints, for example, you can have guest
        // browsing records for non-existant proxies
        //http://en.wikipedia.org/wiki/Top_level_domain#Reserved_domains
        return "example.com";
    }

    if ($proxy_record['hostname'] && strlen($proxy_record['hostname']) > 0)
    {
        $ret = $proxy_record['hostname'];
    }
    else
    {
        $ret = long2ip($proxy_record['addr']);
    }
    
    if ($proxy_record['login_url'] && strlen($proxy_record['login_url']) > 0)
    {
        $ret .= $proxy_record['login_url'];
    }

    return $ret;
}

function make_home_url($config, $proxy_id)
{
    // URL --> https://<proxy ip address|hostname>/login_url

    return "https://".get_proxy_login_url($config, $proxy_id);
}

function make_anonymous_browsing_url($config, $proxy_id, $token)
{
    // URL --> https://<proxy ip address|hostname>/a.php?token=01234567890123456789

    return "https://".get_proxy_host($config, $proxy_id)."/a.php?"."token=".$token;
}

function make_email_verification_url($config, $proxy_id, $token)
{
    // URL --> https://<proxy ip address|hostname>/e.php?token=01234567890123456789

    return "https://".get_proxy_host($config, $proxy_id)."/e.php?"."token=".$token;
}

function make_invite_url($config, $proxy_id, $token)
{
    // URL --> https://<proxy ip address|hostname>/w.php?p=01234567890123456789

    return "https://".get_proxy_host($config, $proxy_id)."/w.php?"."p=".$token;
}

function make_proxied_url($url, $encode = 0)
{
    if($encode)
    {
        return $_SERVER['psiphon_flash_prefix']."/".base64_encode($url);
    }
    return $_SERVER['psiphon_html_prefix']."/".$url;

}

function make_password_reset_url($config, $proxy_id, $token) {
    // URL --> https://<proxy ip address|hostname>/p.php?token=01234567890123456789

    return "https://".get_proxy_host($config, $proxy_id)."/p.php?"."token=".$token;
}

?>
