<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function is_login_url($str) //only [a-z], [0-9] and '/' allowed
{
    if (!$len=strlen($str)) return(false);
    for ($i=1; $i<$len-1; $i++)
    {
        if (($str{$i}<'a' || $str{$i}>'z') && ($str{$i}<'0' || $str{$i}>'9') && ($str{$i}!='/')) return(false);
    }
    return(true);
}

function is_subdomain($str) {
if (!strlen($str)) return(false);
$str=strtolower($str);
if (($str{0}<'a' || $str{0}>'z') && ($str{0}<'0' || $str{0}>'9')) return(false);
$len=strlen($str);
if (($str{$len-1}<'a' || $str{$len-1}>'z') && ($str{$len-1}<'0' || $str{$len-1}>'9')) return(false);
for ($i=1; $i<$len-1; $i++) if (($str{$i}<'a' || $str{$i}>'z') && ($str{$i}<'0' || $str{$i}>'9') && ($str{$i}!='-')) return(false);
return(true);
}

function is_domain($str) {
if (!strlen($str)) return(false);
$subs=explode(".", $str);
if (count($subs)<2) return(false);
foreach ($subs as $sub) if (!is_subdomain($sub)) return(false);
return(true);
}

function is_user($str) {
if (!strlen($str)) return(false);
$str=strtolower($str);
if (($str{0}<'a' || $str{0}>'z') && ($str{0}<'0' || $str{0}>'9') && ($str{0}!='_')) return(false);
$len=strlen($str);
for ($i=1; $i<$len; $i++) if (($str{$i}<'a' || $str{$i}>'z') && ($str{$i}<'0' || $str{$i}>'9') && ($str{$i}!='.') && ($str{$i}!='_') && ($str{$i}!='-') && ($str{$i}!='+')) return(false);
return(true);
}

function is_email($addr) {
if (!strlen($addr)) return(false);
if (!($at=strpos($addr, "@"))) return(false);
if (!is_user(substr($addr, 0, $at)) || !is_domain(substr($addr, $at+1))) return(false);
return(true);
}

// check that password is minimum length and doesn't contain username
function is_valid_password($config, $passwd, $uname)
{
    if ($config['min_pass_length'] == Null)
    {
        // $config must be defined.
        return false;
    }

    // minimum length check
    if (strlen($passwd) < $config['min_pass_length'])
    {
        return false;
    }

    // username substring check
    if (strstr($passwd, $uname))
    {
        return false;
    }

    // TODO:
    // require 3-of-4 character classes?
    // dictionary word check?

    return true;
}

// URL validation
// - stricter than filter_var(FILTER_VAR_URL)
function is_url($url)
{
    // The regex is from here: http://www.phpcentral.com/208-url-validation-php.html
    $urlregex = "^(https?|ftp)\:\/\/([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?[a-z0-9+\$_-]+(\.[a-z0-9+\$_-]+)*(\:[0-9]{2,5})?(\/([a-z0-9+\$_-]\.?)+)*\/?(\?[a-z+&\$_.-][a-z0-9;:@/&%=+\$_.-]*)?(#[a-z_.-][a-z0-9+\$_.-]*)?\$";
    return eregi($urlregex, $url);
}

// Escape helper: iterates over all fields in a nested array structure
function escape_helper($fields, $escape_func)
{
    if (!$fields)
    {
        return $fields;
    }

    // Also make this function work for simple, non-array inputs
    if (!is_array($fields))
    {
        return $escape_func($fields);
    }

    foreach ( $fields as $key => $val )
    {
        if (is_array($val))
        {
            // Recursive call
            $fields[$key] = escape_helper($val, $escape_func);
        }
        else
        {
            $fields[$key] = $escape_func($val);
        }
    }

    return $fields;
}

function escape_smtp_func($val)
{
    // blank line with just "." is end of message, and injecting this input
    // can be used to submit a second email message of attacker's choosing
    // to the SMTP server.
    // Note: should also not allow newlines in subject, but this is already
    // handled in sendmail.php, and it's tricky to apply that to subject but
    // not body in this generic escape function...

    $val=str_replace("\r.\r", "\r>.\r", $val);
    $val=str_replace("\r.\n", "\r>.\n", $val);
    $val=str_replace("\n.\r", "\n>.\r", $val);
    $val=str_replace("\n.\n", "\n>.\n", $val);

    $val=str_replace("\nFrom", "\n>From", $val);
    $val=str_replace("\rFrom", "\r>From", $val);

    return($val);

    // regex not available...
    // return preg_replace("/(^\.$)|(^\.\r\n)/m"," ",$val);
}

// Takes an array of values (may contain subarrays) and removes content unsafe for SMTP
function escape_smtp($fields)
{
    return escape_helper($fields, escape_smtp_func);
}

function escape_html_func($val)
{
    return htmlentities($val, ENT_QUOTES, charset());
}

// Takes an array of values (may contain subarrays) and calls htmlentities on all fields.
function escape_html($fields)
{
    return escape_helper($fields, escape_html_func);
}

// If $int isn't empty/null, cast it to int (to get rid of any illegal non-int values).
// If a simple cast is done without the Null check, the null-ness will be lost and be replaced with 0.
function cast_str_to_int($int_str)
{
    return (strlen($int_str) ? (int)$int_str : Null);
}

function is_hostname($str)
{
   //http://stackoverflow.com/questions/106179/regular-expression-to-match-hostname-or-ip-address
    
    $ValidIpAddressRegex = "^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$";
    $ValidHostnameRegex = "^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$";
    return (eregi($ValidIpAddressRegex, $str) || eregi($ValidHostnameRegex, $str));
}
?>
