<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// invite-m.php
//
// This screen is used to invite new users to a proxy via an email.
// 

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "invite"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

$absolute_max_invite_count = 100;
$max_invite_count = ($record_user['invite_token_count'] == Null ? $absolute_max_invite_count : min($record_user['invite_token_count'], $absolute_max_invite_count));

// Not all controls are shown or editable depending on user classes/permissions.
$proxy_hidden = (!user_has_permission($config, $record_user, 'multi_proxy')
                 && !user_has_permission($config, $record_user, 'other_nodes'));
$user_class_hidden = ($record_user['grp'] == get_normal_user_class_id());
// HACK: If the user can't invite Power Users, then they likely can only invite
// a single user class -- normal Users -- and therefore the control should be disabled.
$user_class_readonly = !user_has_permission($config, $record_user, 'can_grant_grp_'.get_power_user_class_id());

// process invite form

while ($_POST['submit_invite'])
{
    // Make sure the current user has access to the selected proxy.
    if (!can_current_user_access_proxy($config, $record_user, $_POST['proxy']))
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }

    $_POST['user_class'] = cast_str_to_int($_POST['user_class']);

    // Make sure that the user class selected is legitimate.
    if (!can_user_manage_class($config, $record_user, $_POST['user_class']))
    {
        // Trying to use a higher-permission user type.
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }

    // Convert the newline-separated string of addresses into an array.
    $addr_list = explode("\n", $_POST['addr_list']);
    if (!count($addr_list))
    {
        $processing_errors[] = ___("No valid e-mail address was provided");
        $curs = "invite.addr_list";
        break;
    }

    // Validate default homepage url
    $default_homepage_url = $_POST['default_homepage'];
    if ( ! $default_homepage_url ) {
        $default_homepage_url = '';
    }

    if ( ! is_url($default_homepage_url) && $default_homepage_url != '' )
    {
        $processing_errors[] = ___("Bad URL");
        break;
    }

    // Validate the invitation count.

    if ((count($addr_list) <= 0) || (count($addr_list) > $max_invite_count))
    {
        $processing_errors[] = sprintf(___("Invalid invite count (must be between 1 and %d)"), $max_invite_count);
        $curs = "invite.addr_list";
        break;
    }

    $count = 0;

    // Illegal addresses will be re-added to this below, so clear it out.
    // (We also don't want to re-populate the field with addresses that
    // have already been sent to.)
    $_POST['addr_list'] = "";

    // Iterate through each email address, create invitation, and enqueue email.
    
    foreach ($addr_list as $addr)
    {
        // We can't send to illegal email addresses, so re-populate the list
        // with them after sending to the legal ones.
        
        if (!is_email($addr=trim($addr)))
        {
            if (strlen($addr))
            {
                $_POST['addr_list'] .= $addr."\n";
            }
            continue;
        }

        // We will insert into the invitation table before inserting into the
        // email_queue table. Because we aren't using transactions, this means
        // that there may be dangling invitations (ones with no corresponding
        // email). This is better than dangling emails.

        // Create the invitation code.
        //
        // We don't want the invitation code to be guessable within the amount of time
        // that the code is valid.
        $invite_code_attack_count =
            (float)                     // avoid overflow
            $config['invitation_ttl']   // number of seconds that an invite code is valid
            * 20;                        // number of attacks per second (testing shows 50ms per http attack)
        // We want no better than 1-in-200 chance of attack success in the average case (so multiply by 2).
        $invite_code_attack_count = $invite_code_attack_count * 200 * 2;
        // Determine the number of bytes of random data needed to make our invite code as strong as we want it.
        $random_hex_needed = ceil(log($invite_code_attack_count, 16));

        $invite_code = strtoupper(substr(bin2hex(secure_rand(ceil($random_hex_needed/2.0))), 0, $random_hex_needed));
        $url = make_invite_url($config, (int)$_POST['proxy'], $invite_code);
        $invitation_expiration_time = $config["invitation_ttl"]/$config["seconds_in_day"];
        $body = sprintf(___("Welcome to Psiphon!

Note: This invitation expires in %d days.

Do not share the following link or your Psiphon account login details on any public forum.  This will help protect Psiphon from being blocked in your region.

You may encounter an \"invalid security certificate\" warning when you activate or access your Psiphon account.  Please select \"continue\" or \"add an exception\" to enable your browser to establish a secure connection to Psiphon.

Click this link to create your new Psiphon account:

%s

The Psiphon Team"), $invitation_expiration_time, $url);
        $subject = ___("Your Psiphon Invitation");
        $proxy = $_POST['proxy'];

        // Insert a record into the invitation table.

        $query=
            "INSERT INTO invitation (date, proxy, pass, user, email, status, url, lang, grp, homepage) ".
            "VALUES (UNIX_TIMESTAMP(), :proxy, :pass, :user, :email, -1, :url, :lang, :grp, :homepage)";
        db_query_execute(
            $config,
            $query,
            convert_null_array(array(
                ":proxy"    => $proxy,
                ":pass"     => $invite_code,
                ":user"     => $record_user['id'],
                ":email"    => $addr,
                ":url"      => $url,
                ":lang"     => $record_user['last_lang'],
                ":grp"      => $_POST['user_class'],
                ':homepage' => $default_homepage_url
            )));

        // Get the id of the invitation table entry we just created.
        $invitation_id = db_get_last_insert_id($config);

        // Send invite email.

        if (enqueue_email($config, $addr, $subject, $body, $record_user['last_lang']))
        {
            // Update the invitation table to reflect that the email has been sent.
            $query =
                "UPDATE invitation ".
                "SET status = UNIX_TIMESTAMP() ".
                "WHERE id = :invitation_id";
            db_query_execute($config, $query, convert_null_array(array(":invitation_id" => $invitation_id)), true);
        }

        $count++;
    }

    if ($count > 0)
    {
        // Some invitation emails were sent.

        if (strlen($_POST['addr_list']))
        {
            $processing_errors[] = ___("Some invalid e-mail addresses were provided");
        }

        // Increment the current user's invitation-sent and invitation-token info in the DB.

        // The stuff that comes from config doesn't need to be parameterized.
        $query =
            "UPDATE user SET ".
            "inv_sent = inv_sent + :count, ".
            "last_token_count = ".last_token_count_sql($config, $count).", ".
            "last_token_accrual_time = ".last_token_accrual_time_sql($config, $record_user)." ".
            "WHERE id = :id";

        db_query_execute($config, $query, convert_null_array(array(":count" => $count, ":id" => $record_user['id'])));

        // We have updated the current user record and need to update $record_user.
        $record_user = lookup_user($config, "id = :user_id", array(":user_id" => $record_user['id']));

        // And we need to update $max_invite_count.
        $max_invite_count = ($record_user['invite_token_count'] == Null ? $absolute_max_invite_count : min($record_user['invite_token_count'], $absolute_max_invite_count));

        $invitation_expiration_time = $config["invitation_ttl"]/$config["seconds_in_day"];

        $processing_results[] = sprintf( ___("%d invitation(s) queued"), $count );
        $processing_results[] = sprintf( ___("These invites expire in %d days"), $invitation_expiration_time );

        $invite_success = 1;
    }
    else
    {
        // None of the provided email addresses were valid.

        $processing_errors[] = ___("No valid e-mail address was provided");
        $curs = "invite.addr_list";
    }
    
    break;
}

$title = ___("Send invitations");

include($_SERVER[DOCUMENT_ROOT]."/header.php");

?>

<?/* Invite form */?>

<form name="invite" method="post" action="<?=getenv("SCRIPT_NAME")?>">
    <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden"/>

    <table cellpadding="2" cellspacing="0" border="0">
        <tr>
            <th align="right"><?=___("Invite tokens")?></th>
            <td><?=($record_user['invite_token_count'] == Null ? ___("Unlimited") : escape_html($record_user['invite_token_count']))?></td>
        </tr>

        <? if ($proxy_hidden) { ?>
            <input name="proxy" value="<?=$record_user['current_proxy']?>" type="hidden"/>
        <? } else { ?>
            <tr>
                <th align="right"><?=___("Proxy")?></th>
                <?/* Populate proxy combo box. Restore previous proxy selection. */?>
                <td><?echo_proxy_combo($config, "proxy", ($_POST['proxy'] == Null ? $record_user['current_proxy'] : $_POST['proxy']), 0)?></td>
            </tr>
        <? } ?>

        <? if ($user_class_hidden) { ?>
            <input name="user_class" value="<?=$record_user['grp']?>" type="hidden"/>
        <? } else if ($user_class_readonly) { ?>
            <tr>
                <th align="right"><?=___("User Class")?></th>
                <?/* Populate user class combo box. Restore previous proxy selection. */?>
                <td>
                    <?=reset(get_user_class_names($config, $record_user, true))?>
                    <input type="hidden" name="user_class" value="<?=key(get_user_class_names($config, $record_user, true))?>"/>
                </td>
                
            </tr>
        <? } else { ?>
            <tr>
                <th align="right"><?=___("User Class")?></th>
                <?/* Populate user class combo box. Restore previous proxy selection. */?>
                <td><?echo_user_class_combo($config, "user_class", $_POST['user_class'], 0, 1)?></td>
            </tr>
        <? } ?>

        <tr valign="top">
            <th align="right"><?=sprintf(___("E-mail Addresses<br>(One per line,<br>max %d)"), $max_invite_count)?></th>
            <td><textarea name="addr_list" cols="30" rows="20"><?=htmlentities($_POST['addr_list'], ENT_QUOTES, charset())?></textarea></td>
        </tr
        <tr valign="top">
            <th align="right"><?php echo ___("Default Homepage"); ?></th>
            <td><input type="text" name="default_homepage" value="<?php echo htmlentities($default_homepage_url, ENT_QUOTES, charset())?>" /></td>
        </tr>

        <tr>
            <td><input type="hidden" name="submit_invite" value="1"/></td>
            <td><input type="submit" <?=($record_user['invite_token_count'] == "0" ? "disabled=true" : "")?> value="<?=___("Invite")?>"/></td>
        </tr>

    </table>

</form>

<?
include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>
