<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// js-injection/edit.php
//
// This screen is used to edit js injection code into pages browsed in psiphon 
// such as polls, surveys, ads etc.
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

function db_transaction_and_redirect($mode, $array)
{
    global $config;
    $array["valid_from"] = mktime( 0, 0, 0,(int)$array["from_mm"], (int)$array["from_dd"], (int)$array["from_yyyy"]);
    $array["valid_to"] = mktime( 23, 59, 59,(int)$array["to_mm"], (int)$array["to_dd"], (int)$array["to_yyyy"]);
    
    $query_params = array(
            ":description" => $array["description"],
            ":valid_from" => $array["valid_from"],
            ":valid_to" => $array["valid_to"],
            ":notes" => $array["notes"],
            ":injection_code" => $array["injection_code"]
            );

    if($mode == "ADD")
    {
        //Insert injection 
        $query = "INSERT INTO js_injection (description, valid_from, valid_to, notes, injection_code) ".
            "VALUES (:description, :valid_from, :valid_to, :notes, :injection_code)";

        db_query_execute($config, $query, convert_null_array($query_params));

        $array["id"] = db_get_last_insert_id($config);
    }
    if($mode == "EDIT")
    {
        //Update query_params with id
        $query_params[":js_injection_id"] = $array["id"];

        //Update injection
        $query = "UPDATE js_injection SET description = :description, valid_from = :valid_from, ".
            "valid_to = :valid_to, notes = :notes, injection_code = :injection_code where id = :js_injection_id";

        db_query_execute($config, $query, convert_null_array($query_params));
        
        //Delete all current injection/proxy relation 
        $query = "DELETE FROM js_injection_proxy WHERE js_injection_id = :js_injection_id";

        db_query_execute($config, $query, convert_null_array(array(':js_injection_id'=>$array["id"])));

    }
    if($mode == "ADD" || $mode == "EDIT")
    {
        //Insert injection/proxy relation
        $query = "INSERT INTO js_injection_proxy (js_injection_id, proxy_id) ".
            "VALUES (:js_injection_id, :proxy_id)";

        foreach($array["proxies"] as $proxy_id)
        {
            $query_params = array( ":js_injection_id" => $array["id"], ":proxy_id" => $proxy_id);
            db_query_execute($config, $query, convert_null_array($query_params));
        }
        //redirect to "EDIT" mode
        header("Location: ".getenv("SCRIPT_NAME")."?js_injection_id=". $array["id"] ."&js_injection_saved=yes", true, 302);
        exit(0);
    }
}

function error_check($array, &$curs, &$errors)
{
    if(!empty($array["from_mm"]) && !empty($array["from_dd"]) && !empty($array["from_yyyy"])
            && strptime($array["from_dd"]."/".$array["from_mm"]."/".$array["from_yyyy"], "%d/%m/%Y"))
    {
        $from = mktime( 0, 0, 0,(int)$array["from_mm"], (int)$array["from_dd"], (int)$array["from_yyyy"]);
    }
    if(!empty($array["to_mm"]) && !empty($array["to_dd"]) && !empty($array["to_yyyy"])
            && strptime($array["to_dd"]."/".$array["to_mm"]."/".$array["to_yyyy"], "%d/%m/%Y"))
    {
        $to = mktime( 23, 59, 59,(int)$array["to_mm"], (int)$array["to_dd"], (int)$array["to_yyyy"]);
    }

    if (empty($array["description"]))
    {
        $errors[] = ___("Error: description is empty");
        $curs = "forms[0].elements['js_injection[description]']";
    }
    if (!$from)
    {
        $errors[] = ___("Error: malformed 'Valid from' date");
        if(!$curs)
        {
            $curs = "forms[0].elements['js_injection[from_mm]']";
        }
    }
    if (!$to)
    {
        $errors[] = ___("Error: malformed 'Valid to' date");
        if(!$curs)
        {
            $curs = "forms[0].elements['js_injection[to_mm]']";
        }
    }
    if($to && $to < time())
    {
        $errors[] = ___("Error: selected 'Valid to' date is in the past");
        if(!$curs)
        {
            $curs = "forms[0].elements['js_injection[to_mm]']";
        }
    }
    if($to && $from && $to < $from)
    {
        $errors[] = ___("Error: selected 'Valid to' date is less than 'Valid from' date");
        if(!$curs)
        {
            $curs = "forms[0].elements['js_injection[to_mm]']";
        }
    }
    if (empty($array["proxies"]))
    {
        $errors[] = ___("Error: at least one proxy must be selected");
        if(!$curs)
        {
            $curs = "forms[0].elements['js_injection[proxies][0]']";
        }
    }
    if (empty($array["injection_code"]))
    {
        $errors[] = ___("Error: injection code is missing");
        if(!$curs)
        {
            $curs = "forms[0].elements['js_injection[injection_code]']";
        }
    }
    return count($errors);
}
//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "can_inject_js"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

$js_injection = array();
$js_injection["proxies"] = array();
$page_mode = "ADD";
$title = ___("Add new JS injection");
if ($_GET["js_injection_saved"] == "yes")
{
    $processing_results[] = ___("Javascript injection saved");
}
if ($_GET["js_injection_id"])
{
    $js_injection["id"] = cast_str_to_int($_GET["js_injection_id"]);
    if ($js_injection["id"]<= 0)
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }
    $page_mode = "EDIT";
    $title = ___("Edit javascript injection");
}

// Get info for all proxies.
if (!user_has_permission($config, $record_user, "other_nodes"))
{
    // This code does not account for users that don't have full access to all proxies.
    echo "Insufficient permissions";
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

//Add-edit logic
if($_POST["submitted"] == "yes")
{
    $js_injection = array_merge((array)$js_injection, (array)$_POST["js_injection"]);

    foreach($js_injection as $key=>$val)
        $js_injection[$key] = trim($val);

    $js_injection["proxies"] = array_keys((array)$_POST["js_injection"]["proxies"]);

    if(error_check($js_injection, $curs, $processing_errors) == 0)
    {
        db_transaction_and_redirect($page_mode, $js_injection);
    }
}
else if ($page_mode == "EDIT")
{
    $title = ___("Edit javascript injection");
    $query = "SELECT * FROM js_injection WHERE id = :js_injection_id";
    $result = db_query_execute($config, $query, convert_null_array(array(":js_injection_id" => $js_injection["id"])));
    $record = db_fetch_result($config, $result);
    $js_injection["description"] = $record["description"];
    
    if(empty($record)) //no such user return 403
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    }
    
    $js_injection["notes"] = $record["notes"];
    $js_injection["injection_code"] = $record["injection_code"];
    $js_injection["from_dd"] = date("d", $record["valid_from"]);
    $js_injection["from_mm"] = date("m", $record["valid_from"]);
    $js_injection["from_yyyy"] = date("Y", $record["valid_from"]);
    $js_injection["to_dd"] = date("d", $record["valid_to"]);
    $js_injection["to_mm"] = date("m", $record["valid_to"]);
    $js_injection["to_yyyy"] = date("Y", $record["valid_to"]);
    
    $query = "SELECT proxy_id FROM js_injection_proxy WHERE js_injection_id = :js_injection_id";
    $result = db_query_execute($config, $query, convert_null_array(array(":js_injection_id" => $js_injection["id"])));
    while ($record = escape_html(db_fetch_result($config, $result)))
    {
        array_push($js_injection["proxies"], $record["proxy_id"]);
    }

}

//build proxies checkboxes
$query = "SELECT id, ".proxy_label_sql("proxy")." AS label FROM proxy ORDER BY label";
$result = db_query_execute($config, $query, convert_null_array(array()));
$all_proxies = array();
while ($record = escape_html(db_fetch_result($config, $result)))
{
    array_push($all_proxies, $record);
}   
$proxy_checkboxes = "";
foreach ($all_proxies as $proxy)
{
    $proxy_name = 'js_injection[proxies]['.$proxy["id"].']';
    $checked = in_array($proxy["id"], (array)$js_injection["proxies"]);
    $proxy_checkboxes .= '<input type="checkbox" name="'.$proxy_name.'" id="'.$proxy_name.'" '.($checked?'checked':'').' class="element checkbox" value="'.$proxy["id"].'"/>';
    $proxy_checkboxes .= '<label for="'.$proxy_name.'" class="choice">'.$proxy["label"].'</label>';
}
// Screen title. Must come before the header.php include.

$header_link_prefix = "/";
$form_action = $js_injection["id"] ? getenv("SCRIPT_NAME")."?js_injection_id=".$js_injection["id"] : getenv("SCRIPT_NAME");
include($_SERVER[DOCUMENT_ROOT]."/header.php");

?>
<link rel="stylesheet" type="text/css" href="/js-injection/view.css" media="all">
<script type="text/javascript" src="/js-injection/view.js"></script>
<script type="text/javascript" src="/js-injection/calendar.js"></script>
	<div id="form_container">
	
		<form  class="appnitro"  method="post" name="frm" action = "<?=$form_action?>">
                <input name="csrf_token" value="<?=$record_user["csrf_token"]?>" type="hidden"/>
                <input name="submitted" value="yes" type="hidden"/>
			<ul >
			
					<li id="li_1" >
		<label class="description" for="element_1"><?=___("Description")?></label>
		<div>
			<input id="element_1" name="js_injection[description]" class="element text medium" type="text" maxlength="255" value="<?echo escape_html($js_injection[description]);?>"/> 
		</div>
		</li>		<li id="li_2" >
		<label class="description" for="element_2"><?=___("Notes")?></label>
		<div>
			<input id="element_2" name="js_injection[notes]" class="element text large" type="text" maxlength="255" value="<?echo escape_html($js_injection[notes]);?>"/> 
		</div>
		</li>		<li id="li_3" >
		<label class="description" for="element_3"><?=___("Valid from")?></label>
		<span>
			<input id="element_3_1" name="js_injection[from_mm]" class="element text" size="2" maxlength="2" value="<?echo escape_html($js_injection[from_mm]);?>" type="text"> /
			<label for="element_3_1"><?=___("MM")?></label>
		</span>
		<span>
			<input id="element_3_2" name="js_injection[from_dd]" class="element text" size="2" maxlength="2" value="<?echo escape_html($js_injection[from_dd]);?>" type="text"> /
			<label for="element_3_2"><?=___("DD")?></label>
		</span>
		<span>
	 		<input id="element_3_3" name="js_injection[from_yyyy]" class="element text" size="4" maxlength="4" value="<?echo escape_html($js_injection[from_yyyy]);?>" type="text">
			<label for="element_3_3"><?=___("YYYY")?></label>
		</span>
	
		<span id="calendar_3">
			<img id="cal_img_3" class="datepicker" src="calendar.gif" alt="Pick a date.">	
		</span>
		<script type="text/javascript">
			Calendar.setup({
			inputField	 : "element_3_3",
			baseField    : "element_3",
			displayArea  : "calendar_3",
			button		 : "cal_img_3",
			ifFormat	 : "%B %e, %Y",
			onSelect	 : selectDate
			});
		</script>
		</li>		<li id="li_4" >
		<label class="description" for="element_4"><?=___("Valid to")?></label>
		<span>
			<input id="element_4_1" name="js_injection[to_mm]" class="element text" size="2" maxlength="2" value="<?echo escape_html($js_injection[to_mm]);?>" type="text"> /
			<label for="element_4_1"><?=___("MM")?></label>
		</span>
		<span>
			<input id="element_4_2" name="js_injection[to_dd]" class="element text" size="2" maxlength="2" value="<?echo escape_html($js_injection[to_dd]);?>" type="text"> /
			<label for="element_4_2"><?=___("DD")?></label>
		</span>
		<span>
	 		<input id="element_4_3" name="js_injection[to_yyyy]" class="element text" size="4" maxlength="4" value="<?echo escape_html($js_injection[to_yyyy]);?>" type="text">
			<label for="element_4_3"><?=___("YYYY")?></label>
		</span>
	
		<span id="calendar_4">
			<img id="cal_img_4" class="datepicker" src="calendar.gif" alt="Pick a date.">	
		</span>
		<script type="text/javascript">
			Calendar.setup({
			inputField	 : "element_4_3",
			baseField    : "element_4",
			displayArea  : "calendar_4",
			button		 : "cal_img_4",
			ifFormat	 : "%B %e, %Y",
			onSelect	 : selectDate
			});
		</script>
		</li>
                <li>
		<label class="description" for="element_4"><?=___("Proxies")?></label>
                <span>
                <?=$proxy_checkboxes?>
                </span>
                </li>
                <li id="li_2" >
                <label class="description"><?=___("Javascript code (do not include &lt;script&gt; tags)")?></label>
                <div>
                <pre>&lt;script type="text/javascript"&gt;</pre>
                <textarea id="element_2" name="js_injection[injection_code]" class="element textarea large"><?echo htmlentities($js_injection["injection_code"], ENT_QUOTES , charset());?> </textarea> 
                <pre>&lt;/script&gt;</pre>
                </div>
                </li>
			
                <li class="buttons">
				<input id="saveForm" class="button_text" type="submit" name="js_injection[submit]" value="<?=___("Save")?>" />
		</li>
                <li id="li_2" >
                <a href="/js-injection/list.php"><?=___("Back to the list")?></a>
                </li>
                </ul>
		</form>	
	</div>

<?
include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>
