<?php
/*
    Psiphon Circumvention Platform
   Copyright (C) 2009-2010 Psiphon Inc..

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/cr.php");



$token = $_GET['token'];
$passwords_submitted = isset($_POST['submit_button']);


// check token
if (strlen($token) != 32) {
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}


/// Fetch the user to reset password
$record_user = null;

if ( $token )
{
    $query =
        "SELECT * FROM user ".
        "WHERE pass_reset_token = :token AND UNIX_TIMESTAMP() < pass_reset_date + ".$config['password_reset_ttl'];

    $result = db_query_execute($config, $query,
        array(":token" => $token)
    );

    $record_user = db_fetch_result($config, $result);


    if ( ! $record_user )
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
    }
}

$query = "SELECT * FROM proxy WHERE hostname = :hostname";
$result = db_query_execute($config, $query, convert_null_array(array(":hostname" => $_SERVER[HTTP_HOST])));
$proxy_record = db_fetch_result($config, $result);

if ( ! can_current_user_access_proxy($config, $record_user, $proxy_record['id']))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}



$password_error = null;

if ($passwords_submitted)
{
    $password1 = $_POST['password1'];
    $password2 = $_POST['password2'];


    // password validation

    if (!is_valid_password($config, $password1, $record_user['uname']))
        $password_error = sprintf(___("Error: Password should be at least %d characters long and not contain your username"), $config[min_pass_length]);

    if ($password1 != $password2)
        $password_error = ___("Error: Password mismatch");

    // Update user password
    if ( ! $password_error )
    {
        $pass_salt = bin2hex(secure_rand());
        $password = sha1($password1.$pass_salt);

        // unguessable session ID
        $session_id = bin2hex(secure_rand());


        // reset password, password salt and invalidate password reset token.
        $query =
            'UPDATE `user` '.
            'SET '.
                'pass       = :password, '.
                'pass_salt  = :pass_salt, '.
                'session    = :session, '.
                'pass_reset_token = "" '.
            'WHERE id = :id';

        db_query_execute(
                $config,
                $query,
                array(
                    ':password' => $password,
                    ':pass_salt' => $pass_salt,
                    ':session' => $session_id,
                    ':id' => $record_user['id']
                )
        );
    }
}

$password_changed = $passwords_submitted && ! $password_error;
if ($password_changed)
{
    $login_url = $proxy_record['login_url'];
    header('Location: '.$config['ret'].$login_url, true, 302);
    exit;
}


// set page title
$page_title = ___("Reset password");

header("Content-type: text/html; charset=".charset());
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
    <head>
        <title><?= $page_title; ?></title>
        <?=echo_css()?>
    </head>

    <body style="direction: <?=direction()?>">

            <table>
                <tr>
                    <td style="width:50%">
<?php
    include($_SERVER[DOCUMENT_ROOT]."/images/psiphonlogo.html");
?>
                    </td>
                    <td>
                        <?php if ( ! $password_changed ) : ?>
                            <font class="sixteenbluebold"><?=___("Reset password")?></font>
                        <?php else: ?>
                            <font class="sixteenbluebold"><?=___("Password changed")?></font>
                        <?php endif; ?>
                    </td>
                </tr>
            </table>
            <br>


            <?php if ($password_error) : ?>
                <font class="processing_error"><?= $password_error; ?></font>
                <br /><br />
            <?php endif;?>

            <form name="reset_password" action="<?=escape_html(getenv("REQUEST_URI"))?>" method="post">
            <table>
                <tr>
                    <td>
                        <label for="password1"><?= ___("Password"); ?></label></td>
                    <td>
                        <input type="password" name="password1" id="password1" />
                    </td>
                </tr>
                <tr>
                    <td>
                        <label for="password2"><?= ___("Verify password"); ?></label>
                    </td>
                    <td>
                        <input type="password" name="password2" id="password2" /><br />
                    </td>
                </tr>
                <tr>
                    <td colspan="2">
                        <input type="submit" name="submit_button" value="<?= ___("Save"); ?>" />
                    </td>
                </tr>
            </table>
            </form>

            <br>
            <br>
    </body>
</html>