<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "profile"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

$title = ___("Profile");
$curs="profile.npass";

while ($_POST[update])
{

    $q="";

    $_POST[uname] = trim($_POST[uname]);
    if (!strlen($_POST[uname]) || !is_user($_POST[uname]))
    {
        $processing_errors[] = ___("Error: Invalid username format");
        $curs = "profile.uname";
        break;
    }

    if (strlen($_POST[npass]))
    {
        // TODO: safe to trust POST[uname] here?
        if (!is_valid_password($config, $_POST[npass], $_POST[uname]))
        {
            $processing_errors[] = sprintf(___("Error: Password should be at least %d characters long and not contain your username"), $config[min_pass_length]);
            $curs = "profile.password";
            break;
        }

        if ($_POST[npass] != $_POST[vpass])
        {
            $processing_errors[] = ___("Error: Password mismatch");
            $curs = "profile.password";
            break;
        }
    }

    // validate email only if if present: must be valid email or blank
    $_POST["email"]=trim($_POST["email"]);

    if (strlen($_POST["email"]) && !is_email($_POST["email"]))
    {
        $processing_errors[] = ___("Wrong e-mail syntax");
        $curs="profile.email";
        break;
    }

    // validate home page: must be valid domain (or blank),

    if (strlen($_POST["home_page"]) > 0)
    {
        if (!is_url($_POST["home_page"]))
        {
            // try auto inserting the "http://" prefix as many users may omit it

            $_POST["home_page"] = "http://".$_POST["home_page"];

            if (!is_url($_POST["home_page"]))
            {
                $processing_errors[] = ___("Error: Invalid home page format");
                $curs="profile.home_page";
                break;
            }
        }
    }

    foreach ($_POST as $key)
    {
        $_POST[$key] = trim($_POST[$key]);
    }

    // NOTE: Because we don't have transactions, there is a race condition between
    // the conflict check and the insert/update attempt. If a conflict is
    // introduced between the check and the insert/update, a DB error will occur.
    // This is unfortunate but acceptable (at this time).

    if (db_check_if_value_exists(
        $config,
        "user",
        "uname",
        $_POST['uname'],
        "id != :id",
        array(":id" => $record_user['id'])))
    {
        $processing_errors[] = ___("Username already exists in the system");
        $curs = "profile.uname";
        break;
    }

    $query_params = array();

    $query="UPDATE user SET ";

    $query .= "uname = :uname, ";
    $query_params[":uname"] = $_POST["uname"];

    // set email logic:
    // - blank is immediately set to NULL, including candidate set to NULL
    // - skip if no edit (email = old_email), leave candidate as-is
    // - otherwise, set candidate and email the token
    //   the candidate email has already been validated as well-formed above
    //   in this case, the existing email is left as-is

    // unguessable token

    $email_candidate_token = bin2hex(secure_rand());
    $require_enqueue_email_verification = false;

    if (!strlen($_POST[email]))
    {
        $query .= "email = NULL, ";
    }
    elseif ($_POST[email] != $_POST[old_email])
    {
        if (user_has_permission($config, $record_user, "accounts"))
        {
            // users with "accounts" management permission can edit
            // their own email with immediate effect, just like they can
            // in users.php

            $query .= "email = :email, ";
            $query_params[":email"] = $_POST["email"];

            // NOTE: Because we don't have transactions, there is a race condition between
            // the conflict check and the insert/update attempt. If a conflict is
            // introduced between the check and the insert/update, a DB error will occur.
            // This is unfortunate but acceptable (at this time).

            if (db_check_if_value_exists(
                $config,
                "user",
                "email",
                $_POST['email'],
                "id != :id",
                array(":id" => $record_user['id'])))
            {
                $processing_errors[] = ___("E-mail address already exists in the system");
                $curs = "profile.email";
                break;
            }
        }
        else
        {
            // check if enough time has passed since user last caused an
            // email to be sent out
            if ($record_user[email_candidate_date] &&
                    $record_user[email_candidate_date] + $config["email_change_period"] > time())
            {
                $processing_errors[] = ___("Error: You may only change your e-mail address once per hour.  Please try again later.");
                $curs="profile.email";
                break;
            }

            $query .= "email_candidate = :email_candidate, ";
            $query_params[":email_candidate"] = $_POST["email"];

            $query .= "email_candidate_token = :email_candidate_token, ";
            $query_params[":email_candidate_token"] = $email_candidate_token;

            $query .= "email_candidate_date = UNIX_TIMESTAMP(), ";

            // no transactions: don't enqueue the email yet, wait until
            // user record change is saved

            $require_enqueue_email_verification = true;
        }
    }

    // only update home_page if not locked

    if (!$record_user[home_page_locked])
    {
        $query .= "home_page = :home_page, ";
        $query_params[":home_page"] = $_POST["home_page"];
    }

    if ($_POST[npass])
    {
        $pass_salt = bin2hex(secure_rand());
        $query .= "pass = :pass, pass_salt = :pass_salt, ";
        $query_params[":pass"] = sha1($_POST[npass].$pass_salt);
        $query_params[":pass_salt"] = $pass_salt;
    }

    $query .= "xml_html = :xml_html ";
    $query_params[":xml_html"] = $_POST["xml_html"];

    $query .= "WHERE id = :id";
    $query_params[":id"] = $record_user['id'];

    db_query_execute($config, $query, convert_null_array($query_params));

    $processing_results[] = ___("Your profile has been updated");
    $curs="";

    // TODO: why is check_user included here?
    // to reload $record_user? if so, a risky dependency?
    include($_SERVER[DOCUMENT_ROOT]."/check_user.php");

    // send email verification now that user record is updated

    if ($require_enqueue_email_verification)
    {
        enqueue_email_candidate_email(
            $config,
            $record_user[uname],
            $record_user[current_proxy],
            $record_user[lang],
            $record_user[email_candidate],
            $email_candidate_token);

        // tell the user to check inbox for verification email

        $processing_results[] = ___("You have provided a new e-mail address which has not yet been verified.  Psiphon has sent a verification e-mail to this address.  To verify your address, click the link provided when you receive the e-mail.");
    }

    break;
}

// check whether user can delete his own account
$user_can_delete = user_has_permission($config, $record_user, 'delete_account');


include($_SERVER[DOCUMENT_ROOT]."/header.php");


?>
<br>
<div style="width:50em;"><?=sprintf(___("IMPORTANT PRIVACY INFORMATION:<br>
                <ul>
                    <li><b>Username and E-mail address:&nbsp;</b>Your username and e-mail address are visible only to Psiphon administrators. By providing an e-mail address, Psiphon can inform you of new ways to connect if necessary. We recommend you create a GMail (<a href=\"https://mail.google.com\">https://mail.google.com/</a>) e-mail account for this purpose, since it is web-based and simple to use. It is highly recommended that you provide an e-mail address, but it is not required in order to create a Psiphon account.</li>
                    <li style='margin-top:5px'><b>Password:&nbsp;</b>Your password is not visible to any Psiphon administrator or user. Use this password to log in to your Psiphon account. It is recommended that you follow best practices for online account password creation and that you do not use the same password that you use for any other online accounts.</li>
                </ul>
                For more information, please visit our <a href=\"%s\">Resource Centre</a>."), make_proxied_url($config["psiphon_web_resource_page"]))?></div>
<br>

<form name="profile" method="post" action="<?=getenv("SCRIPT_NAME")?>">
<input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden">
<input name="update" value="1" type="hidden">

<table cellpadding="4" cellspacing="0" border="0">

<tr>
<th align="right"><?=___("Username")?></th>
<td><input type="text" name="uname" value="<?=escape_html($record_user[uname])?>" size="35" maxlength="32"></td>
</tr>

<tr>
<th align="right"><?=___("E-mail")?></th>
<td>
    <input type="hidden" name="old_email" value="<?=escape_html($record_user[email])?>">
    <input type="text" name="email" value="<?=escape_html($record_user[email])?>" size="35" maxlength="64">
</td>
</tr>

<?if ($record_user[email_candidate]){?>
<tr>
<th align="right"><?=___("Pending E-mail Address")?></th>
<td>
    <?=escape_html($record_user[email_candidate])?>
</td>
</tr>
<?}?>

<tr>
<th align="right"><?=___("New password")?></th>
<td nowrap><input type="password" name="npass" size="15" title="<?=___("Leave empty to keep your current password")?>"></td>
</tr>

<tr>
<th align="right"><?=___("Verify password")?></th>
<td><input type="password" name="vpass" size="15" title="<?=___("Leave empty to keep your current password")?>"></td>
</tr>

<!-- home page is editable if not locked -->

<tr>
<th align="right"><?=___("Home page")?></th>
<?if (!$record_user[home_page_locked]){?>
<td><input type="text" name="home_page" value="<?=escape_html($record_user[home_page])?>" size="35" maxlength="255"></td>
<?}else{?>
<td><?=htmlentities($record_user[home_page], ENT_QUOTES, charset())?></td>
<?}?>
</tr>

<tr>
<th align="right"><?=___("Feeds as HTML")?></th>
<td nowrap><input type="checkbox" name="xml_html" value="1"<?=$record_user["xml_html"]?" checked":""?>></td>
</tr>
<tr>
<td>&nbsp;</td>
<td><input type="submit" value="<?=___("Update")?>">

</table>

</form>
<?php if ($user_can_delete) : ?>
<br />
        
&nbsp;&nbsp;<a id="delete_href" href="/delete_account.php"><?php echo ___("Delete account");?> [X]</a>

<?php endif; ?>
<? include($_SERVER[DOCUMENT_ROOT]."/footer.php"); ?>
