<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009-2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// propagator-info.php
//
// This screen is used by Propagators to view the status of their Power Users.
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "view_propagator_info"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}
    
// Screen title. Must come before the header.php include.

$title = ___("Propagation");

// $user_id will be non-null if the user wants to edit a proxy.
$user_id = cast_str_to_int($_GET['user_id']);

while ($_POST['submitted'])
{
    $_POST['proxy'] = cast_str_to_int($_POST['proxy']);
    $_POST['user_id'] = cast_str_to_int($_POST['user_id']);

    // Check that the Propagator has access to the target Power User.
    $query = 
        "SELECT COUNT(*) AS proxy_overlap FROM userproxy ".
        "WHERE userid = :poweruserid ".
            "AND proxyid IN (SELECT proxyid FROM userproxy WHERE userid = :propuserid) ";
    $query_params = array(':poweruserid'=>$_POST['user_id'], ':propuserid'=>$record_user['id']);
    $result = db_query_execute($config, $query, convert_null_array($query_params));
    $record = db_fetch_result($config, $result);

    if (!$record['proxy_overlap'])
    {
        // This shouldn't have happened, so don't bother with a clean error message.
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }

    // Check that the Propagator has access to the desired proxy.
    $query =
        "SELECT COUNT(*) AS proxy_found FROM userproxy ".
        "WHERE userid = :userid AND proxyid = :proxyid ";
    $query_params = array(':userid'=>$record_user['id'], ':proxyid'=>$_POST['proxy']);
    $result = db_query_execute($config, $query, convert_null_array($query_params));
    $record = db_fetch_result($config, $result);

    if (!$record['proxy_found'])
    {
        // This shouldn't have happened, so don't bother with a clean error message.
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }

    // Add the new relationship to userproxy and set the proxy as the
    // target user's current_proxy.

    // Note that we're using INSERT IGNORE because the userproxy relationship
    // may already exist.
    $query = "INSERT IGNORE INTO userproxy (userid, proxyid) VALUES (:userid, :proxyid) ";
    $query_params = array(':userid'=>$_POST['user_id'], ':proxyid'=>$_POST['proxy']);
    db_query_execute($config, $query, convert_null_array($query_params));

    $query = "UPDATE user SET current_proxy = :proxyid WHERE id = :userid ";
    db_query_execute($config, $query, convert_null_array($query_params));

    // Send email to user informing them of the change.

    // Retrieve user email address, etc.
    $query = "SELECT * FROM user WHERE id = :userid";
    $query_params = array(':userid'=>$_POST['user_id']);
    $result = db_query_execute($config, $query, convert_null_array($query_params));
    $record = db_fetch_result($config, $result);

    // Check that the user has a valid email address. If not valid, just skip the email.
    if (is_email($record['email']))
    {
        // If the target user doesn't have a last_lang, use the current user's.
        if (!$record['last_lang'])
        {
            $record['last_lang'] = $record_user['last_lang'];
        }

        // generate the email subject and body

        $proxy_url = make_home_url($config, $_POST['proxy']);
        $subject = ___("Your new Psiphon proxy", NULL, $record['last_lang']);
        $body = sprintf(___("Here is your new Psiphon proxy URL:
%s

Please begin using this URL to log into Psiphon.", NULL, $record['last_lang']), $proxy_url);

        // enqueue email

        enqueue_email($config, $record['email'], $subject, $body, $record['last_lang']);
    }

    $processing_results[] = ___("Proxy change successful. Аn email will be sent to the Power User informing them of the change if he/she have provided an email address.");

    break;
}

// Get the proxies that the current user can manage.

$proxy_labels = array();
function proxy_to_label(&$value, $key, $proxy_labels) { $value = $proxy_labels[$value]; }

$query_proxy =
    "SELECT id, ".proxy_label_sql('proxy')." AS proxy_label ".
    "FROM proxy ".
    "WHERE id IN (SELECT proxyid FROM userproxy WHERE userid = :userid) ";
$result = db_query_execute($config, $query_proxy, convert_null_array(array(':userid'=>$record_user['id'])));

while ($record = db_fetch_result($config, $result))
{
    $proxy_labels[$record['id']] = $record['proxy_label'];
}

include($_SERVER[DOCUMENT_ROOT]."/header.php");
?>

<!-- Filled in by javascript after the user table -->
<div id="summary"></div>

<table class="brd">
    <tr>
        <th><?=___("Power User Name")?></th>
        <th><?=___("Invites accepted")?></th>
        <th><?=___("Invites sent")?></th>
        <th><?=___("Proxies")?></th>
        <th><?=___("Created")?></th>
    </tr>

    <?
    // We display all Power Users that have some proxy-overlap with the Propagator.
    // The Proxies column displays the overlapping proxies. The Prop can add
    // accessible proxies to the Power User.

    $proxy_separator = '##';
    $query = "SELECT user.*, FROM_UNIXTIME(created, '%Y-%m-%d') AS created_date, ".
                    "GROUP_CONCAT(userproxy.proxyid SEPARATOR '".$proxy_separator."') AS proxies ".
             "FROM user LEFT OUTER JOIN userproxy ON user.id = userproxy.userid ".
             "WHERE user.grp = ".get_power_user_class_id()." ".
                 "AND userproxy.proxyid IN (SELECT proxyid FROM userproxy WHERE userid = :current_user_id) ".
             "GROUP BY uname ORDER BY uname ";

    $query_params = array(':current_user_id' => $record_user['id']);
    $result = db_query_execute($config, $query, convert_null_array($query_params));

    $total_power_users = 0;
    $total_invites_sent = 0;
    $total_invites_accepted = 0;
    while ($record = escape_html(db_fetch_result($config, $result)))
    {
        // Split the proxies into an array. Then change them to labels.
        $record['proxies'] = explode($proxy_separator, $record['proxies']);
        array_walk($record['proxies'], 'proxy_to_label', $proxy_labels);
        
        $total_power_users += 1;
        $total_invites_sent += (int)$record['inv_sent'];
        $total_invites_accepted += (int)$record['inv_actv'];

        echo "<tr valign=\"top\">\n";
        echo "<td>{$record["uname"]}</td>\n";
        echo "<td>{$record['inv_actv']}</td>\n";
        echo "<td>{$record['inv_sent']}</td>\n";

        // Display only the Power User's current proxy.

        if ($record['id'] == $user_id)
        {
            // This Power User's proxy is being edited.
            echo "<td>";
            echo '<form name="user_edit" method="post" action="'.getenv("SCRIPT_NAME").'">';
            echo '<input name="csrf_token" value="'.$record_user['csrf_token'].'" type="hidden">';
            echo '<input type="hidden" name="submitted" value="1"/>';
            echo '<input type="hidden" name="user_id" value="'.$user_id.'"/>';
            echo_proxy_combo($config, "proxy", Null, FALSE);
            echo '<input type="submit" value="'.___("Submit").'"/>';
            echo '</form>';
            echo '&nbsp;(<a href="'.getenv("SCRIPT_NAME").'">'.___("Cancel").'</a>)';
            echo "</td>\n";
        }
        else
        {
            if (count($record['proxies']) <= 0)
            {
                echo "<td>";
            }
            else if (count($record['proxies']) == 1)
            {
                echo "<td>{$record['proxies'][0]}";
            }
            else
            {
                $size = min(count($record['proxies']), 3);
                echo "<td><select size=".$size."><option>".
                     implode($record['proxies'], "</option><option>").
                     "</option></select>";
            }

            // Display the edit-proxy link
            echo '&nbsp;&nbsp;(<a href="'.getenv("SCRIPT_NAME").'?user_id='.$record['id'].'">'.___("Add").'</a>)';
            echo "</td>\n";
        }

        echo "<td>{$record['created_date']}</td>\n";

        echo "</tr>\n\n";
    }
    ?>
</table>

<?/* This summary section will be hidden (moved to the top) if JavaScript is enabled */?>
<div id="bottom_summary">
    <p>
        <b><?=___("Total Power Users")?>: </b><?=$total_power_users?><br/>
        <b><?=___("Total invites accepted")?>: </b><?=$total_invites_accepted?><br/>
        <b><?=___("Total invites sent")?>: </b><?=$total_invites_sent?><br/>
    </p>
</div>

<script type="text/javascript">
    // Show the summary at the top of the page.
    // Take the contents of the bottom summary section (which is the javascript-disabled
    // failback) and copy them into the top summary section. Then hide the bottom summary.

    var summary_section = document.getElementById('summary');
    while (summary_section.hasChildNodes())
    {
        summary_section.removeChild(summary_section.firstChild);
    }

    var newDiv = document.createElement("div");
    newDiv.innerHTML = document.getElementById('bottom_summary').innerHTML;

    summary_section.appendChild(newDiv);

    // Hide the summary at the bottom

    document.getElementById('bottom_summary').setAttribute('style', 'display:none');
</script>

<?
include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>
