<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009-2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/database_helpers.php");
include_once($_SERVER[DOCUMENT_ROOT]."/includes/cr.php");

// === Reset password form ===

// Check the "lu" parameter matches the encoded login_url for this host; this is
// part of the randomized URL defenses that prevents a scanner from finding anything
// other than 404 unless it provides some known random value.
// This code is adapted from chk.php

$query = "SELECT * FROM proxy WHERE hostname = :hostname";
$result = db_query_execute($config, $query, convert_null_array(array(":hostname" => $_SERVER[HTTP_HOST])));
$proxy_record = db_fetch_result($config, $result);

if (!$proxy_record || $proxy_record['login_url'] != $_GET[lu])
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
}

$login_url = $proxy_record['login_url'];

// Set record_user[last_lang] early as it's referenced in ___()

$reset_lang = strlen($_POST["reset_lang"]) ? $_POST["reset_lang"] : $_GET["reset_lang"];

if (strlen($reset_lang) && array_key_exists($reset_lang, $config["lang"]))
{
    $record_user["last_lang"] = $reset_lang;
}

// this flag used to render form differently (give link back to login and hide reset controls)
$password_sent = false;

while (1)
{
    // === Password reset ===

    if (!strlen($_POST[reset_id]))
    {
        // Note: this message will be posted when the form first
        // displays, which is intended

        $reset_msg = ___("To reset your password, enter your username or e-mail address, and click the reset button.  A password reset link will be e-mailed to you.");
        $login_curs = "reset_id";
        break;
    }

    // works if either username or email address matches
    // existing user, and user has an email address to mail to

    $user_record = lookup_user(
                        $config,
                        "(uname = :uname or email = :email) ".
                        "and email is not null",
                        array(":uname" => $_POST[reset_id], ":email" => $_POST[reset_id]));

    if ($user_record)
    {
        // Check that the user has permission to access this proxy.
        if (!user_has_permission($config, $user_record, "other_nodes")
            && FALSE === array_search($proxy_record['id'], $user_record['assigned_proxies']))
        {
            // This may be a probing attack. Tell the user the rest attempt suceeded.
            $reset_msg = ___("A password reset link was sent to your e-mail address.  If you do not receive an e-mail, double check your username or e-mail address and try again.");
            $password_sent = true;
            break;
        }

        $pass_reset_token = bin2hex(secure_rand());

        // update user record with new, temporary password
        // user.session is set to a large random number to invalidate the session.
        // TODO: Set session to null instead.
        $query = "UPDATE user ".
                 "SET pass_reset_token = :token, ".
                 "pass_reset_date = UNIX_TIMESTAMP() ".
                 "WHERE id = :user_id";

        db_query_execute(
            $config,
            $query,
            convert_null_array(array(
                ":token" => $pass_reset_token,
                ":user_id" => $user_record['id']
            ))
        );

        $passwordResetLink = make_password_reset_url($config, $proxy_id = $proxy_record['id'], $pass_reset_token).
                ' ' . ___("Psiphon password reset", NULL, $user_record['last_lang']);

        // send the email
        // TODO: should be some throttling here, as anyone can send an
        // unlimited number of emails to any user.
        enqueue_email(
                    $config,
                    $user_record['email'],
                    ___("Psiphon password reset", NULL, $user_record['last_lang']),
                    sprintf(___("Follow this link to reset your password: %s.", NULL, $user_record['last_lang']), $passwordResetLink),
                    $user_record['last_lang']);

        // The feedback here leaked information and allowed anyone to probe
        // for valid/invalid email addresses.  This isn't acceptable, so
        // the feedback is now "crippled" so it looks the same in the success
        // and failure cases.  This isn't ideal, but there's a plan to rewrite
        // all of this anyway (see Trac ticket #66)
        // this was the old success prompt, and we may want to reintroduce
        // it again in the future (e.g., if user enters username to reset password,
        // as in that case distinct positive/negative feedback won't leak information
        // about valid email addresses).  but anyways because of information
        // leakage, the positive case (here) and negative case below are
        // now identical
        //$reset_msg = sprintf(___("A password reset link was sent to your e-mail address.  If you do not receive an e-mail, double check your username or e-mail address and try again."), $_POST[reset_id]);
        $reset_msg = ___("A password reset link was sent to your e-mail address.  If you do not receive an e-mail, double check your username or e-mail address and try again.");
        $password_sent = true;
    }
    else
    {
        // this was a probing attack.  leaving the code here as an example
        // of what *not* to do.
        //$reset_msg=___("User not found");
        $reset_msg = ___("A password reset link was sent to your e-mail address.  If you do not receive an e-mail, double check your username or e-mail address and try again.");
        $password_sent = true;
    }

    break;
}

// === Display reset screen ===

header("Content-type: text/html; charset=".charset());
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
    <head>
        <title><?=___("Reset password")?></title>
        <?=echo_css()?>
    </head>

    <body style="direction: <?=direction()?>">

            <table><tr>
                <td style="width:50%">
<?php
    include($_SERVER[DOCUMENT_ROOT]."/images/psiphonlogo.html");
?>
                </td>
                <td><font class="sixteenbluebold"><?=___("Reset password")?></font></td>
            </tr></table>
            <br>

<?php
    if($password_sent)
    {
?>
        <? if ($reset_msg) echo "<font class=\"processing_result\">{$reset_msg}</font><br><br>\n\n"; ?>

        <table>
            <tr>
                <td colspan="2" align="center"><label><font class="fourteenblue"><a href="<?=$login_url?>"><?=___("Login")?></a></font></label></td>
            </tr>
        </table>
<?php
    }
    else
    {
?>
            <? if ($reset_msg) echo "<font class=\"processing_error\">{$reset_msg}</font><br><br>\n\n"; ?>

            <form name="reset_password" method="post" action="<?=escape_html(getenv("REQUEST_URI"))?>">
            <input type="hidden" name="reset_lang" value="<?=$record_user["last_lang"]?>"/>
            <table border="0" cellpadding="2" cellspacing="0">
                <tr>
                    <td><label for="email"><font class="fourteenblue"><?=___("Username (or e-mail)")?></font></label></td>
                    <td><input type="text" name="reset_id" value="<?=escape_html($_POST[reset_id])?>" size="30" maxlength="64" id="email"></td>
                </tr>

                <tr>
                    <td colspan="2" align="right"><input type="submit" value="<?=___("Reset")?>"></td>
                </tr>
            </table>
            </form>

            <br>
            <br>

            <?echo_language_buttons($config, $record_user, "reset_lang", 5);?>

        <script language="javascript" type="text/javascript">
        <!--
        document.reset_password.reset_id.focus();
        //-->
        </script>
<?php
    }
?>
    </body>
</html>

