<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2010 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// user-edit.php
//
// This screen is used to add or edit users.
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "accounts"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

// Chaining paging, filtering, and sorting info back to user list page
$_GET['filterby'] = cast_str_to_int($_GET['filterby']);
$_GET['ofs'] = cast_str_to_int($_GET['ofs']);
$_GET['ord'] = cast_str_to_int($_GET['ord']);
if (isset($_GET['filterby'])) {
    $delimiter = $paging_sorting_filtering_params ? "&" : "?"; 
    $paging_sorting_filtering_params .= $delimiter . "filterby=" . $_GET['filterby'];
}
if (isset($_GET["ord"])) {
    $delimiter = $paging_sorting_filtering_params ? "&" : "?"; 
    $paging_sorting_filtering_params .= $delimiter . "ord=". $_GET["ord"];
}
if (isset($_GET["ofs"])) {
    $delimiter = $paging_sorting_filtering_params ? "&" : "?"; 
    $paging_sorting_filtering_params .= $delimiter . "ofs=". $_GET["ofs"];
}


// This page has two modes: add and edit. If there are no GET params, then it's
// in add-mode. If there's an 'user_id' GET param, this it's in edit-mode. (The
// 'user_id' is the ID of the user to edit.)
$curs = "user_edit.username";
$page_mode = "ADD";
$user_id = Null;
$title = ___("Add a new user");
if (cast_str_to_int($_GET['user_id']) !== Null)
{
    $user_id = cast_str_to_int($_GET['user_id']);
    if ($user_id <= 0)
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }

    $page_mode = "EDIT";
    $title = ___("Edit an existing user");
}

// Get info for all proxies.
if (!user_has_permission($config, $record_user, "other_nodes"))
{
    // This code does not account for users that don't have full access to all proxies.
    echo "Insufficient permissions";
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}
$query = "SELECT id, ".proxy_label_sql("proxy")." AS label FROM proxy ORDER BY label";
$result = db_query_execute($config, $query, convert_null_array(array()));
$all_proxies = array();
while ($record = escape_html(db_fetch_result($config, $result)))
{
    array_push($all_proxies, $record);
}

// Lookup the user record, if in edit mode
// If we're just starting out in edit mode, then we need the info to fill in the form.
// If the form is being submitted, then we need to check if the email is changing.
if ($page_mode == "EDIT")
{
    $query = "SELECT uname, email, grp, current_proxy FROM user WHERE id = :user_id";
    $result = db_query_execute($config, $query, convert_null_array(array(":user_id" => $user_id)));
    $target_record = escape_html(db_fetch_result($config, $result));

    $original_email = $target_record['email'];

    if (!$_POST['submitted'])
    {
        $_POST['user_id'] = $user_id;
        $_POST['username'] = $target_record['uname'];
        $_POST['email'] = $target_record['email'];
        $_POST['user_class'] = $target_record['grp'];

        // If the user has the multi_proxy permission, then only the userproxy
        // entries matter and the user.current_proxy value is irrelevant (on this
        // screen). If the user does not have that permission, then the reverse 
        // is true.

        $_POST['proxies'] = array();
        if (!user_class_has_permission($config, $target_record['grp'], "multi_proxy"))
        {
            $_POST['proxies'][(int)$target_record['current_proxy']] = 'on';
        }
        else
        {
            $query = "SELECT proxyid FROM userproxy WHERE userid = :user_id";
            $result = db_query_execute($config, $query, convert_null_array(array(":user_id" => $user_id)));
            while ($record = escape_html(db_fetch_result($config, $result)))
            {
                $_POST['proxies'][(int)$record['proxyid']] = 'on';
            }
        }
    }
}

// The proxies selection may be an array or an integer.

if (!$_POST['proxies'])
{
    $_POST['proxies'] = array();
}
else if (is_array($_POST['proxies']))
{
    // Convert from 'id'=>'on' form to just an array of the IDs.
    $_POST['proxies'] = array_keys($_POST['proxies']);
}
else
{
    $_POST['proxies'] = array(cast_str_to_int($_POST['proxies']));
}

// Process the submit request
$submit_success = false;
while ($_POST['submitted'])
{
    $user_changes = array();

    // Username
    $_POST['username'] = trim($_POST['username']);
    if (!is_user($_POST['username']))
    {
        $processing_errors[] = ___("Error: Invalid username format");
        $curs = "user_edit.username";
        break;
    }
    $user_changes['uname'] = $_POST['username'];

    // Email
    $_POST['email'] = trim($_POST['email']);
    if (strlen($_POST["email"]) && !is_email($_POST["email"]))
    {
        $processing_errors[] = ___("Error: Invalid e-mail format");
        $curs = "user_edit.email";
        break;
    }

    // User class
    $_POST['user_class'] = (int)$_POST['user_class'];
    if (!can_user_manage_class($config, $record_user, $_POST['user_class']))
    {
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
    }
    $user_changes['grp'] = $_POST['user_class'];

    // Current Proxy
    // Regardless of user type, user.current_proxy will be updated to one of the
    // selected proxies. Changes to the userproxy table will be done below the
    // user table update/insert.
    if (count($_POST['proxies']) > 0)
    {
        $user_changes['current_proxy'] = $_POST['proxies'][0];
    }

    // Password
    $password_changes = array();

    if ($_POST['password'] != $_POST['verify_password'])
    {
        $processing_errors[] = ___("Error: Password mismatch");
        $curs = "user_edit.pass";
        break;
    }

    if (($page_mode == "ADD") || strlen($_POST['password']))
    {
        // TODO: safe to trust POST[username] here?
        if (!is_valid_password($config, $_POST['password'], $_POST['username']))
        {
            $processing_errors[] = sprintf(___("Error: Password should be at least %d characters long and not contain your username"), $config['min_pass_length']);
            $curs = "user_edit.pass";
            break;
        }

        $password_salt = bin2hex(secure_rand());

        $password_changes['pass'] = sha1($_POST['password'].$password_salt);
        $password_changes['pass_salt'] = $password_salt;

        if ($user_id != $record_user['id'])
        {
            // Invalidate the session by setting it to a random value.
            // TODO: Set to NULL instead.
            $password_changes['session'] = bin2hex(secure_rand());
        }
    }

    // Email

    $email_changes = array();

    if (($page_mode == "ADD")
         || (($page_mode == "EDIT")
             && ($_POST['email'] != $original_email)))
    {
        // when the admin sets the user email, we also clear any pending email candidate
        // note: email_candidate_date is not cleared as it's still used to throttle
        // number of emails the user can cause to be sent out

        if (strlen($_POST['email']))
        {
            $email_changes['email'] = $_POST['email'];
        }
        else
        {
            $email_changes['email'] = Null;
        }

        if ($page_mode == "EDIT")
        {
            $email_changes['email_candidate'] = Null;
            $email_changes['email_candidate_token'] = Null;
        }
    }

    // Verify the selected proxies

    $param_string = "";
    $param_array = array();
    string_list_to_params(implode(',', $_POST['proxies']), ",", "p_", $param_string, $param_array);

    $query = "SELECT COUNT(*) AS matches FROM proxy WHERE id IN  (".$param_string.")";

    $result = db_query_execute($config, $query, convert_null_array($param_array));
    $proxy_record = db_fetch_result($config, $result);

    if (!$proxy_record 
        || ($proxy_record['matches'] != count($_POST['proxies'])))
    {
        // Bad input. Unlikely to happen without malice.
        include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
        // 403.php exits
        break;
    }

    // NOTE: Because we don't have transactions, there is a race condition between
    // the uname/email conflict check and the insert/update attempt. If a conflict is
    // introduced between the check and the insert/update, a DB error will occur.
    // This is unfortunate but acceptable (at this time).

    if (db_check_if_value_exists(
            $config,
            "user",
            "uname",
            $user_changes['uname'],
            ($page_mode == "EDIT") ? "id != :id" : "",
            ($page_mode == "EDIT") ? array(":id" => $user_id) : array()))
    {
        $processing_errors[] = ___("Username already exists in the system");
        $curs = "user_edit.username";
        break;
    }

    if (strlen($_POST['email'])
        && db_check_if_value_exists(
            $config,
            "user",
            "email",
            $email_changes['email'],
            ($page_mode == "EDIT") ? "id != :id" : "",
            ($page_mode == "EDIT") ? array(":id" => $user_id) : array()))
    {
        $processing_errors[] = ___("E-mail address already exists in the system");
        $curs = "user_edit.email";
        break;
    }

    // Do the insertion/update of the user

    if ($page_mode == "ADD")
    {
        $query = '';
        $query_params = array();
        $fields_sql = '';
        $values_sql = '';
        
        changes_array_to_sql_and_params_for_insert(
                array_merge($user_changes, $email_changes, $password_changes),
                $fields_sql, 
                $values_sql,
                $query_params);

        $query =
            "INSERT INTO user (".$fields_sql.", created) ".
                "VALUES (".$values_sql.", UNIX_TIMESTAMP())";

        db_query_execute($config, $query, convert_null_array($query_params));
        
        $user_id = db_get_last_insert_id($config);

        // Because the insert was successful, the title of the page changes to EDIT.
        $title = ___("Edit an existing user");
    }
    else // EDIT
    {
        $query = '';
        $query_params = array();
        $update_sql = '';
        
        changes_array_to_sql_and_params_for_update(
                array_merge($user_changes, $email_changes, $password_changes),
                $update_sql,
                $query_params);

        $query = "UPDATE user SET ".$update_sql." WHERE id = :userid";

        $query_params[':userid'] = $user_id;

        db_query_execute($config, $query, convert_null_array($query_params));
    }

    //
    // Insert/upate the proxy associations.
    // If the user has the multi_proxy permission, then all existing userproxy
    // entries are *replaced* with the new values.
    // If the user does not have that permission, then the new values are *added*
    // to the userproxy table.
    //

    // In the multi_proxy case, first clear out the userproxy records for the user.
    if (user_class_has_permission($config, $user_changes['grp'], "multi_proxy"))
    {
        $query = "DELETE FROM userproxy WHERE userid = :userid";
        db_query_execute($config, $query, convert_null_array(array(':userid'=>$user_id)));
    }

    // Insert the new entries. Note that we use INSERT IGNORE because some
    // or all of the records may already exist.
    if (count($_POST['proxies']))
    {
        $query_params = array(':userid' => $user_id);
        $values_sql = "";
        $i = 0;
        foreach ($_POST['proxies'] as $proxy_id)
        {
            $i++;
            $param_name = ":proxyid".$i;
            if (strlen($values_sql)) $values_sql .= ",";
            $values_sql .= "(:userid,{$param_name})";
            $query_params[$param_name] = $proxy_id;
        }

        $query = "INSERT IGNORE INTO userproxy (userid, proxyid) VALUES ".$values_sql;
        db_query_execute($config, $query, convert_null_array($query_params));
    }

    if ($page_mode == "ADD")
    {
        $processing_results[] = ___("New account created");
    }
    else // EDIT
    {
        $processing_results[] = ___("Account has been updated");
    }

    $submit_success = True;

    break;
}

// 
// UI code
//

$action = getenv("SCRIPT_NAME");
if ($user_id !== Null) $action .= "?user_id=$user_id";
$action .= $paging_sorting_filtering_params;

include($_SERVER[DOCUMENT_ROOT]."/header.php");
?>
<form name="user_edit" method="post" action="<?=$action?>">
    <input name="csrf_token" value="<?=$record_user['csrf_token']?>" type="hidden"/>

    <input type="hidden" name="submitted" value="1"/>

    <table border="0" cellpadding="2" cellspacing="0">

        <tr>
            <th align="right"><b><?=___("Username")?></b></th>
            <td><input type="text" name="username" maxlength="64" value="<?=escape_html($_POST['username'])?>"></td>
        </tr>
        <tr>
            <th align="right"><b><?=___("E-mail address (optional)")?></b></th>
            <td><input type="text" name="email" maxlength="64" value="<?=escape_html($_POST['email'])?>"></td>
        </tr>
        <tr>
            <th align="right"><b><?=___("Class")?></b></th>
            <td><? echo_user_class_combo($config, "user_class", $_POST["user_class"], 0) ?></td>
        </tr>
        <tr><td>&nbsp;</td></tr>
        <tr>
            <th align="right"><b><?=___("New password")?></b></th>
            <td><input type="password" name="password" maxlength="25"></td>
        </tr>
        <tr>
            <th align="right"><b><?=___("Verify password")?></b></th>
            <td><input type="password" name="verify_password" maxlength="25"></td>
        </tr>
        <tr><td>&nbsp;</td></tr>

        <tr>
            <th align="right"><b><?=___("Proxies")?></b></th>
        </tr>
        <tr><td></td></tr>
        <tr><td></td><td id="proxies_section">
            <?
            $proxy_checkboxes = "";
            $proxy_radios = "";
            $not_applicable = '<table><tr><td>'.___("Not applicable to selected user class").'</td></tr></table>';
            $current_proxy_checked = False;
            foreach ($all_proxies as $proxy)
            {
                // When switching from displaying checkboxes to radio buttons (e.g.,
                // switching from Propagator to User), we want the selected radio
                // button to be the user's current_proxy, if possible (otherwise,
                // just select any valid). (See Bug #605514.)

                $proxy_name = 'proxies['.$proxy['id'].']';
                $checked = (array_search($proxy['id'], $_POST['proxies']) !== False);

                $proxy_checkboxes .= '<table><tr><td>';
                $proxy_radios .= '<table><tr><td>';

                $proxy_checkboxes .= '<input type="checkbox" name="'.$proxy_name.'" id="'.$proxy_name.'" '.($checked?'checked':'').'/>';
                $proxy_radios .= '<input type="radio" name="proxies" value="'.$proxy['id'].'" id="'.$proxy_name.'" '.((!$current_proxy_checked&&$checked)?'checked':'').'/>';
                $current_proxy_checked = $current_proxy_checked || ($checked && ($proxy['id'] == $target_record['current_proxy']));

                $proxy_checkboxes .= '<label for="'.$proxy_name.'">'.$proxy['label'].'</label>';
                $proxy_radios .=  '<label for="'.$proxy_name.'">'.$proxy['label'].'</label>';

                $proxy_checkboxes .= '</td></tr></table>';
                $proxy_radios .= '</td></tr></table>';
            }
            ?>
            <?=$proxy_checkboxes?>
        </td></tr>
        <script type="text/javascript" src="js/ondomload.js"></script>
        <script type="text/javascript">
            var userClassCombo = document.getElementsByName('user_class')[0];
            userClassCombo.onchange = function()
            {
                var userClassCombo = document.getElementsByName('user_class')[0];
                var userClass = userClassCombo.options[userClassCombo.selectedIndex].value;
                var proxies_section = document.getElementById('proxies_section');

                while (proxies_section.hasChildNodes())
                {
                    proxies_section.removeChild(proxies_section.firstChild);
                }

                var newDiv = document.createElement("div");

                <?/* Users with 'multi_proxy' permission get checkboxes (i.e.,
                 * multiple selection) for proxy selection, users with 'other_nodes'
                 * get no proxy (because they have access to all), and all other
                 * users (normal Users and Power Users, currently) get radio buttons
                 * for proxy selection (i.e., single selection). */?>

                var multiProxyUserClasses = [<?=implode(",", get_user_classes_with_permission($config, 'multi_proxy'))?>];
                var multiProxyUser = false;
                for (var i = 0; i < multiProxyUserClasses.length; i++)
                {
                    if (userClass == multiProxyUserClasses[i])
                    {
                        multiProxyUser = true;
                        break;
                    }
                }

                var otherNodesUserClasses = [<?=implode(",", get_user_classes_with_permission($config, 'other_nodes'))?>];
                var otherNodesUser = false;
                for (var i = 0; i < otherNodesUserClasses.length; i++)
                {
                    if (userClass == otherNodesUserClasses[i])
                    {
                        otherNodesUser = true;
                        break;
                    }
                }
                
                if (otherNodesUser)
                {
                    newDiv.innerHTML = '<?=$not_applicable?>';
                }
                else if (multiProxyUser)
                {
                    newDiv.innerHTML = '<?=$proxy_checkboxes?>';
                }
                else
                {
                    newDiv.innerHTML = '<?=$proxy_radios?>';
                }

                proxies_section.appendChild(newDiv);
            };
            run_on_DOM_load(userClassCombo.onchange);
        </script>
    </table>

    <br/>
    <input type="submit" value="<?=___("Submit")?>"/>
    <br/>
    <?if ($submit_success) {?>
        <a href="users.php?user_id=<?=$user_id.$paging_sorting_filtering_params?>"><b><?=___("<-- Return to Users")?></b></a>
    <?} else {?>
        <a href="users.php?<?=$paging_sorting_filtering_params?>"><?=___("Cancel")?></a>
    <?}?>

    <?if ($page_mode == "ADD") {?>
        <br/><br/>
        <div class="dimtext"><?=___("Note that users created here will not be counted in the stats.")?></div>
    <?}?>

</form>

<?
include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>
