<?
/*
    Psiphon Circumvention Platform
    Copyright (C) 2009 Psiphon Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// users.php
//
// This screen is used to view a list of users, edit user details, and create new users.
//

include_once($_SERVER[DOCUMENT_ROOT]."/includes/common_includes.php");
include($_SERVER[DOCUMENT_ROOT]."/check_user.php");
include($_SERVER[DOCUMENT_ROOT]."/includes/lang_set.php");

$user_edit_page = 'user-edit.php';

$column_number_to_field_name = array(1=>'uname', 2=>'email', 3=>'grp', 4=>'current_proxy', 5=>'proxies', 6=>'inv_actv', 7=>'inv_sent');
$default_sort_field = 1;
//
// Business logic
//

// Check user has permission to access this screen

if (!user_has_permission($config, $record_user, "accounts"))
{
    include($_SERVER[DOCUMENT_ROOT]."/http-errors/403.php");
    // 403.php exits
}

// Screen title. Must come before the header.php include.

$title = ___("Users");

$_GET['disable'] = cast_str_to_int($_GET['disable']);
$last_modified_user_id = cast_str_to_int($_GET['user_id']);
$_GET['filterby'] = cast_str_to_int($_GET['filterby']);
$_GET['ord'] = cast_str_to_int($_GET['ord']);
// If 'ord' param is not set or invalid, sort by default column
if (!$_GET["ord"] || (abs($_GET["ord"]) > count($column_number_to_field_name)))
{
    $_GET["ord"] = (int)$default_sort_field;
}
$orderby_sql = " ORDER BY ".$column_number_to_field_name[abs($_GET["ord"])];
if ($_GET["ord"]<0)
{
    $orderby_sql.=" DESC";
}

// We need to build up the where clause that will be used in the main query.
$query_where_sql = get_user_managed_classes_sql_condition($config, $record_user);
$query_where_sql_params = array();

// The "other_nodes" permission gives the user control over all users and proxies.
$query_proxy_params = array();
if (!user_has_permission($config, $record_user, "other_nodes"))
{
    // If the user doesn't have the "other_nodes" permission, they can only see users
    // associated with the proxy they're currently logged in though.
    $query_where_sql .= " AND userproxy.proxyid = :query_where_sql_proxy";
    $query_where_sql_params[':query_where_sql_proxy'] = $record_user['current_proxy'];

    // And the proxy combo will only contain the current proxy.
    $query_proxy = "SELECT id, hostname, ".proxy_label_sql('proxy')." AS proxy_label FROM proxy WHERE id = :query_proxy_id";
    $query_proxy_params[':query_proxy_id'] = $record_user['current_proxy'];
}
else
{
    $query_proxy = "SELECT id, ".proxy_label_sql('proxy')." AS proxy_label FROM proxy";
}

// Get the proxies that the current user can manage.

$proxy_labels = array();
function proxy_to_label(&$value, $key, $proxy_labels) { $value = $proxy_labels[$value]; }

$result = db_query_execute($config, $query_proxy, convert_null_array($query_proxy_params));

while ($record = db_fetch_result($config, $result))
{
    $proxy_labels[$record['id']] = $record['proxy_label'];
}

// Process user disabling request

while ($_GET['disable'])
{
    // The current user cannot disable himself.
    if ($record_user["id"]==$_GET["disable"])
    {
        $processing_errors[] = ___("Operation not permitted.");
        break;
    }

    // Flip the locked flag in the database.
    $query = "UPDATE user SET locked = (NOT locked) WHERE id = :id AND {$query_where_sql}";

    db_query_execute($config, $query, convert_null_array(array(":id" => $_GET['disable']) + $query_where_sql_params));

    $processing_results[] = ___("Account has been updated");

    $last_modified_user_id = $_GET["disable"];
    
    break;
}

// count total number of users  (unfiltered)
$query = "SELECT COUNT(id) AS cnt FROM user WHERE {$query_where_sql}";
$result = db_query_execute($config, $query, convert_null_array($query_where_sql_params));
$record = db_fetch_result($config, $result);
$total_users_count = $record["cnt"];
db_free_result($result);

$filterby_string = '';
if (isset($_GET['filterby'])) {
    $filterby_string = "&amp;filterby=" . $_GET['filterby'];

    // apply user class filter
    $query_where_sql .= " AND user.grp = :filter_user_class";
    $query_where_sql_params[':filter_user_class'] = $_GET['filterby'];
    $selected_filter = $_GET['filterby'];

    // count filtered users without LIMITs
    $query = "SELECT COUNT(id) AS cnt FROM user WHERE {$query_where_sql}";
    $result = db_query_execute($config, $query, convert_null_array($query_where_sql_params));
    $record = db_fetch_result($config, $result);
    $filtered_users_count = $record["cnt"];
    db_free_result($result);
}

// pager (from tickets.php)/////////////////////////////////////////////////////
$query = "SELECT COUNT(*) AS cnt FROM user WHERE {$query_where_sql}";
$result = db_query_execute($config, $query, convert_null_array($query_where_sql_params));
$record = db_fetch_result($config, $result);
$count = $record["cnt"];
db_free_result($result);

$per_page = $config['users_ppage'];
if (($_GET["ofs"]=(int)$_GET["ofs"])<0) $_GET["ofs"]=0;
$last_page=floor(abs($count-0.1)/$per_page);
if ($_GET["ofs"]>$last_page) include($_SERVER[DOCUMENT_ROOT]."/http-errors/404.php");
$ord_string='';
if (isset($_GET["ord"])) {
    $ord_string = "&amp;ord=". $_GET["ord"];
}
$ofs_string = '';
if (isset($_GET["ofs"])) {
    $ofs_string = "&amp;ofs=". $_GET["ofs"];
}

$page_numbers_control='';
for ($page=0; $page<=$last_page; $page++)
{
    if ($page!=$_GET["ofs"])
    {
       $page_numbers_control.="<a href=\"".getenv("SCRIPT_NAME")."?ofs={$page}{$ord_string}{$filterby_string}\">".($page+1)."</a> &nbsp;\n";
    }
    else
    {
       $page_numbers_control.="[ <b>".($page+1)."</b> ] &nbsp;\n";
   }
}

$limit_offset = abs((int)($_GET[ofs] * $per_page));
$limit_count = abs((int)$per_page);

// end of pager ////////////////////////////////////////////////////////////////
$paging_sorting_filtering_params = $ofs_string.$ord_string.$filterby_string;

function print_sort_header($col_number, $label, $filterby_string)
{
    echo "<a href=\"" . getenv("SCRIPT_NAME") . "?ord=";
    if ($_GET["ord"] == $col_number)
        echo "-{$col_number}{$filterby_string}\"><b>" . $label . " /\\</b>";
    elseif ($_GET["ord"] == -$col_number)
        echo "{$col_number}{$filterby_string}\"><b>" . $label . " \\/</b>";
    else
        echo "-{$col_number}{$filterby_string}\">" . $label;
    echo "</a>\n";
}

include($_SERVER[DOCUMENT_ROOT]."/header.php");
?>
<div>
    <form method="GET" action="<?=getenv("SCRIPT_NAME")?>">
    <label><?=___("User Class Filter") ?>: </label>
<?

$user_classes = get_user_class_names($config, $record_user, 1, 1);

echo_combo_from_array(
        'filterby',
        $user_classes,
        Null, // no transform function
        $selected_filter,
        1, ___("No filter"));

?>
<input type="submit" value="<?=___("Apply filter") ?>">
<? if (isset($filtered_users_count)) {
    echo " &nbsp;&nbsp; ".___("Filtered items count").": ".$filtered_users_count." &nbsp;&nbsp; ";
}
echo ___("Total users").": ".$total_users_count;
?>
</form>

</div>
<? echo $page_numbers_control; ?>
<table class="brd">
    <tr>
        <th><? print_sort_header(1, ___("Name"), $filterby_string) ?></th>
        <th><? print_sort_header(2, ___("E-mail"), $filterby_string)?></th>
        <th><? print_sort_header(3, ___("Class"), $filterby_string)?></th>
        <th><? print_sort_header(4, ___("Current Proxy"), $filterby_string)?></th>
        <th><? print_sort_header(5, ___("Proxies"), $filterby_string)?></th>
        <th><? print_sort_header(6, ___("Invites accepted"), $filterby_string)?></th>
        <th><? print_sort_header(7, ___("Invites sent"), $filterby_string)?></th>
        <th><a href="<?=$user_edit_page."?$paging_sorting_filtering_params" ?>"><?=___("Add new user")?></a></th>
    </tr>

<?
$proxy_separator = '##';
$query = "SELECT user.*, ".
                "GROUP_CONCAT(userproxy.proxyid SEPARATOR '".$proxy_separator."') AS proxies ".
         "FROM user LEFT OUTER JOIN userproxy ON user.id = userproxy.userid ".
         "WHERE {$query_where_sql} GROUP BY uname {$orderby_sql} LIMIT {$limit_offset}, {$limit_count}";

$result = db_query_execute($config, $query, convert_null_array($query_where_sql_params));

while ($record = escape_html(db_fetch_result($config, $result)))
{
    // Split the proxies into an array. Then change them to labels.
    $record['proxies'] = explode($proxy_separator, $record['proxies']);
    array_walk($record['proxies'], 'proxy_to_label', $proxy_labels);

    $_p=$_s="";
    if ($record[id]==$record_user[id]) { $_p="{$_p}<i>"; $_s="</i>{$_s}"; }
    if ($record[locked])               { $_p="{$_p}<s>"; $_s="</s>{$_s}"; }
    if ($record[id]==$last_modified_user_id) { $_p="{$_p}<b>"; $_s="</b>{$_s}"; }

    echo "<tr valign=\"top\">\n";
    echo "<td>{$_p}{$record["uname"]}{$_s}</td>\n";
    echo "<td>{$_p}{$record["email"]}{$_s}</td>\n";
    echo "<td>{$_p}".get_user_class_name($record["grp"])."{$_s}</td>\n";
    echo "<td>{$_p}{$proxy_labels[$record['current_proxy']]}{$_s}</td>\n";

    // Display all of the associated proxies.

    if (count($record['proxies']) <= 0)
    {
        echo "<td>{$_p}{$_s}</td>\n";
    }
    else if (count($record['proxies']) == 1)
    {
        echo "<td>{$_p}{$record['proxies'][0]}{$_s}</td>\n";
    }
    else
    {
        $size = min(count($record['proxies']), 3);
        echo "<td><select size=".$size."><option>".
             implode($record['proxies'], "</option><option>").
             "</option></select></td>\n";
    }

    echo "<td>{$_p}{$record['inv_actv']}{$_s}</td>\n";
    echo "<td>{$_p}{$record['inv_sent']}{$_s}</td>\n";

    echo "<td>{$_p}&nbsp;";
    echo "<a href=\"".$user_edit_page."?user_id={$record[id]}{$paging_sorting_filtering_params}\">".___("Edit")."</a> &nbsp; ";
    $t=($record[locked]?___("Enable account"):___("Disable account"));
    if ($record[id]!=$record_user[id])
    {
        // The "fresh" GET parameter ensures that the disable link is always coloured as not-visited.
        echo "<a href=\"".getenv("SCRIPT_NAME")."?disable={$record[id]}{$paging_sorting_filtering_params}&amp;fresh=".bin2hex(microtime())."\" title=\"{$t}\">X</a>";
    }
    else
    {
        echo "X";
    }

    echo "{$_s}</td>\n";

    echo "</tr>\n\n";
}

?>
</table>
<?
echo "<br>\n".$page_numbers_control;
db_free_result($result);

include($_SERVER[DOCUMENT_ROOT]."/footer.php");
?>
